@@ -27,9 +27,8 @@
  *                                                                         *
  ***************************************************************************/
 #include "Convolution.h"
-#include "../ColorBox.h"
-#include "../plot2D/MultiLayer.h"
-#include "../plot2D/PlotCurve.h"
+#include <PlotCurve.h>
+#include <ColorBox.h>
 
 #include <QMessageBox>
 #include <QLocale>
@@ -65,11 +64,8 @@ void Convolution::setDataFromTable(Table *t, const QString& signalColName, const
 		return;
 	}
 
-    if (d_n > 0)
-	{//delete previously allocated memory
-		delete[] d_x;
-		delete[] d_y;
-	}
+    if (d_n > 0)//delete previously allocated memory
+		freeMemory();
 
 	d_n_response = 0;
 	int rows = d_table->numRows();
@@ -137,11 +133,8 @@ void Convolution::addResultCurve()
 
 	d_table->addCol();
 	d_table->addCol();
-#ifdef Q_CC_MSVC
-    QVarLengthArray<double> x_temp(d_n);
-#else
+
     double x_temp[d_n];
-#endif
 	QLocale locale = app->locale();
 	for (int i = 0; i<d_n; i++){
 		double x = i+1;
@@ -162,14 +155,10 @@ void Convolution::addResultCurve()
 
 	if (d_graphics_display){
 		if (!d_output_graph)
-			d_output_graph = createOutputGraph()->activeLayer();
+			createOutputGraph();
 
     	DataCurve *c = new DataCurve(d_table, d_table->colName(cols), d_table->colName(cols2));
-#ifdef Q_CC_MSVC
-		c->setData(x_temp.data(), d_x, d_n);
-#else
 		c->setData(x_temp, d_x, d_n);
-#endif
     	c->setPen(QPen(ColorBox::color(d_curveColorIndex), 1));
 		d_output_graph->insertPlotItem(c, Graph::Line);
 		d_output_graph->updatePlot();
