@@ -27,7 +27,7 @@
  *                                                                         *
  ***************************************************************************/
 #include "Differentiation.h"
-#include "../plot2D/MultiLayer.h"
+#include <MultiLayer.h>
 
 #include <QLocale>
 
@@ -37,6 +37,20 @@ Differentiation::Differentiation(ApplicationWindow *parent, Graph *g)
 	init();
 }
 
+Differentiation::Differentiation(ApplicationWindow *parent, QwtPlotCurve *c)
+: Filter(parent, c)
+{
+	init();
+	setDataFromCurve(c);
+}
+
+Differentiation::Differentiation(ApplicationWindow *parent, QwtPlotCurve *c, double start, double end)
+: Filter(parent, c)
+{
+	init();
+	setDataFromCurve(c, start, end);
+}
+
 Differentiation::Differentiation(ApplicationWindow *parent, Graph *g, const QString& curveTitle)
 : Filter(parent, g)
 {
@@ -62,13 +76,26 @@ void Differentiation::init()
 {
 	setObjectName(tr("Derivative"));
     d_min_points = 4;
+    d_sort_data = false;
 }
 
 void Differentiation::output()
 {
-    double *result = new double[d_n-1];
-	for (int i = 1; i < d_n-1; i++)
-		result[i]=0.5*((d_y[i+1]-d_y[i])/(d_x[i+1]-d_x[i]) + (d_y[i]-d_y[i-1])/(d_x[i]-d_x[i-1]));
+    double *result = new double[d_n - 1];
+	for (int i = 1; i < d_n - 1; i++){
+		double xl = d_x[i - 1];
+		double xc = d_x[i];
+		double xr = d_x[i + 1];
+
+		if (xr != xc && xl != xc)
+			result[i] = 0.5*((d_y[i + 1] - d_y[i])/(xr - xc) + (d_y[i] - d_y[i - 1])/(xc - xl));
+		else if (xr != xc)
+			result[i] = (d_y[i + 1] - d_y[i])/(xr - xc);
+		else if (xl != xc)
+			result[i] = (d_y[i] - d_y[i - 1])/(xc - xl);
+		else
+			result[i] = result[i - 1];
+	}
 
     ApplicationWindow *app = (ApplicationWindow *)parent();
     QLocale locale = app->locale();
@@ -90,8 +117,8 @@ void Differentiation::output()
 
 	if (d_graphics_display){
 		if (!d_output_graph)
-			d_output_graph = createOutputGraph()->activeLayer();
-
+			createOutputGraph();
     	d_output_graph->insertCurve(d_result_table, tableName + "_2", 0);
+		d_output_graph->updatePlot();
 	}
 }
