@@ -27,13 +27,14 @@
  *                                                                         *
  ***************************************************************************/
 #include "Filter.h"
-#include "../ColorBox.h"
-#include "../Table.h"
-#include "../plot2D/FrameWidget.h"
-#include "../plot2D/LegendWidget.h"
-#include "../plot2D/FunctionCurve.h"
-#include "../plot2D/PlotCurve.h"
-#include "../plot2D/MultiLayer.h"
+
+#include <Table.h>
+#include <FrameWidget.h>
+#include <LegendWidget.h>
+#include <FunctionCurve.h>
+#include <PlotCurve.h>
+#include <MultiLayer.h>
+#include <ColorBox.h>
 
 #include <QApplication>
 #include <QMessageBox>
@@ -41,8 +42,16 @@
 
 #include <gsl/gsl_sort.h>
 
+Filter::Filter(ApplicationWindow *parent, QwtPlotCurve *c)
+: QObject(parent)
+{
+	init();
+	if (c)
+		setObjectName(c->title().text());
+}
+
 Filter::Filter( ApplicationWindow *parent, Graph *g, const QString& name)
-: QObject( parent)
+: QObject(parent)
 {
 	init();
 	setObjectName(name);
@@ -51,7 +60,7 @@ Filter::Filter( ApplicationWindow *parent, Graph *g, const QString& name)
 }
 
 Filter::Filter( ApplicationWindow *parent, Table *t, const QString& name)
-: QObject( parent, name)
+: QObject(parent)
 {
 	init();
 	setObjectName(name);
@@ -89,15 +98,16 @@ void Filter::setInterval(double from, double to)
 	setDataFromCurve (d_curve->title().text(), from, to);
 }
 
-void Filter::setDataCurve(int curve, double start, double end)
+void Filter::setDataCurve(QwtPlotCurve *curve, double start, double end)
 {
-	if (d_n > 0){//delete previousely allocated memory
-		if (d_x) delete[] d_x;
-		if (d_y) delete[] d_y;
-	}
+	if (!curve)
+		return;
+
+    if (d_n > 0)//delete previousely allocated memory
+		freeMemory();
 
 	d_init_err = false;
-	d_curve = d_graph->curve(curve);
+	d_curve = curve;
     if (d_sort_data)
         d_n = sortedCurveData(d_curve, start, end, &d_x, &d_y);
     else
@@ -141,6 +151,42 @@ int Filter::curveIndex(const QString& curveTitle, Graph *g)
 	return d_graph->curveIndex(curveTitle);
 }
 
+bool Filter::setDataFromCurve(QwtPlotCurve *c)
+{
+	if (!c || !c->plot()){
+		d_init_err = true;
+		return false;
+	}
+
+	d_graph = qobject_cast<Graph *>(c->plot());
+	if (!d_graph){
+		d_init_err = true;
+		return false;
+	}
+	d_output_graph = d_graph;
+
+  	d_graph->range(c, &d_from, &d_to);
+    setDataCurve(c, d_from, d_to);
+	return true;
+}
+
+bool Filter::setDataFromCurve(QwtPlotCurve *c, double from, double to)
+{
+	if (!c || !c->plot()){
+		d_init_err = true;
+		return false;
+	}
+
+	d_graph = qobject_cast<Graph *>(c->plot());
+	if (!d_graph){
+		d_init_err = true;
+		return false;
+	}
+	d_output_graph = d_graph;
+    setDataCurve(c, from, to);
+	return true;
+}
+
 bool Filter::setDataFromCurve(const QString& curveTitle, Graph *g)
 {
 	int index = curveIndex(curveTitle, g);
@@ -150,7 +196,7 @@ bool Filter::setDataFromCurve(const QString& curveTitle, Graph *g)
 	}
 
   	d_graph->range(curveTitle, &d_from, &d_to);
-    setDataCurve(index, d_from, d_to);
+    setDataCurve(d_graph->curve(index), d_from, d_to);
 	return true;
 }
 
@@ -162,7 +208,7 @@ bool Filter::setDataFromCurve(const QString& curveTitle, double from, double to,
 		return false;
 	}
 
-	setDataCurve(index, from, to);
+	setDataCurve(d_graph->curve(index), from, to);
 	return true;
 }
 
@@ -197,7 +243,7 @@ void Filter::showLegend()
 			return;
 		}
 	}
-	
+
 	LegendWidget *l = d_output_graph->newLegend(legendInfo());
 	l->repaint();
 }
@@ -224,15 +270,9 @@ bool Filter::run()
 
 void Filter::output()
 {
-#ifdef Q_CC_MSVC
-    QVarLengthArray<double> X(d_points), Y(d_points);
-    calculateOutputData(X.data(), Y.data()); //does the data analysis
-	addResultCurve(X.data(), Y.data());
-#else
     double X[d_points], Y[d_points];
     calculateOutputData(X, Y); //does the data analysis
 	addResultCurve(X, Y);
-#endif
 }
 
 int Filter::sortedCurveData(QwtPlotCurve *c, double start, double end, double **x, double **y)
@@ -243,12 +283,23 @@ int Filter::sortedCurveData(QwtPlotCurve *c, double start, double end, double **
 	int i_start = 0, i_end = 0;
 	int n = curveRange(c, start, end, &i_start, &i_end);
 
-    (*x) = new double[n];
-    (*y) = new double[n];
+    (*x) = (double *)malloc(n*sizeof(double));
+	if (!x){
+		memoryErrorMessage();
+		return 0;
+	}
+
+	(*y) = (double *)malloc(n*sizeof(double));
+	if (!y){
+		free(x);
+		memoryErrorMessage();
+		return 0;
+	}
+
     double *xtemp = new double[n];
     double *ytemp = new double[n];
 
-  	int j=0;
+  	int j = 0;
     for (int i = i_start; i <= i_end; i++){
         xtemp[j] = c->x(i);
         ytemp[j++] = c->y(i);
@@ -274,10 +325,20 @@ int Filter::curveData(QwtPlotCurve *c, double start, double end, double **x, dou
    	int i_start = 0, i_end = 0;
 	int n = curveRange(c, start, end, &i_start, &i_end);
 
-    (*x) = new double[n];
-    (*y) = new double[n];
+	(*x) = (double *)malloc(n*sizeof(double));
+	if (!x){
+		memoryErrorMessage();
+		return 0;
+	}
 
-    int j=0;
+	(*y) = (double *)malloc(n*sizeof(double));
+	if (!y){
+		free(x);
+		memoryErrorMessage();
+		return 0;
+	}
+
+    int j = 0;
     for (int i = i_start; i <= i_end; i++){
         (*x)[j] = c->x(i);
         (*y)[j++] = c->y(i);
@@ -351,7 +412,7 @@ QwtPlotCurve* Filter::addResultCurve(double *x, double *y)
     	c->setPen(QPen(ColorBox::color(d_curveColorIndex), 1));
 
 		if (!d_output_graph)
-			d_output_graph = createOutputGraph()->activeLayer();
+			createOutputGraph();
 
 		d_output_graph->insertPlotItem(c, Graph::Line);
     	d_output_graph->updatePlot();
@@ -366,14 +427,15 @@ void Filter::enableGraphicsDisplay(bool on, Graph *g)
 		if (g)
 			d_output_graph = g;
 		else
-			d_output_graph = createOutputGraph()->activeLayer();
+			createOutputGraph();
 	}
 }
 
 MultiLayer * Filter::createOutputGraph()
 {
 	MultiLayer *ml = ((ApplicationWindow *)parent())->newGraph(tr("Plot") + objectName());
-   	d_output_graph = ml->activeLayer();
+	if (ml)
+   		d_output_graph = ml->activeLayer();
 	return ml;
 }
 
@@ -423,10 +485,8 @@ bool Filter::setDataFromTable(Table *t, const QString& xColName, const QString&
         return false;
 	}
 
-	if (d_n > 0){//delete previousely allocated memory
-		delete[] d_x;
-		delete[] d_y;
-	}
+    if (d_n > 0)//delete previousely allocated memory
+		freeMemory();
 
 	d_graph = 0;
 	d_curve = 0;
@@ -439,8 +499,18 @@ bool Filter::setDataFromTable(Table *t, const QString& xColName, const QString&
 	d_from = X[0];
     d_to = X[d_n-1];
 
-	d_x = new double[d_n];
-    d_y = new double[d_n];
+	d_x = (double *)malloc(d_n*sizeof(double));
+	if (!d_x){
+		memoryErrorMessage();
+		return false;
+	};
+
+    d_y = (double *)malloc(d_n*sizeof(double));
+	if (!d_y){
+		memoryErrorMessage();
+		free(d_x);
+		return false;
+	};
 
     for (int i = 0; i < d_n; i++){
         d_x[i] = X[i];
@@ -461,15 +531,27 @@ bool Filter::setDataFromTable(Table *t, const QString& xColName, const QString&
 
 void Filter::memoryErrorMessage()
 {
+	d_init_err = true;
+
     QMessageBox::critical((ApplicationWindow *)parent(),
         tr("QtiPlot") + " - " + tr("Memory Allocation Error"),
         tr("Not enough memory, operation aborted!"));
 }
 
-Filter::~Filter()
+void Filter::freeMemory()
 {
-	if (d_n > 0){//delete the memory allocated for the data
-		if (d_x) delete[] d_x;
-		if (d_y) delete[] d_y;
+	if (d_x){
+		free(d_x);
+		d_x = NULL;
+	}
+	if (d_y) {
+		free(d_y);
+		d_y = NULL;
 	}
 }
+
+Filter::~Filter()
+{
+	if (d_n > 0)//delete the memory allocated for the data
+		freeMemory();
+}
