@@ -31,8 +31,9 @@
 
 #include <QObject>
 
-#include "../ApplicationWindow.h"
+#include <ApplicationWindow.h>
 
+class MultiLayer;
 class QwtPlotCurve;
 class Graph;
 class Table;
@@ -45,12 +46,15 @@ class Filter : public QObject
 	public:
         Filter(ApplicationWindow *parent, Table *t = 0, const QString& name = QString());
 		Filter(ApplicationWindow *parent, Graph *g = 0, const QString& name = QString());
+		Filter(ApplicationWindow *parent, QwtPlotCurve *c);
 		~Filter();
 
 		//! Actually does the job. Should be reimplemented in derived classes.
 		virtual bool run();
 
-        virtual void setDataCurve(int curve, double start, double end);
+        virtual void setDataCurve(QwtPlotCurve *curve, double start, double end);
+        bool setDataFromCurve(QwtPlotCurve *c);
+        bool setDataFromCurve(QwtPlotCurve *c, double from, double to);
 		bool setDataFromCurve(const QString& curveTitle, Graph *g = 0);
 		bool setDataFromCurve(const QString& curveTitle, double from, double to, Graph *g = 0);
 
@@ -91,14 +95,18 @@ class Filter : public QObject
 		double* y(){return d_y;};
 		//! Returns a pointer to the table created to display the results
         Table *resultTable(){return d_result_table;};
+		//! Returns a pointer to the graph where the result curve should be displayed
+		Graph *outputGraph(){return d_output_graph;};
 
         bool error(){return d_init_err;};
 
 		virtual void enableGraphicsDisplay(bool on = true, Graph *g = 0);
-		
+
 	protected:
         void init();
         void memoryErrorMessage();
+	    //! Frees the memory allocated for the X and Y data sets
+        virtual void freeMemory();
 
         //! Sets x and y to the curve points between start and end. Memory will be allocated with new double[].
   	    //! Returns the number of points within range == size of x and y arrays.
@@ -182,7 +190,7 @@ class Filter : public QObject
 		//! Specifies if the filter should display a result curve
 		bool d_graphics_display;
 
-		QString d_y_col_name;	
+		QString d_y_col_name;
 };
 
 #endif
