@@ -29,12 +29,12 @@
 #include "Fit.h"
 #include "FitModelHandler.h"
 #include "fit_gsl.h"
-#include "../ColorBox.h"
-#include "../Table.h"
-#include "../matrix/Matrix.h"
-#include "../plot2D/QwtErrorPlotCurve.h"
-#include "../plot2D/FunctionCurve.h"
-#include "../plot2D/MultiLayer.h"
+#include <Table.h>
+#include <Matrix.h>
+#include <QwtErrorPlotCurve.h>
+#include <FunctionCurve.h>
+#include <MultiLayer.h>
+#include <ColorBox.h>
 
 #include <gsl/gsl_statistics.h>
 #include <gsl/gsl_blas.h>
@@ -46,6 +46,12 @@
 #include <QLocale>
 #include <QTextStream>
 
+Fit::Fit( ApplicationWindow *parent, QwtPlotCurve *c)
+: Filter( parent, c)
+{
+	init();
+}
+
 Fit::Fit( ApplicationWindow *parent, Graph *g, const QString& name)
 : Filter( parent, g, name)
 {
@@ -64,6 +70,7 @@ void Fit::init()
 	d_n = 0;
 	d_x = 0;
 	d_y = 0;
+	d_w = 0;
 	d_curveColorIndex = 1;
 	d_solver = ScaledLevenbergMarquardt;
 	d_tolerance = 1e-4;
@@ -85,7 +92,7 @@ void Fit::init()
 	d_rss = 0.0;
 	d_adjusted_r_square = GSL_NAN;
 	d_scale_errors = false;
-	d_sort_data = true;
+	d_sort_data = false;
 	d_prec = (((ApplicationWindow *)parent())->fit_output_precision);
 	d_param_table = 0;
 	d_cov_matrix = 0;
@@ -104,8 +111,10 @@ gsl_multifit_fdfsolver * Fit::fitGSL(gsl_multifit_function_fdf f, int &iteration
 	else
 		T = gsl_multifit_fdfsolver_lmsder;
 
+	gsl_set_error_handler_off();
+
 	gsl_multifit_fdfsolver *s = gsl_multifit_fdfsolver_alloc (T, d_n, d_p);
-	gsl_multifit_fdfsolver_set (s, &f, d_param_init);
+	status = gsl_multifit_fdfsolver_set (s, &f, d_param_init);
 
 	size_t iter = 0;
 	bool inRange = true;
@@ -118,14 +127,20 @@ gsl_multifit_fdfsolver * Fit::fitGSL(gsl_multifit_function_fdf f, int &iteration
 		}
 	}
 
+	if (status){
+	    gsl_multifit_covar (s->J, 0.0, covar);
+	    iterations = 0;
+	    return s;
+	}
+
 	do{
 		iter++;
 		status = gsl_multifit_fdfsolver_iterate (s);
 		if (status)
 			break;
-		
+
 		for (int i=0; i<d_p; i++){
-			double p = gsl_vector_get(s->x, i);			
+			double p = gsl_vector_get(s->x, i);
 			if (p < d_param_range_left[i] || p > d_param_range_right[i]){
 				inRange = false;
 				break;
@@ -134,13 +149,14 @@ gsl_multifit_fdfsolver * Fit::fitGSL(gsl_multifit_function_fdf f, int &iteration
 		if (!inRange)
 			break;
 
-		for (int i=0; i<d_p; i++)
+		for (int i = 0; i < d_p; i++)
 			d_results[i] = gsl_vector_get(s->x, i);
 
 		status = gsl_multifit_test_delta (s->dx, s->x, d_tolerance, d_tolerance);
 	} while (inRange && status == GSL_CONTINUE && (int)iter < d_max_iterations);
 
 	gsl_multifit_covar (s->J, 0.0, covar);
+
 	iterations = iter;
 	return s;
 }
@@ -156,8 +172,11 @@ gsl_multimin_fminimizer * Fit::fitSimplex(gsl_multimin_function f, int &iteratio
 	//set all step sizes to 1 can be increased to converge faster
 	gsl_vector_set_all (ss, 10.0);
 
+	gsl_set_error_handler_off();
+
 	gsl_multimin_fminimizer *s_min = gsl_multimin_fminimizer_alloc (T, f.n);
 	status = gsl_multimin_fminimizer_set (s_min, &f, d_param_init, ss);
+
 	double size;
 	size_t iter = 0;
 	bool inRange = true;
@@ -170,6 +189,12 @@ gsl_multimin_fminimizer * Fit::fitSimplex(gsl_multimin_function f, int &iteratio
 		}
 	}
 
+	if (status) {
+	    iterations = 0;
+	    gsl_vector_free(ss);
+	    return s_min;
+	}
+
 	do{
 		iter++;
 		status = gsl_multimin_fminimizer_iterate (s_min);
@@ -202,26 +227,35 @@ gsl_multimin_fminimizer * Fit::fitSimplex(gsl_multimin_function f, int &iteratio
 
 bool Fit::setDataFromTable(Table *t, const QString& xColName, const QString& yColName, int from, int to)
 {
-	if (d_n > 0)//delete old weighting data set
-		delete[] d_w;
-
     if (Filter::setDataFromTable(t, xColName, yColName, from, to)){
-    	d_w = new double[d_n];
-    	for (int i=0; i<d_n; i++)//initialize the weighting data to 1.0
+    	if (d_w)
+			free(d_w);
+
+    	d_w = (double *)malloc(d_n*sizeof(double));
+        if (!d_w){
+            memoryErrorMessage();
+            return false;
+        }
+
+    	for (int i = 0; i < d_n; i++)//initialize the weighting data to 1.0
        		d_w[i] = 1.0;
 		return true;
 	} else
 		return false;
 }
 
-void Fit::setDataCurve(int curve, double start, double end)
+void Fit::setDataCurve(QwtPlotCurve *curve, double start, double end)
 {
-    if (d_n > 0)
-		delete[] d_w;
-
     Filter::setDataCurve(curve, start, end);
 
-    d_w = new double[d_n];
+    if (!d_w){
+        d_w = (double *)malloc(d_n*sizeof(double));
+        if (!d_w){
+            memoryErrorMessage();
+            return;
+        }
+	}
+
     if (d_graph && d_curve && ((PlotCurve *)d_curve)->type() != Graph::Function)
     {
         QList<DataCurve *> lst = ((DataCurve *)d_curve)->errorBarsList();
@@ -229,8 +263,10 @@ void Fit::setDataCurve(int curve, double start, double end)
             QwtErrorPlotCurve *er = (QwtErrorPlotCurve *)c;
             if (!er->xErrors()){
                 d_weighting = Instrumental;
-                for (int i=0; i<d_n; i++)
-                    d_w[i] = er->errorValue(i); //d_w are equal to the error bar values
+                for (int i=0; i<d_n; i++){
+					double e = er->errorValue(i);
+					d_w[i] = 1.0/(e*e);
+				}
                 weighting_dataset = er->title().text();
                 return;
             }
@@ -290,6 +326,9 @@ QString Fit::logFitInfo(int iterations, int status)
 		case Dataset:
 			info += tr("Arbitrary Dataset") + ": " + weighting_dataset;
 			break;
+		case Direct:
+			info += tr("Direct Weighting using Dataset") + ": " + weighting_dataset;
+			break;
 	}
 	info +="\n";
 
@@ -383,7 +422,7 @@ QString Fit::legendInfo()
 			info += locale.toString(sqrt(chi_2_dof*gsl_matrix_get(covar, i, i)), 'e', d_prec);
 		else
 			info += locale.toString(sqrt(gsl_matrix_get(covar, i, i)), 'e', d_prec);
-		
+
 		if (i < d_p - 1)
 			info += "\n";
 	}
@@ -392,6 +431,15 @@ QString Fit::legendInfo()
 
 bool Fit::setWeightingData(WeightingMethod w, const QString& colName)
 {
+	if (d_w)
+		free(d_w);
+
+	d_w = (double *)malloc(d_n*sizeof(double));
+	if (!d_w){
+		memoryErrorMessage();
+		return false;
+	}
+
 	switch (w)
 	{
 		case NoWeighting:
@@ -428,8 +476,10 @@ bool Fit::setWeightingData(WeightingMethod w, const QString& colName)
 					return false;
 				}
 				if (er){
-					for (int j=0; j<d_n; j++)
-						d_w[j] = er->errorValue(j); //d_w are equal to the error bar values
+					for (int j=0; j<d_n; j++){
+						double e = er->errorValue(j);
+						d_w[j] = 1.0/(e*e);
+					}
 				}
 			}
 			break;
@@ -441,10 +491,35 @@ bool Fit::setWeightingData(WeightingMethod w, const QString& colName)
 					weighting_dataset = d_y_col_name;
 
 				for (int i=0; i<d_n; i++)
-					d_w[i] = sqrt(d_y[i]);
+					d_w[i] = 1.0/d_y[i];
 			}
 			break;
 		case Dataset:
+			{//d_w are equal to the values of the arbitrary dataset
+				if (colName.isEmpty())
+					return false;
+
+				Table* t = ((ApplicationWindow *)parent())->table(colName);
+				if (!t)
+					return false;
+
+				if (t->numRows() < d_n){
+  	            	QMessageBox::critical((ApplicationWindow *)parent(), tr("QtiPlot - Error"),
+  	                tr("The column %1 has less points than the fitted data set. Please choose another column!").arg(colName));
+  	                return false;
+  	            }
+
+				weighting_dataset = colName;
+
+				int col = t->colIndex(colName);
+				for (int i = 0; i < d_n; i++){
+					double e = t->cell(i, col);
+					d_w[i] = 1.0/(e*e);
+				}
+			}
+			break;
+
+		case Direct:
 			{//d_w are equal to the values of the arbitrary dataset
 				if (colName.isEmpty())
 					return false;
@@ -462,7 +537,7 @@ bool Fit::setWeightingData(WeightingMethod w, const QString& colName)
 				weighting_dataset = colName;
 
 				int col = t->colIndex(colName);
-				for (int i=0; i<d_n; i++)
+				for (int i = 0; i < d_n; i++)
 					d_w[i] = t->cell(i, col);
 			}
 			break;
@@ -507,11 +582,17 @@ void Fit::writeParametersToTable(Table *t, bool append)
 	ApplicationWindow *app = (ApplicationWindow *)parent();
 	QLocale locale = app->locale();
 
+	double chi_2_dof = chi_2/(d_n - d_p);
+
 	for (int i=0; i<d_p; i++){
 		int j = rows + i;
 		t->setText(j, 0, d_param_names[i]);
 		t->setText(j, 1, locale.toString(d_results[i], 'g', d_prec));
-		t->setText(j, 2, locale.toString(sqrt(gsl_matrix_get(covar, i, i)), 'g', d_prec));
+		if (d_scale_errors)
+		    t->setText(j, 2, locale.toString(sqrt(chi_2_dof*gsl_matrix_get(covar,i,i)), 'g', d_prec));
+		else
+		    t->setText(j, 2, locale.toString(sqrt(gsl_matrix_get(covar, i, i)), 'g', d_prec));
+
 	}
 
 	for (int i=0; i<3; i++)
@@ -540,7 +621,7 @@ double *Fit::errors()
 	if (!d_errors) {
 		d_errors = new double[d_p];
 		double chi_2_dof = chi_2/(d_n - d_p);
-		for (int i=0; i<d_p; i++){
+                for (int i = 0; i < d_p; i++){
 			if (d_scale_errors)
 				d_errors[i] = sqrt(chi_2_dof*gsl_matrix_get(covar,i,i));
 			else
@@ -589,7 +670,7 @@ QwtPlotCurve* Fit::showResiduals()
 	app->hideWindow(outputTable);
 
 	if (!d_output_graph)
-		d_output_graph = createOutputGraph()->activeLayer();
+		createOutputGraph();
 
 	QString tableName = outputTable->objectName();
 	DataCurve *c = new DataCurve(outputTable, tableName + "_1", tableName + "_residue");
@@ -612,15 +693,38 @@ void Fit::showConfidenceLimits(double confidenceLevel)
 		return;
 	}
 
-	double *lcl = new double[d_n];
-	if (!lcl)
+	int points = d_n;
+	double *X = NULL;
+	if (d_gen_function){
+		X = (double *)malloc(d_points*sizeof(double));
+		if (!X){
+			QMessageBox::critical((ApplicationWindow *)parent(), tr("QtiPlot - Memory Allocation Error"),
+			tr("Not enough memory!"));
+			return;
+		}
+		points = d_points;
+		double X0 = d_from;
+		double step = fabs(d_from - d_to)/(points - 1);
+		for (int i = 0; i < points; i++)
+			X[i] = X0 + i*step;
+	} else
+		X = d_x;
+
+	double *lcl = (double *)malloc(d_points*sizeof(double));
+	if (!lcl){
+		QMessageBox::critical((ApplicationWindow *)parent(), tr("QtiPlot - Memory Allocation Error"),
+		tr("Not enough memory!"));
 		return;
-	double *ucl = new double[d_n];
-	if (!ucl)
+	}
+	double *ucl = (double *)malloc(d_points*sizeof(double));
+	if (!ucl){
+		QMessageBox::critical((ApplicationWindow *)parent(), tr("QtiPlot - Memory Allocation Error"),
+		tr("Not enough memory!"));
 		return;
+	}
 
 	ApplicationWindow *app = (ApplicationWindow *)parent();
-	Table *outputTable = app->newTable(d_n, 3, app->generateUniqueName(tr("FitStats"), true), tr("Confidence Limits of %1").arg(d_explanation));
+	Table *outputTable = app->newTable(points, 3, app->generateUniqueName(tr("FitStats"), true), tr("Confidence Limits of %1").arg(d_explanation));
 	if (!outputTable)
 		return;
 
@@ -637,11 +741,10 @@ void Fit::showConfidenceLimits(double confidenceLevel)
 		double dx = d_x[i] - x_mean;
 		sxx += dx*dx;
 	}
-
 	double mse = d_rss/double(d_n - d_p);
 
-	for (int i = 0; i < d_n; i++){
-		double x = d_x[i];
+	for (int i = 0; i < points; i++){
+		double x = X[i];
 		double dx = x - x_mean;
 		double aux = t*sqrt(mse*(1.0/(double)d_n + dx*dx/sxx));
 
@@ -654,27 +757,30 @@ void Fit::showConfidenceLimits(double confidenceLevel)
 		outputTable->setCell(i, 2, upLimit);
 		ucl[i] = upLimit;
 	}
+
 	for (int i = 0; i < outputTable->numCols(); i++)
 		outputTable->table()->adjustColumn(i);
 	app->hideWindow(outputTable);
 
 	if (!d_output_graph)
-		d_output_graph = createOutputGraph()->activeLayer();
+		createOutputGraph();
 
 	QString tableName = outputTable->objectName();
 	DataCurve *c = new DataCurve(outputTable, tableName + "_1", tableName + "_LCL");
-	c->setData(d_x, lcl, d_n);
+	c->setData(X, lcl, points);
     c->setPen(QPen(ColorBox::color(d_curveColorIndex + 2), 1));
 	d_output_graph->insertPlotItem(c, Graph::Line);
 
 	c = new DataCurve(outputTable, tableName + "_1", tableName + "_UCL");
-	c->setData(d_x, ucl, d_n);
+	c->setData(X, ucl, points);
     c->setPen(QPen(ColorBox::color(d_curveColorIndex + 2), 1));
 	d_output_graph->insertPlotItem(c, Graph::Line);
 
     d_output_graph->updatePlot();
-	delete [] lcl;
-	delete [] ucl;
+	free (lcl);
+	free (ucl);
+	if (d_gen_function)
+		free (X);
 }
 
 double Fit::lcl(int parIndex, double confidenceLevel)
@@ -706,15 +812,38 @@ void Fit::showPredictionLimits(double confidenceLevel)
 		return;
 	}
 
-	double *lcl = new double[d_n];
-	if (!lcl)
+	int points = d_n;
+	double *X = NULL;
+	if (d_gen_function){
+		X = (double *)malloc(d_points*sizeof(double));
+		if (!X){
+			QMessageBox::critical((ApplicationWindow *)parent(), tr("QtiPlot - Memory Allocation Error"),
+			tr("Not enough memory!"));
+			return;
+		}
+		points = d_points;
+		double X0 = d_from;
+		double step = fabs(d_to - d_from)/(points - 1);
+		for (int i = 0; i < points; i++)
+			X[i] = X0 + i*step;
+	} else
+		X = d_x;
+
+	double *lcl = (double *)malloc(d_points*sizeof(double));
+	if (!lcl){
+		QMessageBox::critical((ApplicationWindow *)parent(), tr("QtiPlot - Memory Allocation Error"),
+		tr("Not enough memory!"));
 		return;
-	double *ucl = new double[d_n];
-	if (!ucl)
+	}
+	double *ucl = (double *)malloc(d_points*sizeof(double));
+	if (!ucl){
+		QMessageBox::critical((ApplicationWindow *)parent(), tr("QtiPlot - Memory Allocation Error"),
+		tr("Not enough memory!"));
 		return;
+	}
 
 	ApplicationWindow *app = (ApplicationWindow *)parent();
-	Table *outputTable = app->newTable(d_n, 3, app->generateUniqueName(tr("FitStats"), true), tr("Prediction Limits of %1").arg(d_explanation));
+	Table *outputTable = app->newTable(points, 3, app->generateUniqueName(tr("FitStats"), true), tr("Prediction Limits of %1").arg(d_explanation));
 	if (!outputTable)
 		return;
 
@@ -726,15 +855,14 @@ void Fit::showPredictionLimits(double confidenceLevel)
 
 	double t = gsl_cdf_tdist_Pinv(1 - 0.5*(1 - confidenceLevel), d_n - d_p);
 	double x_mean = gsl_stats_mean(d_x, 1, d_n);
-	double sxx = 0.0;//gsl_stats_tss (d_x, 1, d_n);
+	double sxx = 0.0;
 	for (int i = 0; i < d_n; i++){
 		double dx = d_x[i] - x_mean;
 		sxx += dx*dx;
 	}
-
 	double mse = d_rss/double(d_n - d_p);
-	for (int i = 0; i < d_n; i++){
-		double x = d_x[i];
+	for (int i = 0; i < points; i++){
+		double x = X[i];
 		double dx = x - x_mean;
 		double aux = t*sqrt(mse*(1 + 1.0/(double)d_n + dx*dx/sxx));
 
@@ -752,22 +880,24 @@ void Fit::showPredictionLimits(double confidenceLevel)
 	app->hideWindow(outputTable);
 
 	if (!d_output_graph)
-		d_output_graph = createOutputGraph()->activeLayer();
+		createOutputGraph();
 
 	QString tableName = outputTable->objectName();
 	DataCurve *c = new DataCurve(outputTable, tableName + "_1", tableName + "_LPL");
-	c->setData(d_x, lcl, d_n);
+	c->setData(X, lcl, points);
     c->setPen(QPen(ColorBox::color(d_curveColorIndex + 3), 1));
 	d_output_graph->insertPlotItem(c, Graph::Line);
 
 	c = new DataCurve(outputTable, tableName + "_1", tableName + "_UPL");
-	c->setData(d_x, ucl, d_n);
+	c->setData(X, ucl, points);
     c->setPen(QPen(ColorBox::color(d_curveColorIndex + 3), 1));
 	d_output_graph->insertPlotItem(c, Graph::Line);
 
     d_output_graph->updatePlot();
-	delete [] lcl;
-	delete [] ucl;
+	free (lcl);
+	free (ucl);
+	if (d_gen_function)
+		free (X);
 }
 
 void Fit::fit()
@@ -798,8 +928,10 @@ void Fit::fit()
 
 	QApplication::setOverrideCursor(Qt::WaitCursor);
 
+	removeDataSingularities();
+
 	struct FitData d_data = {d_n, d_p, d_x, d_y, d_w, this};
-	
+
 	int status, iterations = d_max_iterations;
 	if(d_solver == NelderMeadSimplex){
 		gsl_multimin_function f;
@@ -808,14 +940,16 @@ void Fit::fit()
 		f.params = &d_data;
 		gsl_multimin_fminimizer *s_min = fitSimplex(f, iterations, status);
 
-		// allocate memory and calculate covariance matrix based on residuals
-		gsl_matrix *J = gsl_matrix_alloc(d_n, d_p);
-		d_df(s_min->x,(void*)f.params, J);
-		gsl_multifit_covar (J, 0.0, covar);
-		chi_2 = s_min->fval;
+		if (!status) {
+		     // allocate memory and calculate covariance matrix based on residuals
+		     gsl_matrix *J = gsl_matrix_alloc(d_n, d_p);
+		     d_df(s_min->x,(void*)f.params, J);
+		     gsl_multifit_covar (J, 0.0, covar);
+		     chi_2 = s_min->fval;
 
-		// free previousely allocated memory
-		gsl_matrix_free (J);
+		     // free previousely allocated memory
+		     gsl_matrix_free (J);
+		}
 		gsl_multimin_fminimizer_free (s_min);
 	} else {
 		gsl_multifit_function_fdf f;
@@ -833,7 +967,7 @@ void Fit::fit()
 	}
 
 	generateFitCurve();
-	
+
 	ApplicationWindow *app = (ApplicationWindow *)parent();
 	if (app->writeFitResultsToLog)
 		app->updateLog(logFitInfo(iterations, status));
@@ -846,38 +980,54 @@ void Fit::generateFitCurve()
 	if (!d_gen_function)
 		d_points = d_n;
 
-	double *X = new double[d_points];
-	double *Y = new double[d_points];
+	double *X = NULL, *Y = NULL;
+	if (d_graphics_display && !d_gen_function){
+		X = (double *)malloc(d_points*sizeof(double));
+		if (!X){
+			QMessageBox::critical((ApplicationWindow *)parent(), tr("QtiPlot - Memory Allocation Error"),
+			tr("Could not allocate enough memory for the fit curves!"));
+			return;
+		}
+		Y = (double *)malloc(d_points*sizeof(double));
+		if (!Y){
+			QMessageBox::critical((ApplicationWindow *)parent(), tr("QtiPlot  - Memory Allocation Error"),
+			tr("Could not allocate enough memory for the fit curves!"));
+			free(X);
+			return;
+		}
+		calculateFitCurveData(X, Y);
+	}
 
-	calculateFitCurveData(X, Y);
     customizeFitResults();
 
 	if (d_graphics_display){
 		if (!d_output_graph)
-			d_output_graph = createOutputGraph()->activeLayer();
+			createOutputGraph();
 
 		if (d_gen_function){
-			insertFitFunctionCurve(QString(objectName()) + tr("Fit"), X, Y);
+			insertFitFunctionCurve(QString(objectName()) + tr("Fit"));
 			d_output_graph->replot();
-		} else
+		} else {
         	d_output_graph->addFitCurve(addResultCurve(X, Y));
+			free(X);
+			free(Y);
+		}
 	}
-	delete [] X;
-	delete [] Y;
 }
 
-FunctionCurve * Fit::insertFitFunctionCurve(const QString& name, double *x, double *y, int penWidth)
+FunctionCurve * Fit::insertFitFunctionCurve(const QString& name, int penWidth, bool updateData)
 {
 	QString title = d_output_graph->generateFunctionName(name);
 	FunctionCurve *c = new FunctionCurve(FunctionCurve::Normal, title);
 	c->setPen(QPen(ColorBox::color(d_curveColorIndex), penWidth));
-	c->setData(x, y, d_points);
-	c->setRange(d_x[0], d_x[d_n-1]);
+	c->setRange(d_from, d_to);
 	c->setFormula(d_formula);
-	
+
 	for (int j=0; j<d_p; j++)
-		c->setConstant(d_param_names[j], d_results[j]);	
-	
+		c->setConstant(d_param_names[j], d_results[j]);
+
+	if (updateData)
+		c->loadData(d_points);
 	d_output_graph->insertPlotItem(c, Graph::Line);
 	d_output_graph->addFitCurve(c);
 	return c;
@@ -946,7 +1096,7 @@ void Fit::setParameterRange(int parIndex, double left, double right)
 {
 	if (!d_param_range_left || !d_param_range_right || parIndex < 0 || parIndex >= d_p)
 		return;
-	
+
 	d_param_range_left[parIndex] = left;
 	d_param_range_right[parIndex] = right;
 }
@@ -957,7 +1107,13 @@ void Fit::initWorkspace(int par)
 	d_param_init = gsl_vector_alloc(par);
 	gsl_vector_set_all (d_param_init, 1.0);
 
+	//gsl_set_error_handler_off();
 	covar = gsl_matrix_alloc (par, par);
+	/*if (!covar){
+		memoryErrorMessage();
+		return;
+	}*/
+
 	d_results = new double[par];
 	d_param_range_left = new double[par];
 	d_param_range_right = new double[par];
@@ -973,27 +1129,27 @@ void Fit::freeWorkspace()
 		gsl_vector_free(d_param_init);
 		d_param_init = NULL;
 	}
-	
+
 	if (covar){
 		gsl_matrix_free (covar);
 		covar = NULL;
 	}
-	
+
 	if (d_results){
 		delete[] d_results;
 		d_results = NULL;
 	}
-	
+
 	if (d_errors){
 		delete[] d_errors;
 		d_errors = NULL;
 	}
-		
+
 	if (d_param_range_left){
 		delete[] d_param_range_left;
 		d_param_range_left = NULL;
 	}
-	
+
 	if (d_param_range_right){
 		delete[] d_param_range_right;
 		d_param_range_right = NULL;
@@ -1002,25 +1158,24 @@ void Fit::freeWorkspace()
 
 void Fit::freeMemory()
 {
-	if (!d_p)
-		return;
+	Filter::freeMemory();
+	if (d_w){
+		free(d_w);
+		d_w = NULL;
+	}
 
-	if (d_x){
-		delete[] d_x;
-		d_x = NULL;
-	}
-	if (d_y) {
-		delete[] d_y;
-		d_y = NULL;
-	}
+    if (d_residuals) {
+		delete[] d_residuals;
+		d_residuals = NULL;
+    }
 }
 
 Fit::~Fit()
 {
+	freeMemory();
+
 	if (!d_p)
 		return;
 
 	freeWorkspace();
-	if (d_residuals) 
-		delete[] d_residuals;
 }
