@@ -31,7 +31,7 @@
 
 #include <QObject>
 
-#include "../ApplicationWindow.h"
+#include <ApplicationWindow.h>
 #include "Filter.h"
 
 #include <gsl/gsl_multifit_nlin.h>
@@ -53,9 +53,10 @@ class Fit : public Filter
 		typedef int (*fit_function_fdf)(const gsl_vector *, void *, gsl_vector *, gsl_matrix *);
 
 		enum Algorithm{ScaledLevenbergMarquardt, UnscaledLevenbergMarquardt, NelderMeadSimplex};
-		enum WeightingMethod{NoWeighting, Instrumental, Statistical, Dataset};
+		enum WeightingMethod{NoWeighting, Instrumental, Statistical, Dataset, Direct};
         enum FitType{BuiltIn = 0, Plugin = 1, User = 2};
 
+		Fit(ApplicationWindow *parent, QwtPlotCurve *c);
 		Fit(ApplicationWindow *parent, Graph *g = 0, const QString& name = QString());
 		Fit(ApplicationWindow *parent, Table *t, const QString& name = QString());
 		~Fit();
@@ -67,16 +68,16 @@ class Fit : public Filter
 		//! Sets the data set to be used for weighting
 		bool setWeightingData(WeightingMethod w, const QString& colName = QString::null);
 
-		void setDataCurve(int curve, double start, double end);
+		void setDataCurve(QwtPlotCurve *curve, double start, double end);
 		bool setDataFromTable(Table *t, const QString& xColName, const QString& yColName, int from = 1, int to = -1);
 
 		QString resultFormula(){return d_result_formula;};
 		QString formula(){return d_formula;};
-		virtual void setFormula(const QString&){};
+		virtual bool setFormula(const QString&, bool = true){return true;};
 
 		int numParameters(){return d_p;};
 		QStringList parameterNames(){return d_param_names;};
-		virtual void setParametersList(const QStringList&){};
+		virtual bool setParametersList(const QStringList&){return true;};
         void setParameterExplanations(const QStringList& lst){d_param_explain = lst;};
 
         double initialGuess(int parIndex){return gsl_vector_get(d_param_init, parIndex);};
@@ -145,9 +146,6 @@ class Fit : public Filter
         QString fileName(){return d_file_name;};
 		void setFileName(const QString& fn){d_file_name = fn;};
 
-        //! Frees the memory allocated for the X and Y data sets
-        void freeMemory();
-
         //! Calculates the data for the output fit curve
         virtual double eval(double *, double){return 0.0;};
 
@@ -163,13 +161,18 @@ class Fit : public Filter
 		//! Customs and stores the fit results according to the derived class specifications. Used by exponential fits.
 		virtual void customizeFitResults(){};
 
+		//! Removes any data singularities before fitting
+		virtual void removeDataSingularities(){};
+
 	protected:
 		//! Allocates the memory for the fit workspace
 		void initWorkspace(int par);
 		//! Frees the memory allocated for the fit workspace
 		void freeWorkspace();
+        //! Frees all the arrays with size d_n. Used when changing the source data sets.
+        virtual void freeMemory();
 		//! Adds the result curve as a FunctionCurve to the plot, if d_gen_function = true
-		virtual FunctionCurve * insertFitFunctionCurve(const QString& name, double *x, double *y, int penWidth = 1);
+		virtual FunctionCurve * insertFitFunctionCurve(const QString& name, int penWidth = 1, bool updateData = true);
 
 		//! Adds the result curve to the plot
 		virtual void generateFitCurve();
