@@ -28,10 +28,10 @@
  ***************************************************************************/
 #include "Integration.h"
 #include "nrutil.h"
-#include "../MyParser.h"
-#include "../plot2D/MultiLayer.h"
-#include "../plot2D/FunctionCurve.h"
-#include "../plot2D/LegendWidget.h"
+#include <MyParser.h>
+#include <MultiLayer.h>
+#include <FunctionCurve.h>
+#include <LegendWidget.h>
 
 #include <QMessageBox>
 #include <QDateTime>
@@ -75,6 +75,20 @@ Integration::Integration(ApplicationWindow *parent, Graph *g)
 	init();
 }
 
+Integration::Integration(ApplicationWindow *parent, QwtPlotCurve *c)
+: Filter(parent, c)
+{
+	init();
+	setDataFromCurve(c);
+}
+
+Integration::Integration(ApplicationWindow *parent, QwtPlotCurve *c, double start, double end)
+: Filter(parent, c)
+{
+	init();
+	setDataFromCurve(c, start, end);
+}
+
 Integration::Integration(ApplicationWindow *parent, Graph *g, const QString& curveTitle)
 : Filter(parent, g)
 {
