@@ -2,8 +2,8 @@
     File                 : fitclasses.cpp
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2006 by Ion Vasilief, Tilman Hoener zu Siederdissen
-    Email (use @ for *)  : ion_vasilief*yahoo.fr, thzs*gmx.net
+    Copyright            : (C) 2006 by Ion Vasilief
+    Email (use @ for *)  : ion_vasilief*yahoo.fr
     Description          : MultiPeakFit module with Lorentz and Gauss peak shapes
 
  ***************************************************************************/
@@ -28,10 +28,10 @@
  ***************************************************************************/
 #include "MultiPeakFit.h"
 #include "fit_gsl.h"
-#include "../ColorBox.h"
 #include "../plot2D/PlotCurve.h"
 #include "../plot2D/FunctionCurve.h"
 #include "../plot2D/MultiLayer.h"
+#include <ColorBox.h>
 
 #include <gsl/gsl_statistics.h>
 
@@ -41,14 +41,20 @@
 MultiPeakFit::MultiPeakFit(ApplicationWindow *parent, Graph *g, PeakProfile profile, int peaks)
 : Fit(parent, g), d_profile(profile)
 {
-init(peaks);
+	init(peaks);
+}
+
+MultiPeakFit::MultiPeakFit(ApplicationWindow *parent, QwtPlotCurve *c, PeakProfile profile, int peaks)
+: Fit(parent, c), d_profile(profile)
+{
+	init(peaks);
 }
 
 MultiPeakFit::MultiPeakFit(ApplicationWindow *parent, Table *t, const QString&, const QString&,
 		  				  int, int, PeakProfile profile, int peaks)
 : Fit(parent, t), d_profile(profile)
 {
-init(peaks);
+	init(peaks);
 }
 
 void MultiPeakFit::init(int peaks)
@@ -67,9 +73,9 @@ void MultiPeakFit::init(int peaks)
 		d_fsimplex = lorentz_multi_peak_d;
 	}
 
-	d_param_init = NULL;
+	/*d_param_init = NULL;
 	covar = NULL;
-	d_results = NULL;
+	d_results = NULL;*/
 
 	setNumPeaks(peaks);
 
@@ -101,7 +107,7 @@ QStringList MultiPeakFit::generateParameterList(int peaks)
 	if (peaks == 1)
 		return QStringList() << "A" << "xc" << "w" << "y0";
 
-	QStringList lst;
+	QStringList lst = QStringList();
 	for (int i = 0; i<peaks; i++){
 		QString index = QString::number(i+1);
 		lst << "A" + index;
@@ -178,20 +184,11 @@ void MultiPeakFit::guessInitialValues()
 
 	double min_out = d_y[imin];
 	double max_out = d_y[imax];
-
-#ifdef Q_CC_MSVC
-    QVarLengthArray<double> temp(d_n);
-#else
     double temp[d_n];
-#endif
 	for (int i = 0; i < d_n; i++)
 		temp[i] = fabs(d_y[i]);
-#ifdef Q_CC_MSVC
-	size_t imax_temp = gsl_stats_max_index(temp.data(), 1, d_n);
-#else
-	size_t imax_temp = gsl_stats_max_index(temp, 1, d_n);
-#endif
 
+	size_t imax_temp = gsl_stats_max_index(temp, 1, d_n);
 	double offset, area;
 	if (imax_temp == imax)
 		offset = min_out;
@@ -215,12 +212,12 @@ void MultiPeakFit::guessInitialValues()
 void MultiPeakFit::customizeFitResults()
 {
 	for (int j=0; j<d_peaks; j++){
-	    d_results[3*j] = fabs(d_results[3*j]);
-        d_results[3*j + 2] = fabs(d_results[3*j + 2]);
+            d_results[3*j] = fabs(d_results[3*j]);
+            d_results[3*j + 2] = fabs(d_results[3*j + 2]);
 	}
 }
 
-void MultiPeakFit::insertPeakFunctionCurve(double *x, double *y, int peak)
+void MultiPeakFit::insertPeakFunctionCurve(int peak)
 {
 	QStringList curves = d_output_graph->curveNamesList();
 	int index = 0;
@@ -232,18 +229,15 @@ void MultiPeakFit::insertPeakFunctionCurve(double *x, double *y, int peak)
 
 	FunctionCurve *c = new FunctionCurve(FunctionCurve::Normal, title);
 	c->setPen(QPen(ColorBox::color(d_peaks_color), 1));
-	c->setData(x, y, d_points);
-	c->setRange(d_x[0], d_x[d_n-1]);
-
-	QString formula = "y0+"+peakFormula(peak + 1, d_profile);
-	QString parameter = QString::number(d_results[d_p-1], 'e', d_prec);
-	formula.replace(d_param_names[d_p-1], parameter);
+	c->setRange(d_from, d_to);
+	c->setFormula("y0 + " + peakFormula(peak + 1, d_profile));
+	c->setConstant(d_param_names[d_p - 1], d_results[d_p - 1]);//y0 - offset
 	for (int j=0; j<3; j++){
 		int p = 3*peak + j;
-		parameter = QString::number(d_results[p], 'e', d_prec);
-		formula.replace(d_param_names[p], parameter);
+		c->setConstant(d_param_names[p], d_results[p]);
 	}
-	c->setFormula(formula.replace("--", "+").replace("-+", "-").replace("+-", "-"));
+	c->loadData(d_points);
+
 	d_output_graph->insertPlotItem(c, Graph::Line);
 	d_output_graph->addFitCurve(c);
 }
@@ -254,70 +248,43 @@ void MultiPeakFit::generateFitCurve()
 	if (!d_gen_function)
 		d_points = d_n;
 
-	gsl_matrix * m = gsl_matrix_alloc (d_points, d_peaks);
-	if (!m){
-		QMessageBox::warning(app, tr("QtiPlot - Fit Error"), tr("Could not allocate enough memory for the fit curves!"));
-		return;
-	}
-
-#ifdef Q_CC_MSVC
-	QVarLengthArray<double> X(d_points), Y(d_points);
-#else
-	double X[d_points], Y[d_points];
-#endif
 	int i, j;
 	int peaks_aux = d_peaks;
 	if (d_peaks == 1)
 		peaks_aux--;
 
 	if (d_gen_function){
-		double step = (d_x[d_n-1] - d_x[0])/(d_points-1);
-		for (i = 0; i<d_points; i++){
-		    double x = d_x[0] + i*step;
-			X[i] = x;
-			double yi = 0;
-			for (j=0; j<d_peaks; j++){
-                double y = evalPeak(d_results, x, j);
-				gsl_matrix_set(m, i, j, y + d_results[d_p - 1]);
-				yi += y;
-			}
-            Y[i] = yi + d_results[d_p - 1];//add offset
-		}
-
         customizeFitResults();
 
 		if (d_graphics_display){
 			if (!d_output_graph)
-				d_output_graph = createOutputGraph()->activeLayer();
+				createOutputGraph();
 
 			if (d_peaks > 1)
-#ifdef Q_CC_MSVC
-				insertFitFunctionCurve(QString(objectName()) + tr("Fit"), X.data(), Y.data(), 2);
-#else
-				insertFitFunctionCurve(QString(objectName()) + tr("Fit"), X, Y, 2);
-#endif
+				insertFitFunctionCurve(QString(objectName()) + tr("Fit"), 2);
 			else
-#ifdef Q_CC_MSVC
-				insertFitFunctionCurve(QString(objectName()) + tr("Fit"), X.data(), Y.data());
-#else
-				insertFitFunctionCurve(QString(objectName()) + tr("Fit"), X, Y);
-#endif
+				insertFitFunctionCurve(QString(objectName()) + tr("Fit"));
 
 			if (generate_peak_curves){
-				for (i=0; i<peaks_aux; i++){//add the peak curves
-					for (j=0; j<d_points; j++)
-						Y[j] = gsl_matrix_get (m, j, i);
-
-#ifdef Q_CC_MSVC
-				insertPeakFunctionCurve(X.data(), Y.data(), i);
-#else
-				insertPeakFunctionCurve(X, Y, i);
-#endif
-				}
+				for (i = 0; i < peaks_aux; i++)//add the peak curves
+					insertPeakFunctionCurve(i);
 			}
 			d_output_graph->replot();
 		}
 	} else {
+		gsl_matrix * m = gsl_matrix_alloc (d_points, d_peaks);
+		if (!m){
+			QMessageBox::warning(app, tr("QtiPlot - Fit Error"),
+			tr("Could not allocate enough memory for the fit curves!"));
+			return;
+		}
+
+		#ifdef Q_CC_MSVC
+			QVarLengthArray<double> X(d_points), Y(d_points);
+		#else
+			double X[d_points], Y[d_points];
+		#endif
+
 		QString tableName = app->generateUniqueName(tr("Fit"));
 		QString dataSet;
 		if (d_curve)
@@ -362,7 +329,7 @@ void MultiPeakFit::generateFitCurve()
 
 		if (d_graphics_display){
 			if (!d_output_graph)
-				d_output_graph = createOutputGraph()->activeLayer();
+				createOutputGraph();
 
 			label = tableName + "_2";
 			DataCurve *c = new DataCurve(d_result_table, tableName + "_1", label);
@@ -397,8 +364,8 @@ void MultiPeakFit::generateFitCurve()
 			}
 			d_output_graph->replot();
 		}
+		gsl_matrix_free(m);
 	}
-	gsl_matrix_free(m);
 }
 
 double MultiPeakFit::eval(double *par, double x)
@@ -454,6 +421,20 @@ QString MultiPeakFit::logFitInfo(int iterations, int status)
  *
  *****************************************************************************/
 
+LorentzFit::LorentzFit(ApplicationWindow *parent, QwtPlotCurve *c)
+: MultiPeakFit(parent, c, MultiPeakFit::Lorentz, 1)
+{
+	init();
+	setDataFromCurve(c);
+}
+
+LorentzFit::LorentzFit(ApplicationWindow *parent, QwtPlotCurve *c, double start, double end)
+: MultiPeakFit(parent, c, MultiPeakFit::Lorentz, 1)
+{
+	init();
+	setDataFromCurve(c, start, end);
+}
+
 LorentzFit::LorentzFit(ApplicationWindow *parent, Graph *g)
 : MultiPeakFit(parent, g, MultiPeakFit::Lorentz, 1)
 {
@@ -494,6 +475,20 @@ void LorentzFit::init()
  *
  *****************************************************************************/
 
+GaussFit::GaussFit(ApplicationWindow *parent, QwtPlotCurve *c)
+: MultiPeakFit(parent, c, MultiPeakFit::Gauss, 1)
+{
+	init();
+	setDataFromCurve(c);
+}
+
+GaussFit::GaussFit(ApplicationWindow *parent, QwtPlotCurve *c, double start, double end)
+: MultiPeakFit(parent, c, MultiPeakFit::Gauss, 1)
+{
+	init();
+	setDataFromCurve(c, start, end);
+}
+
 GaussFit::GaussFit(ApplicationWindow *parent, Graph *g)
 : MultiPeakFit(parent, g, MultiPeakFit::Gauss, 1)
 {
@@ -534,6 +529,20 @@ void GaussFit::init()
  *
  *****************************************************************************/
 
+GaussAmpFit::GaussAmpFit(ApplicationWindow *parent, QwtPlotCurve *c)
+: Fit(parent, c)
+{
+	init();
+	setDataFromCurve(c);
+}
+
+GaussAmpFit::GaussAmpFit(ApplicationWindow *parent, QwtPlotCurve *c, double start, double end)
+: Fit(parent, c)
+{
+	init();
+	setDataFromCurve(c, start, end);
+}
+
 GaussAmpFit::GaussAmpFit(ApplicationWindow *parent, Graph *g)
 : Fit(parent, g)
 {
