@@ -28,8 +28,8 @@
  ***************************************************************************/
 #include "NonLinearFit.h"
 #include "fit_gsl.h"
-#include "../MyParser.h"
-#include "../plot2D/FunctionCurve.h"
+#include <MyParser.h>
+#include <FunctionCurve.h>
 
 #include <QApplication>
 #include <QMessageBox>
@@ -41,6 +41,20 @@ NonLinearFit::NonLinearFit(ApplicationWindow *parent, Graph *g)
 	init();
 }
 
+NonLinearFit::NonLinearFit(ApplicationWindow *parent, QwtPlotCurve *c)
+: Fit(parent, c)
+{
+	init();
+	setDataFromCurve(c);
+}
+
+NonLinearFit::NonLinearFit(ApplicationWindow *parent, QwtPlotCurve *c, double start, double end)
+: Fit(parent, c)
+{
+	init();
+	setDataFromCurve(c, start, end);
+}
+
 NonLinearFit::NonLinearFit(ApplicationWindow *parent, Graph *g, const QString& curveTitle)
 : Fit(parent, g)
 {
@@ -75,27 +89,28 @@ void NonLinearFit::init()
     d_fit_type = User;
 }
 
-void NonLinearFit::setFormula(const QString& s)
+bool NonLinearFit::setFormula(const QString& s, bool guess)
 {
 	if (s.isEmpty()){
 		QMessageBox::critical((ApplicationWindow *)parent(),  tr("QtiPlot - Input function error"),
 				tr("Please enter a valid non-empty expression! Operation aborted!"));
 		d_init_err = true;
-		return;
+		return false;
 	}
 
+	if (d_formula == s)
+		return true;
+
+	if (guess)
+		setParametersList(guessParameters(s));
 	if (!d_p){
 		QMessageBox::critical((ApplicationWindow *)parent(), tr("QtiPlot - Fit Error"),
 				tr("There are no parameters specified for this fit operation. Please define a list of parameters first!"));
 		d_init_err = true;
-		return;
+		return false;
 	}
 
-	if (d_formula == s)
-		return;
-
-	try
-	{
+	try {
 		double *param = new double[d_p];
 		MyParser parser;
 		double xvar;
@@ -104,35 +119,37 @@ void NonLinearFit::setFormula(const QString& s)
 			param[k] = gsl_vector_get(d_param_init, k);
 			parser.DefineVar(d_param_names[k].ascii(), &param[k]);
 		}
-		
+
 		QMapIterator<QString, double> i(d_constants);
  		while (i.hasNext()) {
      		i.next();
 			parser.DefineConst(i.key().ascii(), i.value());
  		}
-	
+
 		parser.SetExpr(s.ascii());
 		parser.Eval() ;
 		delete[] param;
-	}
-	catch(mu::ParserError &e)
-	{
+	} catch(mu::ParserError &e){
 		QMessageBox::critical((ApplicationWindow *)parent(),  tr("QtiPlot - Input function error"), QString::fromStdString(e.GetMsg()));
 		d_init_err = true;
-		return;
+		return false;
 	}
 
 	d_init_err = false;
 	d_formula = s;
+	return true;
 }
 
-void NonLinearFit::setParametersList(const QStringList& lst)
+bool NonLinearFit::setParametersList(const QStringList& lst)
 {
 	if (lst.count() < 1){
 		QMessageBox::critical((ApplicationWindow *)parent(), tr("QtiPlot - Fit Error"),
 				tr("You must provide a list containing at least one parameter for this type of fit. Operation aborted!"));
 		d_init_err = true;
-		return;
+		if (d_p > 0)
+			freeWorkspace();
+		d_p = 0;
+		return false;
 	}
 
 	d_init_err = false;
@@ -146,6 +163,8 @@ void NonLinearFit::setParametersList(const QStringList& lst)
 	d_param_explain.clear();
 	for (int i=0; i<d_p; i++)
 		d_param_explain << "";
+
+	return true;
 }
 
 void NonLinearFit::calculateFitCurveData(double *X, double *Y)
@@ -159,24 +178,24 @@ void NonLinearFit::calculateFitCurveData(double *X, double *Y)
      	i.next();
 		parser.DefineConst(i.key().ascii(), i.value());
  	}
-	
+
 	double x;
 	parser.DefineVar("x", &x);
 	parser.SetExpr(d_formula.ascii());
 
 	if (d_gen_function){
 		double X0 = d_x[0];
-		double step = (d_x[d_n-1]-X0)/(d_points-1);
+		double step = (d_x[d_n - 1] - X0)/(d_points - 1);
 		for (int i=0; i<d_points; i++){
 		    x = X0 + i*step;
 			X[i] = x;
-			Y[i] = parser.Eval();
+			Y[i] = parser.EvalRemoveSingularity(&x, false);
 		}
 	} else {
 		for (int i=0; i<d_points; i++) {
 		    x = d_x[i];
 			X[i] = x;
-			Y[i] = parser.Eval();
+			Y[i] = parser.EvalRemoveSingularity(&x, false);
 		}
 	}
 }
@@ -186,21 +205,21 @@ double NonLinearFit::eval(double *par, double x)
 	MyParser parser;
 	for (int i=0; i<d_p; i++)
 		parser.DefineVar(d_param_names[i].ascii(), &par[i]);
-	
+
 	QMapIterator<QString, double> i(d_constants);
  	while (i.hasNext()) {
      	i.next();
 		parser.DefineConst(i.key().ascii(), i.value());
  	}
-		
+
 	parser.DefineVar("x", &x);
 	parser.SetExpr(d_formula.ascii());
-    return parser.Eval();
+    return parser.EvalRemoveSingularity(&x, false);
 }
-		
+
 void NonLinearFit::setConstant(const QString& parName, double val)
-{	
-	d_constants.insert(parName, val);			
+{
+	d_constants.insert(parName, val);
 }
 
 QString NonLinearFit::logFitInfo(int iterations, int status)
@@ -208,10 +227,10 @@ QString NonLinearFit::logFitInfo(int iterations, int status)
 	QString info = Fit::logFitInfo(iterations, status);
 	if (d_constants.isEmpty())
 		return info;
-	
+
 	ApplicationWindow *app = (ApplicationWindow *)parent();
 	QLocale locale = app->locale();
-	
+
 	QMapIterator<QString, double> i(d_constants);
  	while (i.hasNext()) {
      	i.next();
@@ -227,10 +246,10 @@ QString NonLinearFit::legendInfo()
 	QString info = Fit::legendInfo();
 	if (d_constants.isEmpty())
 		return info;
-	
+
 	ApplicationWindow *app = (ApplicationWindow *)parent();
 	QLocale locale = app->locale();
-	
+
 	QMapIterator<QString, double> i(d_constants);
  	while (i.hasNext()) {
      	i.next();
@@ -239,9 +258,9 @@ QString NonLinearFit::legendInfo()
 	return info;
 }
 
-FunctionCurve * NonLinearFit::insertFitFunctionCurve(const QString& name, double *x, double *y, int penWidth)
+FunctionCurve * NonLinearFit::insertFitFunctionCurve(const QString& name, int penWidth, bool updateData)
 {
-	FunctionCurve *c = Fit::insertFitFunctionCurve(name, x, y, penWidth);
+	FunctionCurve *c = Fit::insertFitFunctionCurve(name, penWidth, false);
 	if (c){
 		QMapIterator<QString, double> i(d_constants);
  		while (i.hasNext()) {
@@ -249,5 +268,118 @@ FunctionCurve * NonLinearFit::insertFitFunctionCurve(const QString& name, double
 			c->setConstant(i.key(), i.value());
  		}
 	}
+	if (updateData)
+		c->loadData(d_points);
 	return c;
 }
+
+QStringList NonLinearFit::guessParameters(const QString& s, bool *error, string *errMsg)
+{
+	QString text = s;
+	text.remove(QRegExp("\\s")).remove(".");
+
+	QStringList parList;
+	try {
+		MyParser parser;
+		ParserTokenReader reader(&parser);
+
+		const char *formula = text.toAscii().data();
+		int length = text.toAscii().length();
+		reader.SetFormula (formula);
+		reader.IgnoreUndefVar(true);
+		int pos = 0;
+		while(pos < length){
+			ParserToken<value_type, string_type> token = reader.ReadNextToken();
+			QString str = QString(token.GetAsString().c_str());
+			if (token.GetCode () == cmVAR && str.contains(QRegExp("\\D"))
+				&& str != "x" && !parList.contains(str))
+				parList << str;
+			pos = reader.GetPos();
+		}
+		parList.sort();
+	} catch(mu::ParserError &e) {
+		if (error){
+			*error = true;
+			*errMsg = e.GetMsg();
+		}
+		return parList;
+	}
+	if (error)
+		*error = false;
+	return parList;
+}
+
+void NonLinearFit::removeDataSingularities()
+{
+	MyParser parser;
+	for (int i = 0; i < d_p; i++){
+		double param = gsl_vector_get(d_param_init, i);
+		parser.DefineVar(d_param_names[i].ascii(), &param);
+	}
+
+	QMapIterator<QString, double> it(d_constants);
+ 	while (it.hasNext()) {
+     	it.next();
+		parser.DefineConst(it.key().ascii(), it.value());
+ 	}
+
+	double xvar;
+	parser.DefineVar("x", &xvar);
+	parser.SetExpr(d_formula.ascii());
+
+    for (int i = 0; i < d_n; i++){
+    	xvar = d_x[i];
+    	try {
+			parser.EvalRemoveSingularity(&xvar);
+    	} catch(MyParser::Pole){
+			QApplication::restoreOverrideCursor();
+			QMessageBox::critical(0, QObject::tr("QtiPlot"),
+			QObject::tr("Ignored data point at x = %1.").arg(xvar));
+
+    		removePole(i);
+    	}
+    }
+}
+
+void NonLinearFit::removePole(int pole)
+{
+	int n = d_n - 1;
+	double *aux_x = (double *)malloc(n*sizeof(double));
+	if (!aux_x)
+		return;
+
+	double *aux_y = (double *)malloc(n*sizeof(double));
+	if (!aux_y){
+		free (aux_x);
+		return;
+	}
+
+	double *aux_w = (double *)malloc(n*sizeof(double));
+	if (!aux_w){
+		free (aux_x);
+		free (aux_y);
+		return;
+	}
+
+	for (int i = 0; i < pole; i++){
+		aux_x [i] = d_x[i];
+		aux_y [i] = d_y[i];
+		aux_w [i] = d_w[i];
+	}
+
+	for (int i = pole + 1; i < d_n; i++){
+		int j = i - 1;
+		aux_x [j] = d_x[i];
+		aux_y [j] = d_y[i];
+		aux_w [j] = d_w[i];
+	}
+
+	free (d_x);
+	free (d_y);
+	free (d_w);
+
+	d_x = aux_x;
+	d_y = aux_y;
+	d_w = aux_w;
+	d_n = n;
+}
