@@ -27,18 +27,20 @@
  *                                                                         *
  ***************************************************************************/
 #include "FitDialog.h"
-#include "../Fit.h"
-#include "../MultiPeakFit.h"
-#include "../ExponentialFit.h"
-#include "../PolynomialFit.h"
-#include "../PluginFit.h"
-#include "../NonLinearFit.h"
-#include "../SigmoidalFit.h"
-#include "../LogisticFit.h"
-#include "../../MyParser.h"
-#include "../../ApplicationWindow.h"
-#include "../../ColorBox.h"
-#include "../../DoubleSpinBox.h"
+#include <Fit.h>
+#include <MultiPeakFit.h>
+#include <ExponentialFit.h>
+#include <PolynomialFit.h>
+#include <PluginFit.h>
+#include <NonLinearFit.h>
+#include <SigmoidalFit.h>
+#include <LogisticFit.h>
+#include <MyParser.h>
+#include <ApplicationWindow.h>
+#include <ColorBox.h>
+#include <DoubleSpinBox.h>
+#include <FunctionCurve.h>
+#include <pixmaps.h>
 
 #include <QListWidget>
 #include <QTableWidget>
@@ -60,7 +62,9 @@
 #include <QLocale>
 #include <stdio.h>
 
-#include <qwt_plot_curve.h>
+#include <muParserToken.h>
+
+using namespace std;
 
 /* XPM */
 static const char * param_range_btn_xpm[] = {
@@ -91,10 +95,11 @@ FitDialog::FitDialog(Graph *g, QWidget* parent, Qt::WFlags fl )
     setObjectName("FitDialog");
 	setWindowTitle(tr("QtiPlot - Fit Wizard"));
 	setSizeGripEnabled(true);
+	setAttribute(Qt::WA_DeleteOnClose);
 
 	d_param_table = 0;
 	d_current_fit = 0;
-	d_preview_curve = 0;
+	d_preview_curve = NULL;
 
 	tw = new QStackedWidget();
 
@@ -111,32 +116,75 @@ FitDialog::FitDialog(Graph *g, QWidget* parent, Qt::WFlags fl )
 
 	setGraph(g);
 	initBuiltInFunctions();
-    loadPlugins();
-    loadUserFunctions();
+
+	ApplicationWindow *app = (ApplicationWindow *)parent;
+	QString pluginsPath = app->fitPluginsPath;
+	QFileInfo fi(pluginsPath);
+	if (pluginsPath.isEmpty() || !fi.isDir() || !fi.isWritable())
+		choosePluginsFolder();
+	else
+    	loadPlugins();
+
+	QString modelsPath = app->fitModelsPath;
+	QFileInfo fim(modelsPath);
+	if (modelsPath.isEmpty() || !fim.isDir() || !fim.isWritable())
+		chooseFitModelsFolder();
+	else
+    	loadUserFunctions();
+
+	if (d_user_functions.size())
+		categoryBox->setCurrentRow(0);
+	else if (d_plugins.size())
+		categoryBox->setCurrentRow(3);
+	else
+		categoryBox->setCurrentRow(1);
 }
 
 void FitDialog::initFitPage()
 {
     ApplicationWindow *app = (ApplicationWindow *)parent();
 
+	QColor blue = QColor(85, 85, 255);
+	QPalette palette = this->palette();
+    palette.setColor(QPalette::WindowText, blue);
+
+	QFont font = this->font();
+	font.setBold(true);
+	font.setPointSize(font.pointSize() + 1);
+
+	QLabel *l = new QLabel(tr("Function") + ":");
+	l->setFont(font);
+	l->setPalette(palette);
+
+	QHBoxLayout *hbox0 = new QHBoxLayout();
+    hbox0->addWidget(l);
+
+    lblFunction = new QLabel();
+    lblFunction->setFont(font);
+    lblFunction->setFrameStyle(QFrame::Box | QFrame::Plain);
+	lblFunction->setLineWidth(2);
+	lblFunction->setMargin(10);
+	lblFunction->setPalette(palette);
+
+    hbox0->addWidget(lblFunction);
+
     QGridLayout *gl1 = new QGridLayout();
     gl1->addWidget(new QLabel(tr("Curve")), 0, 0);
 	boxCurve = new QComboBox();
     gl1->addWidget(boxCurve, 0, 1);
-    gl1->addWidget(new QLabel(tr("Function")), 1, 0);
-	lblFunction = new QLabel();
-    gl1->addWidget(lblFunction, 1, 1);
+
 	boxFunction = new QTextEdit();
 	boxFunction->setReadOnly(true);
-    QPalette palette = boxFunction->palette();
-    palette.setColor(QPalette::Active, QPalette::Base, Qt::lightGray);
+    palette = boxFunction->palette();
+    palette.setColor(QPalette::Base, Qt::lightGray);
     boxFunction->setPalette(palette);
+    boxFunction->setFont(font);
 	boxFunction->setMaximumHeight(50);
-    gl1->addWidget(boxFunction, 2, 1);
 
     QVBoxLayout *vb = new QVBoxLayout();
     vb->addWidget(new QLabel(tr("Initial guesses")));
     btnSaveGuesses = new QPushButton(tr( "&Save" ));
+    btnSaveGuesses->setPixmap(QPixmap(filesave_xpm));
     connect(btnSaveGuesses, SIGNAL(clicked()), this, SLOT(saveInitialGuesses()));
     vb->addWidget(btnSaveGuesses);
     btnParamRange = new QPushButton();
@@ -144,21 +192,21 @@ void FitDialog::initFitPage()
     btnParamRange->setCheckable(true);
     connect(btnParamRange, SIGNAL(toggled(bool)), this, SLOT(showParameterRange(bool)));
     vb->addWidget(btnParamRange);
-    previewBox = new QCheckBox(tr("&Preview"));
-    connect(previewBox, SIGNAL(stateChanged(int)), this, SLOT(updatePreview()));
-    vb->addWidget(previewBox);
+
     vb->addStretch();
 	gl1->addLayout(vb, 3, 0);
 
 	boxParams = new QTableWidget();
-    boxParams->setColumnCount(5);
+    boxParams->setColumnCount(6);
     boxParams->horizontalHeader()->setClickable(false);
     boxParams->horizontalHeader()->setResizeMode (0, QHeaderView::ResizeToContents);
 	boxParams->horizontalHeader()->setResizeMode (1, QHeaderView::Stretch);
     boxParams->horizontalHeader()->setResizeMode (2, QHeaderView::Stretch);
     boxParams->horizontalHeader()->setResizeMode (3, QHeaderView::Stretch);
 	boxParams->horizontalHeader()->setResizeMode (4, QHeaderView::ResizeToContents);
-    QStringList header = QStringList() << tr("Parameter") << tr("From") << tr("Value") << tr("To") << tr("Constant");
+	boxParams->horizontalHeader()->setResizeMode (5, QHeaderView::Stretch);
+
+    QStringList header = QStringList() << tr("Parameter") << tr("From") << tr("Value") << tr("To") << tr("Constant") << tr("Error");
     boxParams->setHorizontalHeaderLabels(header);
     boxParams->verticalHeader()->setResizeMode(QHeaderView::ResizeToContents);
     boxParams->verticalHeader()->hide();
@@ -232,6 +280,7 @@ void FitDialog::initFitPage()
 	boxWeighting->addItem(tr("Instrumental"));
 	boxWeighting->addItem(tr("Statistical"));
 	boxWeighting->addItem(tr("Arbitrary Dataset"));
+	boxWeighting->addItem(tr("Direct Weighting"));
     hbox2->addWidget(boxWeighting);
     QGroupBox *gb4 = new QGroupBox();
     gb4->setLayout(hbox2);
@@ -244,24 +293,111 @@ void FitDialog::initFitPage()
     hbox2->addWidget(colNamesBox);
 
     QHBoxLayout *hbox3 = new QHBoxLayout();
-	buttonEdit = new QPushButton(tr( "<< &Edit function" ) );
-    hbox3->addWidget(buttonEdit);
+	buttonCancel1 = new QPushButton(tr( "&Close" ));
+	buttonCancel1->setPixmap(QPixmap(delete_xpm));
+    hbox3->addWidget(buttonCancel1);
+	hbox3->addStretch();
+	previewBox = new QCheckBox(tr("&Preview"));
+	previewBox->setChecked(false);
+    connect(previewBox, SIGNAL(clicked(bool)), this, SLOT(showPreview(bool)));
+    hbox3->addWidget(previewBox);
+
 	btnDeleteFitCurves = new QPushButton(tr( "&Delete Fit Curves" ));
     hbox3->addWidget(btnDeleteFitCurves);
-	buttonOk = new QPushButton(tr( "&Fit" ) );
+    btnDeleteFitCurves->setPixmap(QPixmap(close_xpm));
+    buttonOk = new QPushButton(tr( "&Fit" ) );
+    buttonOk->setPixmap(QPixmap(play_xpm));
 	buttonOk->setDefault( true );
     hbox3->addWidget(buttonOk);
-	buttonCancel1 = new QPushButton(tr( "&Close" ));
-    hbox3->addWidget(buttonCancel1);
-	buttonAdvanced = new QPushButton(tr( "Custom &Output >>" ));
-    hbox3->addWidget(buttonAdvanced);
-    hbox3->addStretch();
+
+	QFrame *frame1 = new QFrame();
+	frame1->setFrameStyle(QFrame::Box | QFrame::Plain);
+	frame1->setLineWidth(2);
+
+	palette = frame1->palette();
+	palette.setColor(QPalette::WindowText, blue);
+	frame1->setPalette(palette);
+
+	QLabel *selectFunctionLbl = new QLabel(tr("Select Function"));
+	selectFunctionLbl->setFont(font);
+
+	int space = 10;
+	QHBoxLayout *frameLayout = new QHBoxLayout(frame1);
+	frameLayout->addSpacing (space);
+	frameLayout->addWidget(selectFunctionLbl);
+	frameLayout->addSpacing (space);
+
+	buttonEdit = new QPushButton();
+	buttonEdit->setIcon(QPixmap(prev_xpm));
+	buttonEdit->setToolTip(tr("Select Function"));
+
+	QFrame *frame2 = new QFrame();
+	frame2->setFrameShape(QFrame::Box);
+	frame2->setLineWidth(2);
+
+	palette = this->palette();
+	palette.setColor(QPalette::WindowText, Qt::white);
+	palette.setColor(QPalette::Window, blue);
+	frame2->setAutoFillBackground(true);
+	frame2->setPalette(palette);
+
+	QLabel *fittingSessionLbl = new QLabel(tr("Fitting Session"));
+	fittingSessionLbl->setFont(font);
+
+	frameLayout = new QHBoxLayout(frame2);
+	frameLayout->addSpacing (space);
+	frameLayout->addWidget(fittingSessionLbl);
+	frameLayout->addSpacing (space);
+
+	buttonAdvanced = new QPushButton();
+	buttonAdvanced->setIcon(QPixmap(next_xpm));
+	buttonAdvanced->setToolTip(tr("Custom Output"));
+
+	QFrame *frame3 = new QFrame();
+	frame3->setFrameShape(QFrame::Box);
+	frame3->setLineWidth(2);
+
+	palette = this->palette();
+	palette.setColor(QPalette::WindowText, blue);
+	frame3->setPalette(palette);
+
+	QLabel *advancedLbl = new QLabel(tr("Custom Output"));
+	advancedLbl->setFont(font);
+
+	frameLayout = new QHBoxLayout(frame3);
+	frameLayout->addSpacing (space);
+	frameLayout->addWidget(advancedLbl);
+	frameLayout->addSpacing (space);
+
+	QHBoxLayout *hbox4 = new QHBoxLayout();
+	hbox4->addStretch();
+	hbox4->addWidget(frame1);
+	hbox4->addWidget(buttonEdit);
+	hbox4->addWidget(frame2);
+	hbox4->addWidget(buttonAdvanced);
+	hbox4->addWidget(frame3);
+	hbox4->addStretch();
 
     QVBoxLayout *vbox1 = new QVBoxLayout();
+    vbox1->addLayout(hbox0);
+    vbox1->addWidget(boxFunction);
+    vbox1->addSpacing(20);
     vbox1->addWidget(gb1);
     vbox1->addLayout(hbox1);
     vbox1->addWidget(gb4);
+    vbox1->addSpacing(10);
     vbox1->addLayout(hbox3);
+    vbox1->addSpacing(10);
+
+    QFrame *frame4 = new QFrame();
+	frame4->setFrameShape(QFrame::HLine);
+	frame4->setLineWidth(10);
+	palette = this->palette();
+	palette.setColor(QPalette::WindowText, Qt::darkGray);
+	frame4->setPalette(palette);
+	vbox1->addWidget(frame4);
+	vbox1->addSpacing(10);
+    vbox1->addLayout(hbox4);
 
     fitPage = new QWidget();
     fitPage->setLayout(vbox1);
@@ -276,16 +412,35 @@ void FitDialog::initFitPage()
 	connect( buttonAdvanced, SIGNAL(clicked()), this, SLOT(showAdvancedPage() ) );
     connect( tableNamesBox, SIGNAL( activated(int) ), this, SLOT( selectSrcTable(int) ) );
 
-	connect(boxColor, SIGNAL(activated(int)), this, SLOT(updatePreview()));
+	connect(boxColor, SIGNAL(activated(int)), this, SLOT(updatePreviewColor(int)));
 	setFocusProxy(boxFunction);
 }
 
 void FitDialog::initEditPage()
 {
+	QColor blue = QColor(85, 85, 255);
+	QPalette palette;
+    palette.setColor(QPalette::WindowText, blue);
+
+	QFont font = this->font();
+	font.setBold(true);
+	font.setPointSize(font.pointSize() + 1);
+
     QGridLayout *gl1 = new QGridLayout();
-    gl1->addWidget(new QLabel(tr("Category")), 0, 0);
-    gl1->addWidget(new QLabel(tr("Function")), 0, 1);
-    gl1->addWidget(new QLabel(tr("Expression")), 0, 2);
+    QLabel *l = new QLabel(tr("Category"));
+    l->setFont(font);
+    l->setPalette(palette);
+    gl1->addWidget(l, 0, 0);
+
+    l = new QLabel(tr("Function"));
+    l->setFont(font);
+	l->setPalette(palette);
+    gl1->addWidget(l, 0, 1);
+
+    l = new QLabel(tr("Expression"));
+    l->setFont(font);
+	l->setPalette(palette);
+    gl1->addWidget(l, 0, 2);
 
 	categoryBox = new QListWidget();
 	categoryBox->addItem(tr("User defined"));
@@ -329,7 +484,8 @@ void FitDialog::initEditPage()
 	btnAddFunc = new QPushButton(tr( "&Save" ));
     gl2->addWidget(btnAddFunc, 0, 2);
     gl2->addWidget(new QLabel(tr("Parameters")), 1, 0);
-	boxParam = new QLineEdit("a, b");
+	boxParam = new QLabel();
+	boxParam->setFrameStyle(QFrame::Box | QFrame::Sunken);
     gl2->addWidget(boxParam, 1, 1);
 	btnDelFunc = new QPushButton( tr( "&Remove" ));
     gl2->addWidget(btnDelFunc, 1, 2);
@@ -339,8 +495,14 @@ void FitDialog::initEditPage()
 
 	editBox = new QTextEdit();
 	editBox->setTextFormat(Qt::PlainText);
+	editBox->setAcceptRichText(false);
+	connect(editBox->document(), SIGNAL(contentsChange(int,int,int)), this, SLOT(guessParameters()));
 	editBox->setFocus();
 
+	boxErrorMsg = new QLabel();
+	boxErrorMsg->setFrameStyle(QFrame::Box | QFrame::Sunken);
+	boxErrorMsg->hide();
+
     QVBoxLayout *vbox1 = new QVBoxLayout();
 	btnAddTxt = new QPushButton(tr( "Add &expression" ) );
     vbox1->addWidget(btnAddTxt);
@@ -348,21 +510,80 @@ void FitDialog::initEditPage()
     vbox1->addWidget(btnAddName);
 	buttonClear = new QPushButton(tr( "Rese&t" ));
     vbox1->addWidget(buttonClear);
+	vbox1->addStretch();
 	buttonCancel2 = new QPushButton(tr( "&Close" ));
+	buttonCancel2->setPixmap(QPixmap(delete_xpm));
     vbox1->addWidget(buttonCancel2);
-    btnContinue = new QPushButton(tr( "&Fit >>" ));
-    vbox1->addWidget(btnContinue);
-    vbox1->addStretch();
+
+	QVBoxLayout *vb = new QVBoxLayout();
+	vb->addWidget(editBox);
+	vb->addWidget(boxErrorMsg);
 
     QHBoxLayout *hbox2 = new QHBoxLayout();
-	hbox2->addWidget(editBox);
+	hbox2->addLayout(vb);
     hbox2->addLayout(vbox1);
 
+	palette.setColor(QPalette::WindowText, Qt::white);
+    palette.setColor(QPalette::Window, blue);
+
+	QFrame *frame1 = new QFrame();
+	frame1->setFrameStyle(QFrame::Box | QFrame::Plain);
+	frame1->setLineWidth(2);
+	frame1->setAutoFillBackground(true);
+	frame1->setPalette(palette);
+
+	QLabel *selectFunctionLbl = new QLabel(tr("Select Function"));
+	selectFunctionLbl->setFont(font);
+
+	int space = 10;
+	QHBoxLayout *frameLayout = new QHBoxLayout(frame1);
+	frameLayout->addSpacing (space);
+	frameLayout->addWidget(selectFunctionLbl);
+	frameLayout->addSpacing (space);
+
+	btnContinue = new QPushButton();
+	btnContinue->setIcon(QPixmap(next_xpm));
+	btnContinue->setToolTip(tr("Start Fitting Session"));
+
+	QFrame *frame2 = new QFrame();
+	frame2->setFrameShape(QFrame::Box);
+	frame2->setLineWidth(2);
+
+	palette = this->palette();
+	palette.setColor(QPalette::WindowText, blue);
+	frame2->setPalette(palette);
+
+	QLabel *fittingSessionLbl = new QLabel(tr("Fitting Session"));
+	fittingSessionLbl->setFont(font);
+
+	frameLayout = new QHBoxLayout(frame2);
+	frameLayout->addSpacing (space);
+	frameLayout->addWidget(fittingSessionLbl);
+	frameLayout->addSpacing (space);
+
+	QHBoxLayout *hbox3 = new QHBoxLayout();
+	hbox3->addStretch();
+	hbox3->addWidget(frame1);
+    hbox3->addWidget(btnContinue);
+    hbox3->addWidget(frame2);
+
     QVBoxLayout *vbox2 = new QVBoxLayout();
     vbox2->addLayout(gl1);
     vbox2->addLayout(hbox1);
     vbox2->addWidget(gb);
     vbox2->addLayout(hbox2);
+	vbox2->addSpacing (10);
+
+    QFrame *frame3 = new QFrame();
+	frame3->setFrameShape(QFrame::HLine);
+	frame3->setLineWidth(5);
+	palette = this->palette();
+	palette.setColor(QPalette::WindowText, Qt::darkGray);
+	frame3->setPalette(palette);
+	vbox2->addWidget(frame3);
+
+	vbox2->addSpacing (10);
+    vbox2->addLayout(hbox3);
 
     editPage = new QWidget();
     editPage->setLayout(vbox2);
@@ -474,20 +695,15 @@ void FitDialog::initAdvancedPage()
 	connect( plotLabelBox, SIGNAL(stateChanged (int)), this, SLOT(enableApplyChanges(int)));
 
     QHBoxLayout *hbox1 = new QHBoxLayout();
-
-	btnBack = new QPushButton(tr( "<< &Fit" ));
-	connect( btnBack, SIGNAL(clicked()), this, SLOT(returnToFitPage()));
-    hbox1->addWidget(btnBack);
-
+	buttonCancel3 = new QPushButton(tr( "&Close" ));
+	buttonCancel3->setPixmap(QPixmap(delete_xpm));
+    hbox1->addWidget(buttonCancel3);
+    hbox1->addStretch();
 	btnApply = new QPushButton(tr( "&Apply" ));
 	btnApply->setEnabled(false);
 	connect( btnApply, SIGNAL(clicked()), this, SLOT(applyChanges()));
     hbox1->addWidget(btnApply);
 
-	buttonCancel3 = new QPushButton(tr( "&Close" ));
-    hbox1->addWidget(buttonCancel3);
-    hbox1->addStretch();
-
     QVBoxLayout *vbox1 = new QVBoxLayout();
     vbox1->addWidget(gb1);
     vbox1->addWidget(gb2);
@@ -497,6 +713,70 @@ void FitDialog::initAdvancedPage()
     vbox1->addStretch();
     vbox1->addLayout(hbox1);
 
+	QColor blue = QColor(85, 85, 255);
+	QPalette palette = this->palette();
+    palette.setColor(QPalette::WindowText, blue);
+
+	QFont font = this->font();
+	font.setBold(true);
+	font.setPointSize(font.pointSize() + 1);
+
+	QFrame *frame1 = new QFrame();
+	frame1->setFrameStyle(QFrame::Box | QFrame::Plain);
+	frame1->setLineWidth(2);
+	frame1->setPalette(palette);
+
+	QLabel *selectFunctionLbl = new QLabel(tr("Fitting Session"));
+	selectFunctionLbl->setFont(font);
+
+	int space = 10;
+	QHBoxLayout *frameLayout = new QHBoxLayout(frame1);
+	frameLayout->addSpacing (space);
+	frameLayout->addWidget(selectFunctionLbl);
+	frameLayout->addSpacing (space);
+
+	btnBack = new QPushButton();
+	connect( btnBack, SIGNAL(clicked()), this, SLOT(returnToFitPage()));
+	btnBack->setIcon(QPixmap(prev_xpm));
+	btnBack->setToolTip(tr("Fitting Session"));
+
+	QFrame *frame2 = new QFrame();
+	frame2->setFrameShape(QFrame::Box);
+	frame2->setLineWidth(2);
+
+	palette = this->palette();
+	palette.setColor(QPalette::WindowText, Qt::white);
+	palette.setColor(QPalette::Window, blue);
+	frame2->setAutoFillBackground(true);
+	frame2->setPalette(palette);
+
+	QLabel *fittingSessionLbl = new QLabel(tr("Custom Output"));
+	fittingSessionLbl->setFont(font);
+
+	frameLayout = new QHBoxLayout(frame2);
+	frameLayout->addSpacing (space);
+	frameLayout->addWidget(fittingSessionLbl);
+	frameLayout->addSpacing (space);
+
+	QHBoxLayout *hbox2 = new QHBoxLayout();
+	hbox2->addWidget(frame1);
+	hbox2->addWidget(btnBack);
+	hbox2->addWidget(frame2);
+	hbox2->addStretch();
+
+	vbox1->addSpacing (10);
+
+	QFrame *frame3 = new QFrame();
+	frame3->setFrameShape(QFrame::HLine);
+	frame3->setLineWidth(10);
+	palette = this->palette();
+	palette.setColor(QPalette::WindowText, Qt::darkGray);
+	frame3->setPalette(palette);
+	vbox1->addWidget(frame3);
+
+	vbox1->addSpacing (10);
+	vbox1->addLayout(hbox2);
+
     advancedPage = new QWidget();
 	advancedPage->setLayout(vbox1);
     tw->addWidget(advancedPage);
@@ -585,6 +865,7 @@ void FitDialog::setGraph(Graph *g)
 		return;
 
 	d_graph = g;
+
 	boxCurve->clear();
 	boxCurve->addItems(d_graph->analysableCurvesList());
 
@@ -624,11 +905,6 @@ void FitDialog::saveUserFunction()
 				tr("Please enter a function name!"));
 		boxName->setFocus();
 		return;
-	} else if (boxParam->text().remove(QRegExp("[,;\\s]")).isEmpty()){
-		QMessageBox::critical(this, tr("QtiPlot - Input function error"),
-				tr("Please enter at least one parameter name!"));
-		boxParam->setFocus();
-		return;
 	}
 
 	if (builtInFunctionNames().contains(boxName->text())){
@@ -652,17 +928,20 @@ void FitDialog::saveUserFunction()
 	if (lst.contains(name)){
 		int index = lst.findIndex(name);
 		d_current_fit = (NonLinearFit *)d_user_functions[index];
-		d_current_fit->setParametersList(boxParam->text().split(QRegExp("[,;]+[\\s]*"), QString::SkipEmptyParts));
-        d_current_fit->setFormula(formula);
-        d_current_fit->save(d_current_fit->fileName());
+        if (d_current_fit->setFormula(formula))
+        	d_current_fit->save(d_current_fit->fileName());
 
 		if (funcBox->currentItem()->text() == name)
 			showExpression(index);
 	} else {
-	    ApplicationWindow *app = (ApplicationWindow *)this->parent();
+		ApplicationWindow *app = (ApplicationWindow *)this->parent();
+		d_current_fit = new NonLinearFit(app, d_graph);
+		if (!d_current_fit->setFormula(formula))
+			return;
+
 		QString filter = tr("QtiPlot fit model")+" (*.fit);;";
 		filter += tr("All files")+" (*.*)";
-		QString fn = QFileDialog::getSaveFileName(app, tr("QtiPlot") + " - " + tr("Save Fit Model As"),
+		QString fn = ApplicationWindow::getFileName(app, tr("QtiPlot") + " - " + tr("Save Fit Model As"),
                                 app->fitModelsPath + "/" + name, filter);
 		if (!fn.isEmpty()){
             QFileInfo fi(fn);
@@ -671,10 +950,8 @@ void FitDialog::saveUserFunction()
             if (!baseName.contains("."))
                 fn.append(".fit");
 
-            d_current_fit = new NonLinearFit(app, d_graph);
             d_current_fit->setObjectName(name);
-            d_current_fit->setParametersList(boxParam->text().split(QRegExp("[,;]+[\\s]*"), QString::SkipEmptyParts));
-            d_current_fit->setFormula(formula);
+
             if (d_current_fit->save(fn)){
                 QStringList lst = userFunctionNames();
                 lst << name;
@@ -738,7 +1015,6 @@ void FitDialog::showFitPage()
 	ApplicationWindow *app = (ApplicationWindow *)parent();
     if (!boxUseBuiltIn->isChecked()){
         d_current_fit = new NonLinearFit(app, d_graph);
-		d_current_fit->setParametersList(boxParam->text().split(QRegExp("[,;]+[\\s]*"), QString::SkipEmptyParts));
 		formula = parseFormula(formula);
 		d_current_fit->setFormula(formula);
     }
@@ -783,7 +1059,7 @@ void FitDialog::showFitPage()
         it->setFlags(!Qt::ItemIsEditable);
 #endif
         it->setBackground(QBrush(Qt::lightGray));
-        it->setForeground(QBrush(Qt::darkRed));
+        it->setForeground(Qt::black);
         QFont font = it->font();
         font.setBold(true);
         it->setFont(font);
@@ -805,7 +1081,13 @@ void FitDialog::showFitPage()
 		sb->setValue(d_current_fit->initialGuess(i));
         connect(sb, SIGNAL(valueChanged(double)), this, SLOT(updatePreview()));
         boxParams->setCellWidget(i, 2, sb);
+
+        it = new QTableWidgetItem();
+		it->setFlags(!Qt::ItemIsEditable);
+		it->setText("--");
+		boxParams->setItem(i, 5, it);
 	}
+
     for (int i = 0; i<parameters; i++)
         boxParams->item (i, 0)->setText(paramList[i]);
 
@@ -836,8 +1118,6 @@ void FitDialog::showFitPage()
 
 void FitDialog::showEditPage()
 {
-    if (d_current_fit)
-        d_current_fit->freeMemory();
 	tw->setCurrentWidget(editPage);
 }
 
@@ -859,13 +1139,17 @@ void FitDialog::showAdvancedPage()
 void FitDialog::setFunction(bool ok)
 {
 	editBox->setEnabled(!ok);
-	boxParam->setEnabled(!ok);
 	boxName->setEnabled(!ok);
 	btnAddFunc->setEnabled(!ok);
 	btnAddName->setEnabled(!ok);
 	btnAddTxt->setEnabled(!ok);
 	buttonClear->setEnabled(!ok);
 
+	setCurrentFit(funcBox->currentRow());//reset current fit
+	setEditorTextColor(Qt::black);// reset text color to black
+	boxErrorMsg->hide();
+	boxErrorMsg->clear();
+
 	if (ok){
 		boxName->setText(funcBox->currentItem()->text());
 		editBox->setText(explainBox->text());
@@ -932,28 +1216,38 @@ void FitDialog::showFunctionsList(int category)
 }
 
 void FitDialog::chooseFolder()
+{
+	if (categoryBox->currentRow() == 3)//plugins
+        choosePluginsFolder();
+	else if (!categoryBox->currentRow())//user-defined
+	    chooseFitModelsFolder();
+}
+
+void FitDialog::choosePluginsFolder()
 {
 	ApplicationWindow *app = (ApplicationWindow *)this->parent();
-	if (categoryBox->currentRow() == 3){//plugins
-        QString dir = QFileDialog::getExistingDirectory(this, tr("Choose the plugins folder"), app->fitPluginsPath);
+	QString dir = QFileDialog::getExistingDirectory(this, tr("Choose the plugins folder"), app->fitPluginsPath);
         if (!dir.isEmpty()){
             funcBox->clear();
             explainBox->clear();
             app->fitPluginsPath = dir;
             loadPlugins();
         }
-	} else if (!categoryBox->currentRow()){//user-defined
-	    QString dir = QFileDialog::getExistingDirectory(this, tr("Choose the fit models folder"), app->fitModelsPath);
-        if (!dir.isEmpty()){
-            funcBox->clear();
-            explainBox->clear();
-            app->fitModelsPath = dir;
-            loadUserFunctions();
+}
 
-            QString path = app->fitModelsPath + "/";
-            foreach(Fit *fit, d_built_in_functions)
-                fit->setFileName(path + fit->objectName() + ".fit");
-        }
+void FitDialog::chooseFitModelsFolder()
+{
+	ApplicationWindow *app = (ApplicationWindow *)this->parent();
+	QString dir = QFileDialog::getExistingDirectory(this, tr("Choose the fit models folder"), app->fitModelsPath);
+	if (!dir.isEmpty()){
+		funcBox->clear();
+		explainBox->clear();
+		app->fitModelsPath = dir;
+		loadUserFunctions();
+
+		QString path = app->fitModelsPath + "/";
+		foreach(Fit *fit, d_built_in_functions)
+			fit->setFileName(path + fit->objectName() + ".fit");
 	}
 }
 
@@ -993,11 +1287,40 @@ void FitDialog::showParseFunctions()
 	funcBox->addItems(MyParser::functionsList());
 }
 
+void FitDialog::setCurrentFit(int function)
+{
+	if (function < 0)
+        return;
+
+	switch(categoryBox->currentRow()){
+		case 0:
+			if (d_user_functions.size() > function)
+				d_current_fit = d_user_functions[function];
+		break;
+
+		case 1:
+			d_current_fit = d_built_in_functions[function];
+		break;
+
+		case 2:
+		break;
+
+		case 3:
+			if (d_plugins.size() > 0)
+		    	d_current_fit = d_plugins[function];
+		break;
+	}
+}
+
 void FitDialog::showExpression(int function)
 {
     if (function < 0)
         return;
 
+	setCurrentFit(function);
+	if (!d_current_fit)
+		return;
+
 	if (categoryBox->currentRow() == 2)
 		explainBox->setText(MyParser::explainFunction(function));
 	else if (categoryBox->currentRow() == 1){
@@ -1013,20 +1336,16 @@ void FitDialog::showExpression(int function)
 			polynomOrderLabel->show();
 			polynomOrderBox->show();
 		}
-
-		d_current_fit = d_built_in_functions[function];
 		explainBox->setText(d_current_fit->formula());
 		setFunction(boxUseBuiltIn->isChecked());
 	} else if (categoryBox->currentRow() == 0){
-		if (d_user_functions.size() > function){
-			d_current_fit = d_user_functions[function];
+		if (d_user_functions.size() > function)
 			explainBox->setText(d_current_fit->formula());
-		} else
+		else
 			explainBox->clear();
 		setFunction(boxUseBuiltIn->isChecked());
 	} else if (categoryBox->currentRow() == 3){
 		if (d_plugins.size() > 0){
-		    d_current_fit = d_plugins[function];
 			explainBox->setText(d_current_fit->formula());
 			setFunction(boxUseBuiltIn->isChecked());
 		} else
@@ -1105,7 +1424,7 @@ void FitDialog::accept()
 	NonLinearFit *nlf = qobject_cast<NonLinearFit *>(d_current_fit);
 	if (nlf)
 		nlf->removeConstants();
-	
+
 	try {
 		if (!boxParams->isColumnHidden(4)){
 			int j = 0;
@@ -1121,8 +1440,8 @@ void FitDialog::accept()
 				} else {
 					double val = ((DoubleSpinBox*)boxParams->cellWidget(i, 2))->value();
 					QString constName = boxParams->item(i, 0)->text();
-					((NonLinearFit *)d_current_fit)->setConstant(constName, val);
-					parser.DefineConst(constName.ascii(), val);	
+					nlf->setConstant(constName, val);
+					parser.DefineConst(constName.ascii(), val);
 				}
 			}
 		} else {
@@ -1155,9 +1474,10 @@ void FitDialog::accept()
 #else
 			modifyGuesses (paramsInit);
 #endif
-		if (d_current_fit->type() == Fit::User){
-			d_current_fit->setParametersList(parameters);
-			d_current_fit->setFormula(formula);
+
+		if (nlf){
+			if (!nlf->setParametersList(parameters)) return;
+			if (!nlf->setFormula(formula, false)) return;
 		}
 
 #ifdef Q_CC_MSVC
@@ -1169,12 +1489,12 @@ void FitDialog::accept()
 		if (!d_current_fit->setDataFromCurve(curve, start, end) ||
 			!d_current_fit->setWeightingData ((Fit::WeightingMethod)boxWeighting->currentIndex(),
 					       tableNamesBox->currentText()+"_"+colNamesBox->currentText())) return;
-			
+
 		if (btnParamRange->isEnabled()){
 			for (int i=0; i<n; i++)
 				d_current_fit->setParameterRange(i, paramRangeLeft[i], paramRangeRight[i]);
 		}
-		
+
 		d_current_fit->setTolerance(eps);
 		d_current_fit->setOutputPrecision(app->fit_output_precision);
 		d_current_fit->setAlgorithm((Fit::Algorithm)boxAlgorithm->currentIndex());
@@ -1185,21 +1505,28 @@ void FitDialog::accept()
 			d_current_fit->scaleErrors(scaleErrorsBox->isChecked());
 		d_current_fit->fit();
 		double *res = d_current_fit->results();
+		double *err = d_current_fit->errors();
+		QLocale locale = app->locale();
 		if (!boxParams->isColumnHidden(4)){
 			int j = 0;
-			for (int i=0; i<rows; i++){
+			for (int i = 0; i < rows; i++){
                 QCheckBox *cb = (QCheckBox*)boxParams->cellWidget(i, 4);
-				if (!cb->isChecked())
+				if (!cb->isChecked()){
+					boxParams->item(i, 5)->setText(QString(QChar(0x00B1)) + " " + locale.toString(err[j], 'e', boxPrecision->value()));
 					((DoubleSpinBox*)boxParams->cellWidget(i, 2))->setValue(res[j++]);
+				} else
+					boxParams->item(i, 5)->setText("--");
 			}
 		} else {
-			for (int i=0; i<rows; i++)
+			for (int i = 0; i < rows; i++){
 				((DoubleSpinBox*)boxParams->cellWidget(i, 2))->setValue(res[i]);
+				boxParams->item(i, 5)->setText(QString(QChar(0x00B1)) + " " + locale.toString(err[i], 'e', boxPrecision->value()));
+			}
 		}
 
 		if (globalParamTableBox->isChecked() && d_param_table)
 			d_current_fit->writeParametersToTable(d_param_table, true);
-		
+
 		if (previewBox->isChecked())
 			updatePreview();
 	}
@@ -1213,25 +1540,9 @@ void FitDialog::modifyGuesses(double* initVal)
 	ExponentialFit *ef = qobject_cast<ExponentialFit *>(d_current_fit);
 	if (ef){
 		if (ef->isExponentialGrowth())
-			initVal[1] = -1/initVal[1];
-		else 
-			initVal[1] = 1/initVal[1];
+			initVal[1] = -initVal[1];
 		return;
 	}
-	
-	TwoExpFit *two_ef = qobject_cast<TwoExpFit *>(d_current_fit);
-	if (two_ef){
-		initVal[1] = 1/initVal[1];
-		initVal[3] = 1/initVal[3];
-		return;
-	}
-	
-	ThreeExpFit *three_ef = qobject_cast<ThreeExpFit *>(d_current_fit);
-	if (three_ef){
-		initVal[1] = 1/initVal[1];
-		initVal[3] = 1/initVal[3];
-		initVal[5] = 1/initVal[5];
-	}
 }
 
 void FitDialog::changeDataRange()
@@ -1272,7 +1583,7 @@ void FitDialog::selectSrcTable(int tabnr)
 
 void FitDialog::enableWeightingParameters(int index)
 {
-	if (index == Fit::Dataset){
+	if (index == Fit::Dataset || index == Fit::Direct){
 		tableNamesBox->setEnabled(true);
 		colNamesBox->setEnabled(true);
 	} else {
@@ -1283,15 +1594,18 @@ void FitDialog::enableWeightingParameters(int index)
 
 void FitDialog::closeEvent (QCloseEvent * e)
 {
-    if (d_preview_curve){
+    if (d_graph && d_preview_curve){
         d_preview_curve->detach();
-        d_graph->replot();
         delete d_preview_curve;
+        d_graph->replot();
     }
 
 	if(d_current_fit && d_current_fit->dataSize() && plotLabelBox->isChecked())
 		d_current_fit->showLegend();
 
+	if (d_current_fit)
+		delete d_current_fit;
+
 	e->accept();
 }
 
@@ -1424,6 +1738,11 @@ void FitDialog::saveInitialGuesses()
     if (!d_current_fit)
         return;
 
+	ApplicationWindow *app = (ApplicationWindow *)this->parent();
+	QFileInfo fi(app->fitModelsPath);
+	if (app->fitModelsPath.isEmpty() || !fi.isDir() || !fi.isWritable())
+		chooseFitModelsFolder();
+
 	int rows = boxParams->rowCount();
     for (int i=0; i<rows; i++)
         d_current_fit->setInitialGuess(i, ((DoubleSpinBox*)boxParams->cellWidget(i, 2))->value());
@@ -1432,10 +1751,9 @@ void FitDialog::saveInitialGuesses()
     if (!fileName.isEmpty())
         d_current_fit->save(fileName);
     else {
-	    ApplicationWindow *app = (ApplicationWindow *)this->parent();
 		QString filter = tr("QtiPlot fit model") + " (*.fit);;";
 		filter += tr("All files") + " (*.*)";
-		QString fn = QFileDialog::getSaveFileName(app, tr("QtiPlot") + " - " + tr("Save Fit Model As"),
+		QString fn = ApplicationWindow::getFileName(app, tr("QtiPlot") + " - " + tr("Save Fit Model As"),
                                 app->fitModelsPath + "/" + d_current_fit->objectName(), filter);
 		if (!fn.isEmpty()){
             QFileInfo fi(fn);
@@ -1463,65 +1781,76 @@ void FitDialog::returnToFitPage()
 	tw->setCurrentWidget(fitPage);
 }
 
+void FitDialog::showPreview(bool on)
+{
+	if (on)
+		updatePreview();
+	else if (d_preview_curve){
+        d_preview_curve->setVisible(false);
+        d_graph->replot();
+    }
+}
+
 void FitDialog::updatePreview()
 {
-    if (!previewBox->isChecked() && d_preview_curve){
-        d_preview_curve->detach();
-        d_graph->replot();
-        delete d_preview_curve;
-        d_preview_curve = 0;
-        return;
-    }
-
     if (!d_current_fit || !previewBox->isChecked())
-        return;
-
-    int d_points = generatePointsBox->value();
-#ifdef Q_CC_MSVC
-    QVarLengthArray<double> X(d_points), Y(d_points);
-#else
-    double X[d_points], Y[d_points];
-#endif
-    int p = boxParams->rowCount();
-#ifdef Q_CC_MSVC
-    QVarLengthArray<double> parameters(p);
-#else
-    double parameters[p];
-#endif
-    for (int i=0; i<p; i++)
-        parameters[i] = ((DoubleSpinBox*)boxParams->cellWidget(i, 2))->value();
-    if (d_current_fit->type() == Fit::BuiltIn)
-#ifdef Q_CC_MSVC
-        modifyGuesses(parameters.data());
-#else
-        modifyGuesses(parameters);
-#endif
-
-    double x0 = boxFrom->value();
-    double step = (boxTo->value() - x0)/(d_points - 1);
-    for (int i=0; i<d_points; i++){
-        double x = x0 + i*step;
-        X[i] = x;
-#ifdef Q_CC_MSVC
-        Y[i] = d_current_fit->eval(parameters.data(), x);
-#else
-        Y[i] = d_current_fit->eval(parameters, x);
-#endif
-    }
+		return;
 
 	if (!d_preview_curve){
-        d_preview_curve = new QwtPlotCurve();
+		d_preview_curve = new FunctionCurve();
+		d_preview_curve->attach(d_graph);
 		d_preview_curve->setRenderHint(QwtPlotItem::RenderAntialiased, d_graph->antialiasing());
-        d_preview_curve->attach(d_graph);
+		d_preview_curve->setPen(QPen(ColorBox::color(boxColor->currentIndex()), 1));
 	}
 
-    d_preview_curve->setPen(QPen(ColorBox::color(boxColor->currentIndex()), 1));
-#ifdef Q_CC_MSVC
-    d_preview_curve->setData(X.data(), Y.data(), d_points);
-#else
-    d_preview_curve->setData(X, Y, d_points);
-#endif
-    d_graph->replot();
+	bool changedVar = false;
+	int parameters = boxParams->rowCount();
+	QMap<QString, double> variables = d_preview_curve->constants();
+	if (variables.size () != parameters)
+		changedVar = true;
+	else {
+		for (int i = 0; i < parameters; i++){
+			QString key = boxParams->item(i, 0)->text();
+			if (variables.value(key) != ((DoubleSpinBox*)boxParams->cellWidget(i, 2))->value()){
+				changedVar = true;
+				break;
+			}
+		}
+ 	}
+
+	if (!changedVar && d_preview_curve->startRange() == boxFrom->value() &&
+		d_preview_curve->endRange() == boxTo->value() &&
+		d_preview_curve->formulas()[0] == d_current_fit->formula()){
+		if (!d_preview_curve->isVisible()){
+			d_preview_curve->setVisible(true);
+			d_graph->replot();
+		}
+		return;
+	}
+
+	d_preview_curve->setRange(boxFrom->value(), boxTo->value());
+	d_preview_curve->setFormula(d_current_fit->formula());
+
+	if (changedVar){
+		d_preview_curve->removeConstants();
+		for (int i = 0; i < parameters; i++)
+			d_preview_curve->setConstant(boxParams->item(i, 0)->text(),
+						((DoubleSpinBox*)boxParams->cellWidget(i, 2))->value());
+	}
+
+	d_preview_curve->loadData(generatePointsBox->value());
+	if (!d_preview_curve->isVisible())
+		d_preview_curve->setVisible(true);
+	d_graph->replot();
+}
+
+void FitDialog::updatePreviewColor(int colorIndex)
+{
+    if (!d_preview_curve)
+		return;
+
+    d_preview_curve->setPen(QPen(ColorBox::color(colorIndex), 1));
+	d_graph->replot();
 }
 
 void FitDialog::showParameterRange(bool on)
@@ -1587,3 +1916,35 @@ void FitDialog::showPredictionLimits()
 
 	d_current_fit->showPredictionLimits(boxConfidenceLevel->value());
 }
+
+void FitDialog::guessParameters()
+{
+	if (boxUseBuiltIn->isChecked())
+		return;
+
+	QString text = editBox->text().remove(QRegExp("\\s")).remove(".");
+	if (text.isEmpty()){
+		boxParam->clear();
+		return;
+	}
+
+	bool error = false;
+	string errMsg;
+	boxParam->setText(NonLinearFit::guessParameters(text, &error, &errMsg).join(", "));
+	if (error){
+		setEditorTextColor(Qt::red);// highlight text in red
+		boxErrorMsg->setText(tr("Error: ") + QString(errMsg.c_str()));
+		boxErrorMsg->show();
+	} else {
+		setEditorTextColor(Qt::black);// reset text color to black
+		boxErrorMsg->hide();
+		boxErrorMsg->clear();
+	}
+}
+
+void FitDialog::setEditorTextColor(const QColor& c)
+{
+	QPalette palette = editBox->palette();
+	palette.setColor (QPalette::Text, c);
+	editBox->setPalette(palette);
+}
