@@ -2,8 +2,8 @@
     File                 : PolynomFitDialog.cpp
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2006 by Ion Vasilief, Tilman Hoener zu Siederdissen
-    Email (use @ for *)  : ion_vasilief*yahoo.fr, thzs*gmx.net
+    Copyright            : (C) 2006 by Ion Vasilief
+    Email (use @ for *)  : ion_vasilief*yahoo.fr
     Description          : Fit polynomial dialog
 
  ***************************************************************************/
@@ -27,11 +27,11 @@
  *                                                                         *
  ***************************************************************************/
 #include "PolynomFitDialog.h"
-#include "../PolynomialFit.h"
-#include "../../plot2D/Graph.h"
-#include "../../ColorBox.h"
-#include "../../ApplicationWindow.h"
-#include "../../DoubleSpinBox.h"
+#include <PolynomialFit.h>
+#include <Graph.h>
+#include <ApplicationWindow.h>
+#include <DoubleSpinBox.h>
+#include <ColorBox.h>
 
 #include <QSpinBox>
 #include <QCheckBox>
