@@ -1,219 +1,218 @@
 #include "fit_gsl.h"
-#include "../MyParser.h"
+#include <MyParser.h>
 #include "NonLinearFit.h"
 
 #include <math.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <stddef.h>
-#include <qmessagebox.h>
+#include <QMessageBox>
 #include <gsl/gsl_blas.h>
 #include <gsl/gsl_math.h>
 
-int expd3_f (const gsl_vector * x, void *params, gsl_vector * f) {
-    size_t n = ((struct FitData *)params)->n;
+int expd3_f (const gsl_vector * x, void *params, gsl_vector * f){
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
-    double A1=gsl_vector_get(x,0);
-    double t1=gsl_vector_get(x,1);
-    double A2=gsl_vector_get(x,2);
-    double t2=gsl_vector_get(x,3);
-    double A3=gsl_vector_get(x,4);
-    double t3=gsl_vector_get(x,5);
-    double y0=gsl_vector_get(x,6);
-    size_t i;
-    for (i = 0; i < n; i++) {
-        double Yi = A1 * exp (-X[i]*t1) + A2 * exp (-X[i]*t2) + A3 * exp (-X[i]*t3) +y0;
-        gsl_vector_set (f, i, (Yi - Y[i])/sigma[i]);
+    double A1 = gsl_vector_get(x, 0);
+    double t1 = gsl_vector_get(x, 1);
+    double A2 = gsl_vector_get(x, 2);
+    double t2 = gsl_vector_get(x, 3);
+    double A3 = gsl_vector_get(x, 4);
+    double t3 = gsl_vector_get(x, 5);
+    double y0 = gsl_vector_get(x, 6);
+    for (int i = 0; i < n; i++) {
+    	double t = -X[i];
+        double Yi = A1 * exp(t/t1) + A2 * exp(t/t2) + A3 * exp(t/t3) + y0;
+		double s = 1.0/sqrt(sigma[i]);
+        gsl_vector_set (f, i, (Yi - Y[i])/s);
     }
     return GSL_SUCCESS;
 }
-double expd3_d (const gsl_vector * x, void *params) {
-    size_t n = ((struct FitData *)params)->n;
+double expd3_d (const gsl_vector * x, void *params){
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
-    double A1=gsl_vector_get(x,0);
-    double t1=gsl_vector_get(x,1);
-    double A2=gsl_vector_get(x,2);
-    double t2=gsl_vector_get(x,3);
-    double A3=gsl_vector_get(x,4);
-    double t3=gsl_vector_get(x,5);
-    double y0=gsl_vector_get(x,6);
-    size_t i;
-    double val=0;
-    for (i = 0; i < n; i++) {
-        double dYi = ((A1 * exp (-X[i]*t1) + A2 * exp (-X[i]*t2) + A3 * exp (-X[i]*t3) +y0)-Y[i])/sigma[i];
-        val+=dYi * dYi;
+    double A1 = gsl_vector_get(x, 0);
+    double t1 = gsl_vector_get(x, 1);
+    double A2 = gsl_vector_get(x, 2);
+    double t2 = gsl_vector_get(x, 3);
+    double A3 = gsl_vector_get(x, 4);
+    double t3 = gsl_vector_get(x, 5);
+    double y0 = gsl_vector_get(x, 6);
+    double val = 0;
+    for (int i = 0; i < n; i++) {
+    	double t = -X[i];
+		double s = 1.0/sqrt(sigma[i]);
+        double dYi = ((A1 * exp(t/t1) + A2 * exp(t/t2) + A3 * exp(t/t3) + y0) - Y[i])/s;
+        val += dYi * dYi;
     }
     return val;
 }
-int expd3_df (const gsl_vector * x, void *params, gsl_matrix * J) {
-    size_t n = ((struct FitData *)params)->n;
+int expd3_df (const gsl_vector * x, void *params, gsl_matrix * J){
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *sigma = ((struct FitData *)params)->sigma;
-    double A1=gsl_vector_get(x,0);
-    double l1=gsl_vector_get(x,1);
-    double A2=gsl_vector_get(x,2);
-    double l2=gsl_vector_get(x,3);
-    double A3=gsl_vector_get(x,4);
-    double l3=gsl_vector_get(x,5);
-    size_t i;
-    for (i = 0; i < n; i++) {
+    double A1 = gsl_vector_get(x, 0);
+    double l1 = gsl_vector_get(x, 1);
+    double A2 = gsl_vector_get(x, 2);
+    double l2 = gsl_vector_get(x, 3);
+    double A3 = gsl_vector_get(x, 4);
+    double l3 = gsl_vector_get(x, 5);
+    for (int i = 0; i < n; i++) {
         /* Jacobian matrix J(i,j) = dfi / dxj, */
         /* where fi = (Yi - yi)/sigma[i],      */
-        /*       Yi = A1 * exp(-xi*l1) + A2 * exp(-xi*l2) +y0  */
-        /* and the xj are the parameters (A1,l1,A2,l2,y0) */
+        /*       Yi = A1 * exp(-xi/l1) + A2 * exp(-xi/l2) + A3 * exp(-xi/l3) + y0  */
+        /* and the xj are the parameters (A1, l1 ,A2, l2, A3, l3, y0) */
         double t = X[i];
-        double s = sigma[i];
+        double s = 1.0/sqrt(sigma[i]);
         double e1 = exp(-t*l1)/s;
         double e2 = exp(-t*l2)/s;
         double e3 = exp(-t*l3)/s;
         gsl_matrix_set (J, i, 0, e1);
-        gsl_matrix_set (J, i, 1, -t * A1 * e1);
+        gsl_matrix_set (J, i, 1, t * A1 * e1/(l1*l1));
         gsl_matrix_set (J, i, 2, e2);
-        gsl_matrix_set (J, i, 3, -t * A2 * e2);
+        gsl_matrix_set (J, i, 3, t * A2 * e2/(l2*l2));
         gsl_matrix_set (J, i, 4, e3);
-        gsl_matrix_set (J, i, 5, -t * A3 * e3);
+        gsl_matrix_set (J, i, 5, t * A3 * e3/(l3*l3));
         gsl_matrix_set (J, i, 6, 1/s);
     }
     return GSL_SUCCESS;
 }
-int expd3_fdf (const gsl_vector * x, void *params, gsl_vector * f, gsl_matrix * J) {
+int expd3_fdf (const gsl_vector * x, void *params, gsl_vector * f, gsl_matrix * J){
     expd3_f (x, params, f);
     expd3_df (x, params, J);
     return GSL_SUCCESS;
 }
-int expd2_f (const gsl_vector * x, void *params, gsl_vector * f) {
-    size_t n = ((struct FitData *)params)->n;
+int expd2_f (const gsl_vector * x, void *params, gsl_vector * f){
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
-    double A1=gsl_vector_get(x,0);
-    double t1=gsl_vector_get(x,1);
-    double A2=gsl_vector_get(x,2);
-    double t2=gsl_vector_get(x,3);
-    double y0=gsl_vector_get(x,4);
-    size_t i;
-    for (i = 0; i < n; i++) {
-        double Yi = A1 * exp (-X[i]*t1) + A2 * exp (-X[i]*t2) + y0;
-        gsl_vector_set (f, i, (Yi - Y[i])/sigma[i]);
+    double A1 = gsl_vector_get(x, 0);
+    double t1 = gsl_vector_get(x, 1);
+    double A2 = gsl_vector_get(x, 2);
+    double t2 = gsl_vector_get(x, 3);
+    double y0 = gsl_vector_get(x, 4);
+    for (int i = 0; i < n; i++) {
+    	double t = -X[i];
+        double Yi = A1 * exp (t/t1) + A2 * exp (t/t2) + y0;
+		double s = 1.0/sqrt(sigma[i]);
+        gsl_vector_set (f, i, (Yi - Y[i])/s);
     }
     return GSL_SUCCESS;
 }
-double expd2_d (const gsl_vector * x, void *params) {
-    size_t n = ((struct FitData *)params)->n;
+double expd2_d (const gsl_vector * x, void *params){
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
-    double A1=gsl_vector_get(x,0);
-    double t1=gsl_vector_get(x,1);
-    double A2=gsl_vector_get(x,2);
-    double t2=gsl_vector_get(x,3);
-    double y0=gsl_vector_get(x,4);
-    size_t i;
-    double val=0;
-    for (i = 0; i < n; i++) {
-        double dYi = ((A1 * exp (-X[i]*t1) + A2 * exp (-X[i]*t2) + y0)-Y[i])/sigma[i];
-        val+=dYi * dYi;
+    double A1 = gsl_vector_get(x, 0);
+    double t1 = gsl_vector_get(x, 1);
+    double A2 = gsl_vector_get(x, 2);
+    double t2 = gsl_vector_get(x, 3);
+    double y0 = gsl_vector_get(x, 4);
+    double val = 0;
+    for (int i = 0; i < n; i++) {
+    	double t = -X[i];
+		double s = 1.0/sqrt(sigma[i]);
+        double dYi = ((A1 * exp (t/t1) + A2 * exp (t/t2) + y0) - Y[i])/s;
+        val += dYi * dYi;
     }
     return val;
 }
-int expd2_df (const gsl_vector * x, void *params, gsl_matrix * J) {
-    size_t n = ((struct FitData *)params)->n;
+int expd2_df (const gsl_vector * x, void *params, gsl_matrix * J){
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *sigma = ((struct FitData *)params)->sigma;
-    double A1=gsl_vector_get(x,0);
-    double l1=gsl_vector_get(x,1);
-    double A2=gsl_vector_get(x,2);
-    double l2=gsl_vector_get(x,3);
-    size_t i;
-    for (i = 0; i < n; i++) {
+    double A1 = gsl_vector_get(x, 0);
+    double l1 = gsl_vector_get(x, 1);
+    double A2 = gsl_vector_get(x, 2);
+    double l2 = gsl_vector_get(x, 3);
+    for (int i = 0; i < n; i++) {
         /* Jacobian matrix J(i,j) = dfi / dxj, */
         /* where fi = (Yi - yi)/sigma[i],      */
-        /*       Yi = A1 * exp(-xi*l1) + A2 * exp(-xi*l2) +y0  */
-        /* and the xj are the parameters (A1,l1,A2,l2,y0) */
-        double s = sigma[i];
+        /*       Yi = A1 * exp(-xi/l1) + A2 * exp(-xi/l2) + y0  */
+        /* and the xj are the parameters (A1, l1, A2, l2, y0) */
+        double s = 1.0/sqrt(sigma[i]);
         double t = X[i];
-        double e1 = exp(-t*l1)/s;
-        double e2 = exp(-t*l2)/s;
+        double e1 = exp(-t/l1)/s;
+        double e2 = exp(-t/l2)/s;
         gsl_matrix_set (J, i, 0, e1);
-        gsl_matrix_set (J, i, 1, -t * A1 * e1);
+        gsl_matrix_set (J, i, 1, t * A1 * e1/(l1*l1));
         gsl_matrix_set (J, i, 2, e2);
-        gsl_matrix_set (J, i, 3, -t * A2 * e2);
+        gsl_matrix_set (J, i, 3, t * A2 * e2/(l2*l2));
         gsl_matrix_set (J, i, 4, 1/s);
     }
     return GSL_SUCCESS;
 }
-int expd2_fdf (const gsl_vector * x, void *params,
-               gsl_vector * f, gsl_matrix * J) {
+int expd2_fdf (const gsl_vector * x, void *params, gsl_vector * f, gsl_matrix * J){
     expd2_f (x, params, f);
     expd2_df (x, params, J);
     return GSL_SUCCESS;
 }
 int exp_f (const gsl_vector * x, void *params, gsl_vector * f) {
-    size_t n = ((struct FitData *)params)->n;
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
     double A = gsl_vector_get (x, 0);
     double lambda = gsl_vector_get (x, 1);
     double b = gsl_vector_get (x, 2);
-    size_t i;
-    for (i = 0; i < n; i++) {
-        double Yi = A * exp (-lambda * X[i]) + b;
-        gsl_vector_set (f, i, (Yi - Y[i])/sigma[i]);
+    for (int i = 0; i < n; i++) {
+        double Yi = A * exp (-X[i]/lambda) + b;
+        double s = 1.0/sqrt(sigma[i]);
+        gsl_vector_set (f, i, (Yi - Y[i])/s);
     }
     return GSL_SUCCESS;
 }
 double exp_d (const gsl_vector * x, void *params) {
-    size_t n = ((struct FitData *)params)->n;
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
     double A = gsl_vector_get (x, 0);
     double lambda = gsl_vector_get (x, 1);
     double b = gsl_vector_get (x, 2);
-    size_t i;
     double val=0;
-    for (i = 0; i < n; i++) {
-        double dYi = ((A * exp (-lambda * X[i]) + b)-Y[i])/sigma[i];
-        val+=dYi * dYi;
+    for (int i = 0; i < n; i++) {
+        double s = 1.0/sqrt(sigma[i]);
+        double dYi = ((A * exp(-X[i]/lambda) + b) - Y[i])/s;
+        val += dYi * dYi;
     }
     return val;
 }
-int exp_df (const gsl_vector * x, void *params,
-            gsl_matrix * J) {
-    size_t n = ((struct FitData *)params)->n;
+int exp_df (const gsl_vector * x, void *params, gsl_matrix * J) {
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *sigma = ((struct FitData *)params)->sigma;
     double A = gsl_vector_get (x, 0);
     double lambda = gsl_vector_get (x, 1);
-    size_t i;
-    for (i = 0; i < n; i++) {
+    for (int i = 0; i < n; i++) {
         /* Jacobian matrix J(i,j) = dfi / dxj, */
         /* where fi = (Yi - yi)/sigma[i],      */
-        /*       Yi = A * exp(-lambda * i) + b  */
-        /* and the xj are the parameters (A,lambda,b) */
+        /*       Yi = A * exp(-i/lambda) + b  */
+        /* and the xj are the parameters (A, lambda, b) */
         double t = X[i];
-        double s = sigma[i];
-        double e = exp(-lambda * t);
+        double s = 1.0/sqrt(sigma[i]);
+        double e = exp(-t/lambda);
         gsl_matrix_set (J, i, 0, e/s);
-        gsl_matrix_set (J, i, 1, -t * A * e/s);
+        gsl_matrix_set (J, i, 1, t * A * e/(s*lambda*lambda));
         gsl_matrix_set (J, i, 2, 1/s);
     }
     return GSL_SUCCESS;
 }
-int exp_fdf (const gsl_vector * x, void *params,
-             gsl_vector * f, gsl_matrix * J) {
+
+int exp_fdf (const gsl_vector * x, void *params, gsl_vector * f, gsl_matrix * J) {
     exp_f (x, params, f);
     exp_df (x, params, J);
     return GSL_SUCCESS;
 }
 
 int gauss_f (const gsl_vector * x, void *params, gsl_vector * f) {
-    size_t n = ((struct FitData *)params)->n;
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
@@ -221,16 +220,16 @@ int gauss_f (const gsl_vector * x, void *params, gsl_vector * f) {
     double A = gsl_vector_get (x, 1);
     double C = gsl_vector_get (x, 2);
     double w = gsl_vector_get (x, 3);
-    size_t i;
-    for (i = 0; i < n; i++) {
-        double diff=X[i]-C;
-        double Yi = A*exp(-0.5*diff*diff/(w*w))+Y0;
-        gsl_vector_set (f, i, (Yi - Y[i])/sigma[i]);
+    for (int i = 0; i < n; i++) {
+        double diff = X[i]-C;
+        double Yi = A*exp(-0.5*diff*diff/(w*w)) + Y0;
+		double s = 1.0/sqrt(sigma[i]);
+		gsl_vector_set (f, i, (Yi - Y[i])/s);
     }
     return GSL_SUCCESS;
 }
 double gauss_d (const gsl_vector * x, void *params) {
-    size_t n = ((struct FitData *)params)->n;
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
@@ -238,30 +237,29 @@ double gauss_d (const gsl_vector * x, void *params) {
     double A = gsl_vector_get (x, 1);
     double C = gsl_vector_get (x, 2);
     double w = gsl_vector_get (x, 3);
-    size_t i;
     double val=0;
-    for (i = 0; i < n; i++) {
-        double diff=X[i]-C;
-        double dYi = ((A*exp(-0.5*diff*diff/(w*w))+Y0)-Y[i])/sigma[i];
-        val+=dYi * dYi;
+    for (int i = 0; i < n; i++) {
+        double diff = X[i] - C;
+        double s = 1.0/sqrt(sigma[i]);
+        double dYi = ((A*exp(-0.5*diff*diff/(w*w)) + Y0) - Y[i])/s;
+        val += dYi * dYi;
     }
     return val;
 }
-int gauss_df (const gsl_vector * x, void *params,
-              gsl_matrix * J) {
-    size_t n = ((struct FitData *)params)->n;
+int gauss_df (const gsl_vector * x, void *params, gsl_matrix * J) {
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *sigma = ((struct FitData *)params)->sigma;
     double A = gsl_vector_get (x, 1);
     double C = gsl_vector_get (x, 2);
     double w = gsl_vector_get (x, 3);
-    size_t i;
-    for (i = 0; i < n; i++) {
+    for (int i = 0; i < n; i++) {
         /* Jacobian matrix J(i,j) = dfi / dxj,	 */
         /* where fi = Yi - yi,					*/
         /* Yi = y=A*exp[-(Xi-xc)^2/(2*w*w)]+B		*/
         /* and the xj are the parameters (B,A,C,w) */
-        double s = sigma[i];
+        //double s = sigma[i];
+        double s = 1.0/sqrt(sigma[i]);
         double diff = X[i]-C;
         double e = exp(-0.5*diff*diff/(w*w))/s;
         gsl_matrix_set (J, i, 0, 1/s);
@@ -278,30 +276,30 @@ int gauss_fdf (const gsl_vector * x, void *params,
     return GSL_SUCCESS;
 }
 int gauss_multi_peak_f (const gsl_vector * x, void *params, gsl_vector * f) {
-    size_t n = ((struct FitData *)params)->n;
-    size_t p = ((struct FitData *)params)->p;
+    int n = ((struct FitData *)params)->n;
+    int p = ((struct FitData *)params)->p;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
-    size_t peaks = (p-1)/3;
+    int peaks = (p-1)/3;
     double *a = new double[peaks];
     double *xc = new double[peaks];
     double *w2 = new double[peaks];
     double offset = gsl_vector_get (x, p-1);
-    size_t i,j;
-    for (i = 0; i < peaks; i++) {
+    for (int i = 0; i < peaks; i++) {
         xc[i] = gsl_vector_get(x, 3*i+1);
         double wi = gsl_vector_get(x, 3*i+2);
         a[i] = sqrt(M_2_PI)*gsl_vector_get(x, 3*i)/wi;
         w2[i] = wi*wi;
     }
-    for (i = 0; i < n; i++) {
+    for (int i = 0; i < n; i++) {
         double res = 0;
-        for (j = 0; j < peaks; j++) {
+        for (int j = 0; j < peaks; j++) {
             double diff=X[i]-xc[j];
             res += a[j]*exp(-2*diff*diff/w2[j]);
         }
-        gsl_vector_set(f, i, (res + offset - Y[i])/sigma[i]);
+		double s = 1.0/sqrt(sigma[i]);
+        gsl_vector_set(f, i, (res + offset - Y[i])/s);
     }
     delete[] a;
     delete[] xc;
@@ -309,32 +307,32 @@ int gauss_multi_peak_f (const gsl_vector * x, void *params, gsl_vector * f) {
     return GSL_SUCCESS;
 }
 double gauss_multi_peak_d (const gsl_vector * x, void *params) {
-    size_t n = ((struct FitData *)params)->n;
-    size_t p = ((struct FitData *)params)->p;
+    int n = ((struct FitData *)params)->n;
+    int p = ((struct FitData *)params)->p;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
-    size_t peaks = (p-1)/3;
+    int peaks = (p-1)/3;
     double *a = new double[peaks];
     double *xc = new double[peaks];
     double *w2 = new double[peaks];
     double offset = gsl_vector_get (x, p-1);
-    size_t i,j;
-    double val=0;
-    for (i = 0; i < peaks; i++) {
+
+    for (int i = 0; i < peaks; i++) {
         xc[i] = gsl_vector_get(x, 3*i+1);
         double wi = gsl_vector_get(x, 3*i+2);
         a[i] = sqrt(M_2_PI)*gsl_vector_get(x, 3*i)/wi;
         w2[i] = wi*wi;
     }
-    double t;
-    for (i = 0; i < n; i++) {
+	double val=0;
+    for (int i = 0; i < n; i++) {
         double res = 0;
-        for (j = 0; j < peaks; j++) {
+        for (int j = 0; j < peaks; j++) {
             double diff=X[i]-xc[j];
             res+= a[j]*exp(-2*diff*diff/w2[j]);
         }
-        t = (res+offset-Y[i])/sigma[i];
+		double s = 1.0/sqrt(sigma[i]);
+        double t = (res + offset - Y[i])/s;
         val += t*t;
     }
     delete[] a;
@@ -343,23 +341,22 @@ double gauss_multi_peak_d (const gsl_vector * x, void *params) {
     return val;
 }
 int gauss_multi_peak_df (const gsl_vector * x, void *params, gsl_matrix * J) {
-    size_t n = ((struct FitData *)params)->n;
-    size_t p = ((struct FitData *)params)->p;
+    int n = ((struct FitData *)params)->n;
+    int p = ((struct FitData *)params)->p;
     double *X = ((struct FitData *)params)->X;
     double *sigma = ((struct FitData *)params)->sigma;
-    size_t peaks = (p-1)/3;
+    int peaks = (p-1)/3;
     double *a = new double[peaks];
     double *xc = new double[peaks];
     double *w = new double[peaks];
-    size_t i,j;
-    for (i = 0; i<peaks; i++) {
+    for (int i = 0; i<peaks; i++) {
         a[i] = gsl_vector_get (x, 3*i);
         xc[i] = gsl_vector_get (x, 3*i+1);
         w[i] = gsl_vector_get (x, 3*i+2);
     }
-    for (i = 0; i<n; i++) {
-        double s = sigma[i];
-        for (j = 0; j<peaks; j++) {
+    for (int i = 0; i<n; i++) {
+        double s = 1.0/sqrt(sigma[i]);
+        for (int j = 0; j<peaks; j++) {
             double diff = X[i]-xc[j];
             double w2 = w[j]*w[j];
             double e = sqrt(M_2_PI)/s*exp(-2*diff*diff/w2);
@@ -380,29 +377,29 @@ int gauss_multi_peak_fdf (const gsl_vector * x, void *params, gsl_vector * f, gs
     return GSL_SUCCESS;
 }
 int lorentz_multi_peak_f (const gsl_vector * x, void *params, gsl_vector * f) {
-    size_t n = ((struct FitData *)params)->n;
-    size_t p = ((struct FitData *)params)->p;
+    int n = ((struct FitData *)params)->n;
+    int p = ((struct FitData *)params)->p;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
-    size_t peaks = (p-1)/3;
+    int peaks = (p-1)/3;
     double *a = new double[peaks];
     double *xc = new double[peaks];
     double *w = new double[peaks];
     double offset = gsl_vector_get (x, p-1);
-    size_t i,j;
-    for (i = 0; i < peaks; i++) {
+    for (int i = 0; i < peaks; i++) {
         a[i] = gsl_vector_get(x, 3*i);
         xc[i] = gsl_vector_get(x, 3*i+1);
         w[i] = gsl_vector_get(x, 3*i+2);
     }
-    for (i = 0; i < n; i++) {
+    for (int i = 0; i < n; i++) {
         double res = 0;
-        for (j = 0; j < peaks; j++) {
+        for (int j = 0; j < peaks; j++) {
             double diff=X[i]-xc[j];
             res += a[j]*w[j]/(4*diff*diff+w[j]*w[j]);
         }
-        gsl_vector_set(f, i, (M_2_PI*res + offset - Y[i])/sigma[i]);
+		double s = 1.0/sqrt(sigma[i]);
+        gsl_vector_set(f, i, (M_2_PI*res + offset - Y[i])/s);
     }
     delete[] a;
     delete[] xc;
@@ -410,30 +407,31 @@ int lorentz_multi_peak_f (const gsl_vector * x, void *params, gsl_vector * f) {
     return GSL_SUCCESS;
 }
 double lorentz_multi_peak_d (const gsl_vector * x, void *params) {
-    size_t n = ((struct FitData *)params)->n;
-    size_t p = ((struct FitData *)params)->p;
+    int n = ((struct FitData *)params)->n;
+    int p = ((struct FitData *)params)->p;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
-    size_t peaks = (p-1)/3;
+    int peaks = (p-1)/3;
     double *a = new double[peaks];
     double *xc = new double[peaks];
     double *w = new double[peaks];
     double offset = gsl_vector_get (x, p-1);
-    size_t i,j;
-    double val=0,t;
-    for (i = 0; i < peaks; i++) {
+    for (int i = 0; i < peaks; i++) {
         a[i] = gsl_vector_get(x, 3*i);
         xc[i] = gsl_vector_get(x, 3*i+1);
         w[i] = gsl_vector_get(x, 3*i+2);
     }
-    for (i = 0; i < n; i++) {
+
+	double val = 0;
+    for (int i = 0; i < n; i++) {
         double res = 0;
-        for (j = 0; j < peaks; j++) {
+        for (int j = 0; j < peaks; j++) {
             double diff = X[i]-xc[j];
             res += a[j]*w[j]/(4*diff*diff+w[j]*w[j]);
         }
-        t = (M_2_PI*res + offset - Y[i])/sigma[i];
+		double s = 1.0/sqrt(sigma[i]);
+        double t = (M_2_PI*res + offset - Y[i])/s;
         val += t*t;
     }
     delete[] a;
@@ -442,23 +440,22 @@ double lorentz_multi_peak_d (const gsl_vector * x, void *params) {
     return GSL_SUCCESS;
 }
 int lorentz_multi_peak_df (const gsl_vector * x, void *params, gsl_matrix * J) {
-    size_t n = ((struct FitData *)params)->n;
-    size_t p = ((struct FitData *)params)->p;
+    int n = ((struct FitData *)params)->n;
+    int p = ((struct FitData *)params)->p;
     double *X = ((struct FitData *)params)->X;
     double *sigma = ((struct FitData *)params)->sigma;
-    size_t peaks = (p-1)/3;
+    int peaks = (p-1)/3;
     double *a = new double[peaks];
     double *xc = new double[peaks];
     double *w = new double[peaks];
-    size_t i,j;
-    for (i = 0; i<peaks; i++) {
+    for (int i = 0; i<peaks; i++) {
         a[i] = gsl_vector_get (x, 3*i);
         xc[i] = gsl_vector_get (x, 3*i+1);
         w[i] = gsl_vector_get (x, 3*i+2);
     }
-    for (i = 0; i<n; i++) {
-        double s = sigma[i];
-        for (j = 0; j<peaks; j++) {
+    for (int i = 0; i<n; i++) {
+        double s = 1.0/sqrt(sigma[i]);
+        for (int j = 0; j<peaks; j++) {
             double diff = X[i]-xc[j];
             double diff2 = diff*diff;
             double w2 = w[j]*w[j];
@@ -483,78 +480,88 @@ int lorentz_multi_peak_fdf (const gsl_vector * x, void *params, gsl_vector * f,
 }
 
 int user_f(const gsl_vector * x, void *params, gsl_vector * f) {
-    size_t n = ((struct FitData *)params)->n;
-    size_t p = ((struct FitData *)params)->p;
+    int n = ((struct FitData *)params)->n;
+    int p = ((struct FitData *)params)->p;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
-		
+
 	NonLinearFit *fitter = (NonLinearFit *)((struct FitData *) params)->fitter;
 	const char *function = fitter->formula().ascii();
 	QStringList parNames = fitter->parameterNames();
-	
+
 	MyParser parser;
     try {
         double *parameters = new double[p];
         double xvar;
         parser.DefineVar("x", &xvar);
-        for (int i=0; i<(int)p; i++) {
-            parameters[i]=gsl_vector_get(x,i);
+        for (int i = 0; i < p; i++) {
+            parameters[i] = gsl_vector_get(x,i);
             parser.DefineVar(parNames[i].ascii(), &parameters[i]);
         }
-		
+
 		QMapIterator<QString, double> i(fitter->constants());
  		while (i.hasNext()){
      		i.next();
 			parser.DefineConst(i.key().ascii(), i.value());
  		}
-		
+
         parser.SetExpr(function);
-        for (int j = 0; j < (int)n; j++) {
-            xvar=X[j];
-            gsl_vector_set (f, j, (parser.Eval() - Y[j])/sigma[j]);
-        }
+        for (int j = 0; j < n; j++) {
+            xvar = X[j];
+			double s = 1.0/sqrt(sigma[j]);
+			try {
+			    gsl_vector_set (f, j, (parser.EvalRemoveSingularity(&xvar) - Y[j])/s);
+			} catch (MyParser::Pole) {
+			    return GSL_ESING;
+			}
+         }
         delete[] parameters;
     } catch (mu::ParserError &e) {
-        QMessageBox::critical(0,"QtiPlot - Input function error",QString::fromStdString(e.GetMsg()));
+        QMessageBox::critical(0, "QtiPlot - Input function error", QString::fromStdString(e.GetMsg()));
         return GSL_EINVAL;
-    }	
+    }
     return GSL_SUCCESS;
 }
 
 double user_d(const gsl_vector * x, void *params) {
-    size_t n = ((struct FitData *)params)->n;
-    size_t p = ((struct FitData *)params)->p;
+    int n = ((struct FitData *)params)->n;
+    int p = ((struct FitData *)params)->p;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
-    
+
 	NonLinearFit *fitter = (NonLinearFit *)((struct FitData *) params)->fitter;
 	const char *function = fitter->formula().ascii();
 	QStringList parNames = fitter->parameterNames();
-	
+
     double val=0;
     MyParser parser;
     try {
         double *parameters = new double[p];
         double xvar;
         parser.DefineVar("x", &xvar);
-        for (int i=0; i<(int)p; i++) {
+        for (int i=0; i < p; i++) {
             parameters[i]=gsl_vector_get(x,i);
             parser.DefineVar(parNames[i].ascii(), &parameters[i]);
         }
-		
+
 		QMapIterator<QString, double> i(fitter->constants());
  		while (i.hasNext()){
      		i.next();
 			parser.DefineConst(i.key().ascii(), i.value());
  		}
-		
+
         parser.SetExpr(function);
-        for (int j = 0; j < (int)n; j++) {
-            xvar=X[j];
-            double t=(parser.Eval() - Y[j])/sigma[j];
-            val+=t*t;
+        for (int j = 0; j < n; j++) {
+            xvar = X[j];
+			double s = 1.0/sqrt(sigma[j]);
+            try {
+				double t = (parser.EvalRemoveSingularity(&xvar) - Y[j])/s;
+				val += t*t;
+			} catch (MyParser::Pole) {
+				return GSL_POSINF; //weird, I know. blame gsl.
+			}
         }
         delete[] parameters;
     } catch (mu::ParserError &e) {
@@ -565,36 +572,41 @@ double user_d(const gsl_vector * x, void *params) {
 }
 
 int user_df(const gsl_vector *x, void *params, gsl_matrix *J) {
-    size_t n = ((struct FitData *)params)->n;
-    size_t p = ((struct FitData *)params)->p;
+    int n = ((struct FitData *)params)->n;
+    int p = ((struct FitData *)params)->p;
     double *X = ((struct FitData *)params)->X;
     double *sigma = ((struct FitData *)params)->sigma;
-    
+
 	NonLinearFit *fitter = (NonLinearFit *)((struct FitData *) params)->fitter;
 	const char *function = fitter->formula().ascii();
 	QStringList parNames = fitter->parameterNames();
-	
+
 	try {
         double *param = new double[p];
         MyParser parser;
         double xvar;
         parser.DefineVar("x", &xvar);
-        for (int k=0; k<(int)p; k++) {
-            param[k]=gsl_vector_get(x,k);
+        for (int k=0; k<p; k++) {
+            param[k] = gsl_vector_get(x,k);
             parser.DefineVar(parNames[k].ascii(), &param[k]);
         }
-		
+
 		QMapIterator<QString, double> i(fitter->constants());
  		while (i.hasNext()){
      		i.next();
 			parser.DefineConst(i.key().ascii(), i.value());
  		}
-		
+
         parser.SetExpr(function);
-        for (int i = 0; i<(int)n; i++) {
+        for (int i = 0; i < n; i++) {
             xvar = X[i];
-            for (int j=0; j<(int)p; j++)
-                gsl_matrix_set (J, i, j, 1/sigma[i]*parser.Diff(&param[j], param[j]));
+			double s = 1.0/sqrt(sigma[i]);
+            for (int j = 0; j < p; j++)
+	        try {
+				gsl_matrix_set (J, i, j, 1.0/s*parser.DiffRemoveSingularity(&xvar, &param[j], param[j]));
+			} catch (MyParser::Pole) {
+				return GSL_ESING;
+			}
         }
         delete[] param;
     } catch (mu::ParserError &) {
@@ -604,13 +616,15 @@ int user_df(const gsl_vector *x, void *params, gsl_matrix *J) {
 }
 
 int user_fdf(const gsl_vector * x, void *params, gsl_vector * f, gsl_matrix * J) {
-    user_f (x, params, f);
-    user_df (x, params, J);
-    return GSL_SUCCESS;
+    int status = user_f (x, params, f);
+    if (status)
+    	return status;
+    status = user_df (x, params, J);
+    return status;
 }
 
 int boltzmann_f (const gsl_vector * x, void *params, gsl_vector * f) {
-    size_t n = ((struct FitData *)params)->n;
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
@@ -618,15 +632,15 @@ int boltzmann_f (const gsl_vector * x, void *params, gsl_vector * f) {
     double A2 = gsl_vector_get (x, 1);
     double x0 = gsl_vector_get (x, 2);
     double dx = gsl_vector_get (x, 3);
-    size_t i;
-    for (i = 0; i < n; i++) {
-        double Yi = (A1-A2)/(1+exp((X[i]-x0)/dx))+A2;
-        gsl_vector_set (f, i, (Yi - Y[i])/sigma[i]);
+    for (int i = 0; i < n; i++) {
+        double Yi = (A1-A2)/(1 + exp((X[i] - x0)/dx)) + A2;
+		double s = 1.0/sqrt(sigma[i]);
+        gsl_vector_set (f, i, (Yi - Y[i])/s);
     }
     return GSL_SUCCESS;
 }
 double boltzmann_d (const gsl_vector * x, void *params) {
-    size_t n = ((struct FitData *)params)->n;
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
@@ -634,29 +648,28 @@ double boltzmann_d (const gsl_vector * x, void *params) {
     double A2 = gsl_vector_get (x, 1);
     double x0 = gsl_vector_get (x, 2);
     double dx = gsl_vector_get (x, 3);
-    size_t i;
     double val=0;
-    for (i = 0; i < n; i++) {
-        double dYi = ((A1-A2)/(1+exp((X[i]-x0)/dx)) + A2 - Y[i])/sigma[i];
-        val+=dYi * dYi;
+    for (int i = 0; i < n; i++) {
+		double s = 1.0/sqrt(sigma[i]);
+        double dYi = ((A1 - A2)/(1 + exp((X[i] - x0)/dx)) + A2 - Y[i])/s;
+        val += dYi * dYi;
     }
     return val;
 }
 int boltzmann_df (const gsl_vector * x, void *params, gsl_matrix * J) {
-    size_t n = ((struct FitData *)params)->n;
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *sigma = ((struct FitData *)params)->sigma;
     double A1 = gsl_vector_get (x, 0);
     double A2 = gsl_vector_get (x, 1);
     double x0 = gsl_vector_get (x, 2);
     double dx = gsl_vector_get (x, 3);
-    size_t i;
-    for (i = 0; i < n; i++) {
+    for (int i = 0; i < n; i++) {
         /* Jacobian matrix J(i,j) = dfi / dxj,		*/
         /* where fi = Yi - yi,						*/
         /* Yi = (A1-A2)/(1+exp((X[i]-x0)/dx)) + A2	*/
         /* and the xj are the parameters (A1,A2,x0,dx)*/
-        double s = sigma[i];
+        double s = 1.0/sqrt(sigma[i]);
         double diff = X[i]-x0;
         double e = exp(diff/dx);
         double r = 1/(1+e);
@@ -675,7 +688,7 @@ int boltzmann_fdf (const gsl_vector * x, void *params, gsl_vector * f, gsl_matri
 }
 
 int logistic_f (const gsl_vector * x, void *params, gsl_vector * f) {
-    size_t n = ((struct FitData *)params)->n;
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
@@ -684,17 +697,17 @@ int logistic_f (const gsl_vector * x, void *params, gsl_vector * f) {
     double A2 = gsl_vector_get (x, 1);
     double x0 = gsl_vector_get (x, 2);
     double p = gsl_vector_get (x, 3);
-    size_t i;
-    for (i = 0; i < n; i++) {
-        double Yi = (A1-A2)/(1+pow(X[i]/x0, p))+A2;
-        gsl_vector_set (f, i, (Yi - Y[i])/sigma[i]);
+    for (int i = 0; i < n; i++) {
+        double Yi = (A1 - A2)/(1 + pow(X[i]/x0, p)) + A2;
+		double s = 1.0/sqrt(sigma[i]);
+        gsl_vector_set (f, i, (Yi - Y[i])/s);
     }
 
     return GSL_SUCCESS;
 }
 
 double logistic_d (const gsl_vector * x, void *params) {
-    size_t n = ((struct FitData *)params)->n;
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *Y = ((struct FitData *)params)->Y;
     double *sigma = ((struct FitData *)params)->sigma;
@@ -703,17 +716,17 @@ double logistic_d (const gsl_vector * x, void *params) {
     double A2 = gsl_vector_get (x, 1);
     double x0 = gsl_vector_get (x, 2);
     double p = gsl_vector_get (x, 3);
-    size_t i;
     double val=0;
-    for (i = 0; i < n; i++) {
-        double dYi = ((A1-A2)/(1+pow(X[i]/x0, p)) + A2 - Y[i])/sigma[i];
+    for (int i = 0; i < n; i++) {
+		double s = 1.0/sqrt(sigma[i]);
+        double dYi = ((A1 - A2)/(1 + pow(X[i]/x0, p)) + A2 - Y[i])/s;
         val += dYi * dYi;
     }
     return val;
 }
 
 int logistic_df (const gsl_vector * x, void *params, gsl_matrix * J) {
-    size_t n = ((struct FitData *)params)->n;
+    int n = ((struct FitData *)params)->n;
     double *X = ((struct FitData *)params)->X;
     double *sigma = ((struct FitData *)params)->sigma;
 
@@ -721,13 +734,12 @@ int logistic_df (const gsl_vector * x, void *params, gsl_matrix * J) {
     double A2 = gsl_vector_get (x, 1);
     double x0 = gsl_vector_get (x, 2);
     double p = gsl_vector_get (x, 3);
-    size_t i;
-    for (i = 0; i < n; i++) {
+    for (int i = 0; i < n; i++) {
         /* Jacobian matrix J(i,j) = dfi / dxj,		*/
         /* where fi = Yi - yi,						*/
         /* Yi = (A1-A2)/(1+(X[i]/x0)^p)) + A2	*/
         /* and the xj are the parameters (A1,A2,x0,p)*/
-        double s = sigma[i];
+        double s = 1.0/sqrt(sigma[i]);
         double rap = X[i]/x0;
         double r = 1/(1 + pow(rap, p));
         double aux = (A1-A2)*r*r*pow(rap, p);
