@@ -2,8 +2,8 @@
     File                 : Matrix.cpp
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2004-2007 by Ion Vasilief, Knut Franke
-    Email (use @ for *)  : ion_vasilief*yahoo.fr, knut.franke*gmx.de
+    Copyright            : (C) 2004-2009 by Ion Vasilief
+    Email (use @ for *)  : ion_vasilief*yahoo.fr
     Description          : Matrix worksheet class
 
  ***************************************************************************/
@@ -28,10 +28,11 @@
  ***************************************************************************/
 #include "Matrix.h"
 #include "MatrixCommand.h"
-#include "../plot2D/Graph.h"
-#include "../ApplicationWindow.h"
-#include "../muParserScript.h"
-#include "../ScriptingEnv.h"
+#include <Graph.h>
+#include <ApplicationWindow.h>
+#include <muParserScript.h>
+#include <ScriptingEnv.h>
+#include <ColorMapEditor.h>
 
 #include <QtGlobal>
 #include <QTextStream>
@@ -56,6 +57,8 @@
 #include <QSvgGenerator>
 #include <QFile>
 #include <QUndoStack>
+#include <QTextDocumentWriter>
+#include <QTextTable>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -63,6 +66,10 @@
 #include <gsl/gsl_math.h>
 #include <gsl/gsl_linalg.h>
 
+#ifdef EMF_OUTPUT
+#include <EmfEngine.h>
+#endif
+
 Matrix::Matrix(ScriptingEnv *env, int r, int c, const QString& label, ApplicationWindow* parent, const QString& name, Qt::WFlags f)
 : MdiSubWindow(label, parent, name, f), scripted(env)
 {
@@ -84,8 +91,8 @@ void Matrix::initGlobals()
     imageLabel = NULL;
 
     d_header_view_type = ColumnRow;
-	d_color_map_type = GrayScale;
-	d_color_map = QwtLinearColorMap(Qt::black, Qt::white);
+	d_color_map_type = Default;
+	d_color_map = applicationWindow()->d_3D_color_map;
     d_column_width = 100;
 
 	formula_str = "";
@@ -167,6 +174,7 @@ void Matrix::setCoordinates(double xs, double xe, double ys, double ye)
 	y_end = ye;
 
 	emit modifiedWindow(this);
+	modifiedData(this);
 }
 
 void Matrix::save(const QString &fn, const QString &info, bool saveAsTemplate)
@@ -200,25 +208,44 @@ void Matrix::save(const QString &fn, const QString &info, bool saveAsTemplate)
 
 	if (d_color_map_type != Custom)
 		t << "ColorPolicy\t" + QString::number(d_color_map_type) + "\n";
-	else {
-		t << "<ColorMap>\n";
-		t << "\t<Mode>" + QString::number(d_color_map.mode()) + "</Mode>\n";
-		t << "\t<MinColor>" + d_color_map.color1().name() + "</MinColor>\n";
-		t << "\t<MaxColor>" + d_color_map.color2().name() + "</MaxColor>\n";
-		QwtArray <double> colors = d_color_map.colorStops();
-		int stops = (int)colors.size();
-		t << "\t<ColorStops>" + QString::number(stops - 2) + "</ColorStops>\n";
-		for (int i = 1; i < stops - 1; i++){
-			t << "\t<Stop>" + QString::number(colors[i]) + "\t";
-			t << QColor(d_color_map.rgb(QwtDoubleInterval(0,1), colors[i])).name();
-			t << "</Stop>\n";
-		}
-		t << "</ColorMap>\n";
-	}
+	else
+		t << ColorMapEditor::saveToXmlString(d_color_map);
 
-    if (notTemplate)
-        t << d_matrix_model->saveToString();
-    t <<"</matrix>\n";
+    if (notTemplate){//save data
+		t << "<data>\n";
+		double* d_data = d_matrix_model->dataVector();
+		int d_rows = numRows();
+		int d_cols = numCols();
+		int cols = d_cols - 1;
+
+		for(int i = 0; i < d_rows; i++){
+			int aux = d_cols*i;
+			bool emptyRow = true;
+			for(int j = 0; j < d_cols; j++){
+				if (gsl_finite(d_data[aux + j])){
+					emptyRow = false;
+					break;
+				}
+			}
+			if (emptyRow)
+				continue;
+
+			t << QString::number(i) + "\t";
+
+			for(int j = 0; j < cols; j++){
+				double val = d_data[aux + j];
+				if (gsl_finite(val))
+					t << QString::number(val, 'e', 16);
+				t << "\t";
+			}
+			double val = d_data[aux + cols];
+			if (gsl_finite(val))
+				t << QString::number(val, 'e', 16);
+			t << "\n";
+		}
+		t << "</data>\n";
+    }
+    t << "</matrix>\n";
 }
 
 void Matrix::restore(const QStringList &lst)
@@ -280,6 +307,16 @@ void Matrix::restore(const QStringList &lst)
     resetView();
 }
 
+void Matrix::setNumericPrecision(int prec)
+{
+	if (prec < 0)
+		prec = 0;
+	else if (prec > 13)
+		prec = 13;
+
+	num_precision = prec;
+}
+
 void Matrix::setNumericFormat(const QChar& f, int prec)
 {
 	if (txt_format == f && num_precision == prec)
@@ -323,9 +360,6 @@ void Matrix::setDimensions(int rows, int cols)
 	if (r == rows && c == cols)
 		return;
 
-	if (rows <= 0 || cols <= 0 || INT_MAX/rows < cols) //avoid integer overflow
-		return;
-
 	if(rows*cols > r*c && !d_matrix_model->canResize(rows, cols))
 		return;
 
@@ -338,6 +372,7 @@ void Matrix::setDimensions(int rows, int cols)
 		resetView();
 	}
 	emit modifiedWindow(this);
+	modifiedData(this);
 }
 
 double Matrix::integrate()
@@ -454,9 +489,7 @@ bool Matrix::canCalculate(bool useMuParser)
 		return false;
 
 	if (useMuParser){
-    	muParserScript *mup = new muParserScript(scriptEnv, formula_str, this, QString("<%1>").arg(objectName()), false);
-		connect(mup, SIGNAL(error(const QString&,const QString&,int)), scriptEnv, SIGNAL(error(const QString&, const QString&,int)));
-
+    	muParserScript *mup = new muParserScript(scriptEnv, formula_str, this, QString("<%1>").arg(objectName()));
     	double *ri = mup->defineVariable("i");
     	double *rr = mup->defineVariable("row");
     	double *cj = mup->defineVariable("j");
@@ -464,12 +497,13 @@ bool Matrix::canCalculate(bool useMuParser)
     	double *x = mup->defineVariable("x");
     	double *y = mup->defineVariable("y");
 
-		if (!mup->compile())
-			return false;
-
 		double r = 1.0;
         *ri = r; *rr = r; *y = r;
         double c = 1.0; *cj = c; *cc = c; *x = c;
+
+		if (!mup->compile())
+			return false;
+
 		int codeLines = mup->codeLines();
 		if (codeLines == 1 && gsl_isnan(mup->evalSingleLine()))
 			return false;
@@ -510,10 +544,12 @@ bool Matrix::muParserCalculate(int startRow, int endRow, int startCol, int endCo
 		d_undo_stack->push(new MatrixUndoCommand(d_matrix_model, MuParserCalculate, startRow, endRow,
 												startCol, endCol, buffer, tr("Calculate Values")));
 		emit modifiedWindow(this);
+		modifiedData(this);
 		return true;
 	} else if(ignoreUndo()){
 		d_matrix_model->muParserCalculate(startRow, endRow, startCol, endCol);
 		emit modifiedWindow(this);
+		modifiedData(this);
 		return true;
 	}
 	return false;
@@ -529,10 +565,12 @@ bool Matrix::calculate(int startRow, int endRow, int startCol, int endCol, bool
     	d_undo_stack->push(new MatrixUndoCommand(d_matrix_model, Calculate, startRow, endRow,
 												startCol, endCol, buffer, tr("Calculate Values")));
 		emit modifiedWindow(this);
+		modifiedData(this);
 		return true;
 	} else if(ignoreUndo()){
 		d_matrix_model->calculate(startRow, endRow, startCol, endCol);
 		emit modifiedWindow(this);
+		modifiedData(this);
 		return true;
 	}
 	return false;
@@ -556,9 +594,11 @@ void Matrix::clearSelection()
 	if (buffer){
     	d_undo_stack->push(new MatrixUndoCommand(d_matrix_model, Clear, startRow, endRow, startCol, endCol, buffer, tr("Clear Selection")));
     	emit modifiedWindow(this);
+    	modifiedData(this);
 	} else if (ignoreUndo()){
 		d_matrix_model->clear(startRow, endRow, startCol, endCol);
 		emit modifiedWindow(this);
+		modifiedData(this);
 	}
 }
 
@@ -596,7 +636,7 @@ void Matrix::copySelection()
 
 void Matrix::pasteSelection()
 {
-     if (d_view_type == ImageView)
+	if (d_view_type == ImageView)
         return;
 
 	QString text = QApplication::clipboard()->text();
@@ -605,7 +645,7 @@ void Matrix::pasteSelection()
 
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
-	QStringList linesList = text.split(applicationWindow()->endOfLine(), QString::SkipEmptyParts);
+	QStringList linesList = text.split(ApplicationWindow::guessEndOfLine(text));
 	int rows = linesList.size();
 	if (!rows)
 		return;
@@ -670,9 +710,11 @@ void Matrix::pasteSelection()
 					leftCol, rightCol, clipboardBuffer, rows, cols, backupBuffer, oldRows,
 					oldCols, tr("Paste")));
 		emit modifiedWindow(this);
+		modifiedData(this);
 	} else if (ignoreUndo()){
 		d_matrix_model->pasteData(clipboardBuffer, topRow, leftCol, rows, cols);
 		emit modifiedWindow(this);
+		modifiedData(this);
 	}
 }
 
@@ -706,10 +748,12 @@ void Matrix::deleteSelectedRows()
     	d_undo_stack->push(new MatrixDeleteRowsCommand(d_matrix_model, startRow, count, buffer, tr("Delete Rows") + " " +
                       QString::number(startRow + 1) + " - " + QString::number(startRow + count)));
     	emit modifiedWindow(this);
+    	modifiedData(this);
 	} else if (ignoreUndo()){
 		d_matrix_model->removeRows(startRow, count);
 		d_table_view->reset();
 		emit modifiedWindow(this);
+		modifiedData(this);
 	}
 }
 
@@ -737,10 +781,12 @@ void Matrix::deleteSelectedColumns()
     	d_undo_stack->push(new MatrixDeleteColsCommand(d_matrix_model, startCol, count, buffer, tr("Delete Columns") + " " +
                       QString::number(startCol + 1) + " - " + QString::number(startCol + count)));
     	emit modifiedWindow(this);
+    	modifiedData(this);
 	} else if (ignoreUndo()){
 		d_matrix_model->removeColumns(startCol, count);
 		d_table_view->reset();
 		emit modifiedWindow(this);
+		modifiedData(this);
 	}
 }
 
@@ -791,6 +837,7 @@ void Matrix::insertRow()
                       QString::number(index.row() + 1)));
 	d_table_view->reset();
 	emit modifiedWindow(this);
+	modifiedData(this);
 }
 
 void Matrix::insertColumn()
@@ -810,6 +857,7 @@ void Matrix::insertColumn()
                       QString::number(index.column() + 1)));
 	d_table_view->reset();
 	emit modifiedWindow(this);
+	modifiedData(this);
 }
 
 void Matrix::customEvent(QEvent *e)
@@ -818,9 +866,27 @@ void Matrix::customEvent(QEvent *e)
 		scriptingChangeEvent((ScriptingChangeEvent*)e);
 }
 
-void Matrix::exportRasterImage(const QString& fileName, int quality)
+void Matrix::exportRasterImage(const QString& fileName, int quality, int dpi)
 {
-	d_matrix_model->renderImage().save(fileName, 0, quality);
+	if (!dpi)
+		dpi = logicalDpiX();
+
+	QImage image = d_matrix_model->renderImage();
+	int dpm = (int)ceil(100.0/2.54*dpi);
+	image.setDotsPerMeterX(dpm);
+	image.setDotsPerMeterY(dpm);
+	if (fileName.endsWith(".odf")){
+		QTextDocument *document = new QTextDocument();
+		QTextCursor cursor = QTextCursor(document);
+		cursor.movePosition(QTextCursor::End);
+		cursor.insertText(objectName());
+		cursor.insertBlock();
+		cursor.insertImage(image);
+
+		QTextDocumentWriter writer(fileName);
+		writer.write(document);
+	} else
+		image.save(fileName, 0, quality);
 }
 
 void Matrix::exportToFile(const QString& fileName)
@@ -836,7 +902,14 @@ void Matrix::exportToFile(const QString& fileName)
 	} else if(fileName.contains(".svg")){
 		exportSVG(fileName);
 		return;
-	} else {
+	}
+#ifdef EMF_OUTPUT
+	else if(fileName.contains(".emf")){
+		exportEMF(fileName);
+		return;
+	}
+#endif
+	else {
 		QList<QByteArray> list = QImageWriter::supportedImageFormats();
     	for(int i=0 ; i<list.count() ; i++){
 			if (fileName.contains( "." + list[i].toLower())){
@@ -850,21 +923,16 @@ void Matrix::exportToFile(const QString& fileName)
 
 void Matrix::exportSVG(const QString& fileName)
 {
-	#if QT_VERSION >= 0x040300
-		if (d_view_type != ImageView)
-			return;
+	int width = numRows();
+	int height = numCols();
 
-		int width = numRows();
-		int height = numCols();
+	QSvgGenerator svg;
+	svg.setFileName(fileName);
+	svg.setSize(QSize(width, height));
 
-		QSvgGenerator svg;
-        svg.setFileName(fileName);
-        svg.setSize(QSize(width, height));
-
-		QPainter p(&svg);
-        p.drawImage (QRect(0, 0, width, height), d_matrix_model->renderImage());
-		p.end();
-	#endif
+	QPainter p(&svg);
+	p.drawImage (QRect(0, 0, width, height), d_matrix_model->renderImage());
+	p.end();
 }
 
 void Matrix::exportPDF(const QString& fileName)
@@ -877,6 +945,96 @@ void Matrix::print()
     print(QString());
 }
 
+void Matrix::print(QPrinter *printer)
+{
+	if (!printer)
+		return;
+
+	QPainter p;
+	if ( !p.begin(printer))
+		return; // paint on printer
+
+	int dpiy = printer->logicalDpiY();
+	const int margin = (int) ( (1/2.54)*dpiy ); // 1 cm margins
+
+	if (d_view_type == ImageView){
+		p.drawImage (printer->pageRect(), d_matrix_model->renderImage());
+		return;
+	}
+
+	QHeaderView *vHeader = d_table_view->verticalHeader();
+
+	int rows = numRows();
+	int cols = numCols();
+	int height = margin;
+	int i, vertHeaderWidth = vHeader->width();
+	int right = margin + vertHeaderWidth;
+
+	// print header
+	p.setFont(QFont());
+	QString header_label = d_matrix_model->headerData(0, Qt::Horizontal).toString();
+	QRect br = p.boundingRect(br, Qt::AlignCenter, header_label);
+	p.drawLine(right, height, right, height+br.height());
+	QRect tr(br);
+
+	for(i=0; i<cols; i++){
+		int w = d_table_view->columnWidth(i);
+		tr.setTopLeft(QPoint(right,height));
+		tr.setWidth(w);
+		tr.setHeight(br.height());
+		header_label = d_matrix_model->headerData(i, Qt::Horizontal).toString();
+		p.drawText(tr, Qt::AlignCenter, header_label,-1);
+		right += w;
+		p.drawLine(right, height, right, height+tr.height());
+
+		if (right >= printer->width()-2*margin )
+			break;
+	}
+
+	p.drawLine(margin + vertHeaderWidth, height, right-1, height);//first horizontal line
+	height += tr.height();
+	p.drawLine(margin, height, right-1, height);
+
+	// print table values
+	for(i=0;i<rows;i++){
+		right = margin;
+		QString cell_text = d_matrix_model->headerData(i, Qt::Horizontal).toString()+"\t";
+		tr = p.boundingRect(tr, Qt::AlignCenter, cell_text);
+		p.drawLine(right, height, right, height+tr.height());
+
+		br.setTopLeft(QPoint(right,height));
+		br.setWidth(vertHeaderWidth);
+		br.setHeight(tr.height());
+		p.drawText(br,Qt::AlignCenter,cell_text,-1);
+		right += vertHeaderWidth;
+		p.drawLine(right, height, right, height+tr.height());
+
+		for(int j=0; j<cols; j++){
+			int w = d_table_view->columnWidth (j);
+			cell_text = text(i,j)+"\t";
+			tr = p.boundingRect(tr,Qt::AlignCenter,cell_text);
+			br.setTopLeft(QPoint(right,height));
+			br.setWidth(w);
+			br.setHeight(tr.height());
+			p.drawText(br, Qt::AlignCenter, cell_text, -1);
+			right += w;
+			p.drawLine(right, height, right, height+tr.height());
+
+			if (right >= printer->width()-2*margin )
+				break;
+		}
+		height += br.height();
+		p.drawLine(margin, height, right-1, height);
+
+		if (height >= printer->height()-margin ){
+			printer->newPage();
+			height = margin;
+			p.drawLine(margin, height, right, height);
+		}
+	}
+	p.end();
+}
+
 void Matrix::print(const QString& fileName)
 {
 	QPrinter printer;
@@ -891,95 +1049,12 @@ void Matrix::print(const QString& fileName)
         if (printDialog.exec() != QDialog::Accepted)
             return;
     }
-		printer.setFullPage( true );
-		QPainter p;
-		if ( !p.begin(&printer ) )
-			return; // paint on printer
-		int dpiy = printer.logicalDpiY();
-		const int margin = (int) ( (1/2.54)*dpiy ); // 1 cm margins
-
-        if (d_view_type == ImageView){
-            p.drawImage (printer.pageRect(), d_matrix_model->renderImage());
-            return;
-        }
-
-		QHeaderView *vHeader = d_table_view->verticalHeader();
-
-		int rows = numRows();
-		int cols = numCols();
-		int height = margin;
-		int i, vertHeaderWidth = vHeader->width();
-		int right = margin + vertHeaderWidth;
-
-		// print header
-		p.setFont(QFont());
-		QString header_label = d_matrix_model->headerData(0, Qt::Horizontal).toString();
-		QRect br = p.boundingRect(br, Qt::AlignCenter, header_label);
-		p.drawLine(right, height, right, height+br.height());
-		QRect tr(br);
-
-		for(i=0; i<cols; i++){
-			int w = d_table_view->columnWidth(i);
-			tr.setTopLeft(QPoint(right,height));
-			tr.setWidth(w);
-			tr.setHeight(br.height());
-			header_label = d_matrix_model->headerData(i, Qt::Horizontal).toString();
-			p.drawText(tr, Qt::AlignCenter, header_label,-1);
-			right += w;
-			p.drawLine(right, height, right, height+tr.height());
-
-			if (right >= printer.width()-2*margin )
-				break;
-		}
-
-		p.drawLine(margin + vertHeaderWidth, height, right-1, height);//first horizontal line
-		height += tr.height();
-		p.drawLine(margin, height, right-1, height);
-
-		// print table values
-		for(i=0;i<rows;i++){
-			right = margin;
-			QString cell_text = d_matrix_model->headerData(i, Qt::Horizontal).toString()+"\t";
-			tr = p.boundingRect(tr, Qt::AlignCenter, cell_text);
-			p.drawLine(right, height, right, height+tr.height());
-
-			br.setTopLeft(QPoint(right,height));
-			br.setWidth(vertHeaderWidth);
-			br.setHeight(tr.height());
-			p.drawText(br,Qt::AlignCenter,cell_text,-1);
-			right += vertHeaderWidth;
-			p.drawLine(right, height, right, height+tr.height());
-
-			for(int j=0; j<cols; j++){
-				int w = d_table_view->columnWidth (j);
-				cell_text = text(i,j)+"\t";
-				tr = p.boundingRect(tr,Qt::AlignCenter,cell_text);
-				br.setTopLeft(QPoint(right,height));
-				br.setWidth(w);
-				br.setHeight(tr.height());
-				p.drawText(br, Qt::AlignCenter, cell_text, -1);
-				right += w;
-				p.drawLine(right, height, right, height+tr.height());
-
-				if (right >= printer.width()-2*margin )
-					break;
-			}
-			height += br.height();
-			p.drawLine(margin, height, right-1, height);
-
-			if (height >= printer.height()-margin ){
-				printer.newPage();
-				height = margin;
-				p.drawLine(margin, height, right, height);
-			}
-		}
+	printer.setFullPage( true );
+	print(&printer);
 }
 
 void Matrix::exportVector(const QString& fileName, int res, bool color)
 {
-    if (d_view_type != ImageView)
-        return;
-
 	if ( fileName.isEmpty() ){
 		QMessageBox::critical(this, tr("QtiPlot - Error"), tr("Please provide a valid file name!"));
         return;
@@ -1012,6 +1087,19 @@ void Matrix::exportVector(const QString& fileName, int res, bool color)
     paint.end();
 }
 
+#ifdef EMF_OUTPUT
+void Matrix::exportEMF(const QString& fileName)
+{
+	int width = numRows();
+	int height = numCols();
+
+	EmfPaintDevice emf(QSize(width, height), fileName);
+	QPainter p(&emf);
+	p.drawImage (QRect(0, 0, width, height), d_matrix_model->renderImage());
+	p.end();
+}
+#endif
+
 void Matrix::range(double *min, double *max)
 {
 	double d_min = cell(0, 0);
@@ -1181,6 +1269,7 @@ void Matrix::initImageView()
     imageLabel->setBackgroundRole(QPalette::Base);
     imageLabel->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
     imageLabel->setScaledContents(true);
+    imageLabel->installEventFilter(this);
     d_stack->addWidget(imageLabel);
 }
 
@@ -1201,7 +1290,7 @@ void Matrix::initTableView()
 	// set header properties
 	QHeaderView* hHeader = (QHeaderView*)d_table_view->horizontalHeader();
 	hHeader->setMovable(false);
-	hHeader->setResizeMode(QHeaderView::Fixed);
+	hHeader->setResizeMode(QHeaderView::Interactive);
 	hHeader->setDefaultSectionSize(d_column_width);
 
     int cols = numCols();
@@ -1235,11 +1324,13 @@ void Matrix::importImage(const QString& fn)
     	d_undo_stack->push(new MatrixSetImageCommand(d_matrix_model, image, d_view_type, 0,
 							numRows() - 1, 0, numCols() - 1, buffer, tr("Import Image") + " \"" + fn + "\""));
 		emit modifiedWindow(this);
+		modifiedData(this);
 	} else if (ignoreUndo()){
 		d_matrix_model->setImage(image);
 		setViewType(ImageView, false);
 		displayImage(image);
 		emit modifiedWindow(this);
+		modifiedData(this);
 	}
 	setWindowLabel(fn);
 }
@@ -1254,14 +1345,25 @@ void Matrix::importImage(const QImage& image)
     	d_undo_stack->push(new MatrixSetImageCommand(d_matrix_model, image, d_view_type, 0,
 							numRows() - 1, 0, numCols() - 1, buffer, tr("Import Image")));
 		emit modifiedWindow(this);
+		modifiedData(this);
 	} else if (ignoreUndo()){
 		d_matrix_model->setImage(image);
 		setViewType(ImageView, false);
 		displayImage(image);
 		emit modifiedWindow(this);
+		modifiedData(this);
 	}
 }
 
+void Matrix::setDefaultColorMap()
+{
+	d_color_map_type = Default;
+	d_color_map = applicationWindow()->d_3D_color_map;
+	if (d_view_type == ImageView)
+		displayImage(d_matrix_model->renderImage());
+	emit modifiedWindow(this);
+}
+
 void Matrix::setGrayScale()
 {
     d_color_map_type = GrayScale;
@@ -1376,32 +1478,29 @@ void Matrix::fft(bool inverse)
     	d_undo_stack->push(new MatrixFftCommand(inverse, d_matrix_model, 0, numRows() - 1,
 							0, numCols() - 1, buffer, commandText));
 		emit modifiedWindow(this);
+		modifiedData(this);
 	} else if (ignoreUndo()){
 		d_matrix_model->fft(inverse);
 		emit modifiedWindow(this);
+		modifiedData(this);
 	}
 }
 
-bool Matrix::exportASCII(const QString& fname, const QString& separator, bool exportSelection)
+bool Matrix::exportODF(const QString& fname, bool exportSelection)
 {
-	QFile f(fname);
-	if ( !f.open( QIODevice::WriteOnly ) ){
-		QApplication::restoreOverrideCursor();
-		QMessageBox::critical(this, tr("QtiPlot - ASCII Export Error"),
-				tr("Could not write to file: <br><h4>%1</h4><p>Please verify that you have the right to write to this location!").arg(fname));
-		return false;
-	}
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
+	int topRow = 0;
+	int bottomRow = numRows() - 1;
+	int leftCol = 0;
+	int rightCol = numCols() - 1;
 
-	int rows = numRows();
-	int cols = numCols();
-	QTextStream t( &f );
-	QString eol = applicationWindow()->endOfLine();
 	if (exportSelection && d_view_type == TableView){
         QModelIndexList selectedIndexes = d_table_view->selectionModel()->selectedIndexes();
-        int topRow = selectedIndexes[0].row();
-        int bottomRow = topRow;
-        int leftCol = selectedIndexes[0].column();
-        int rightCol = leftCol;
+        topRow = selectedIndexes[0].row();
+        bottomRow = topRow;
+        leftCol = selectedIndexes[0].column();
+        rightCol = leftCol;
         foreach(QModelIndex index, selectedIndexes){
             int row = index.row();
             if (row < topRow)
@@ -1415,26 +1514,110 @@ bool Matrix::exportASCII(const QString& fname, const QString& separator, bool ex
             if (col > rightCol)
                 rightCol = col;
         }
+	}
 
-		for (int i = topRow; i <= bottomRow; i++){
-			for (int j = leftCol; j < rightCol; j++){
-				t << d_matrix_model->text(i, j);
-				t << separator;
-			}
-			t << d_matrix_model->text(i, rightCol);
-			t << eol;
-		}
-	} else {
-		for (int i=0; i<rows; i++) {
-			for (int j=0; j<cols-1; j++){
-				t << d_matrix_model->text(i,j);
-				t << separator;
-			}
-			t << d_matrix_model->text(i, cols-1);
-			t << eol;
+	QTextDocument *document = new QTextDocument();
+	QTextCursor cursor = QTextCursor(document);
+
+	QTextTableFormat tableFormat;
+	tableFormat.setAlignment(Qt::AlignCenter);
+	tableFormat.setCellPadding(1);
+	tableFormat.setHeaderRowCount(0);
+	tableFormat.setBorder (1);
+	tableFormat.setBorderStyle(QTextFrameFormat::BorderStyle_Solid);
+	tableFormat.setWidth(QTextLength(QTextLength::PercentageLength, 100));
+
+	cursor.insertTable(abs(bottomRow - topRow) + 1, abs(rightCol - leftCol) + 1, tableFormat);
+
+	for (int i = topRow; i <= bottomRow; i++){
+		for (int j = leftCol; j <= rightCol; j++){
+			cursor.insertText(d_matrix_model->text(i, j));
+			cursor.movePosition(QTextCursor::NextCell);
 		}
 	}
+
+	QTextDocumentWriter writer(fname);
+	if (fname.endsWith(".html"))
+		writer.setFormat("HTML");
+	writer.write(document);
+
+	QApplication::restoreOverrideCursor();
+	return true;
+}
+
+bool Matrix::exportASCII(const QString& fname, const QString& separator, bool exportSelection)
+{
+	QFile f(fname);
+	if ( !f.open( QIODevice::WriteOnly ) ){
+		QMessageBox::critical(this, tr("QtiPlot - ASCII Export Error"),
+				tr("Could not write to file: <br><h4>%1</h4><p>Please verify that you have the right to write to this location!").arg(fname));
+		return false;
+	}
+
+	if (fname.endsWith(".odf") || fname.endsWith(".html")){
+		f.close();
+		return exportODF(fname, exportSelection);
+	}
+
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
+	int topRow = 0;
+	int bottomRow = numRows() - 1;
+	int leftCol = 0;
+	int rightCol = numCols() - 1;
+
+	QTextStream t( &f );
+	QString eol = applicationWindow()->endOfLine();
+	QString sep = separator;
+
+	bool exportTex = (fname.endsWith(".tex")) ? true : false;
+	if (exportTex){
+		t << "\\begin{tabular}{|";
+		eol = "\\\\\\hline\n";
+		sep = " & ";
+	}
+
+	if (exportSelection && d_view_type == TableView){
+        QModelIndexList selectedIndexes = d_table_view->selectionModel()->selectedIndexes();
+        topRow = selectedIndexes[0].row();
+        bottomRow = topRow;
+        leftCol = selectedIndexes[0].column();
+        rightCol = leftCol;
+        foreach(QModelIndex index, selectedIndexes){
+            int row = index.row();
+            if (row < topRow)
+                topRow = row;
+            if (row > bottomRow)
+                bottomRow = row;
+
+            int col = index.column();
+            if (col < leftCol)
+                leftCol = col;
+            if (col > rightCol)
+                rightCol = col;
+        }
+	}
+
+	if (exportTex){
+		for (int i = leftCol; i <= rightCol; i++)
+			t << "c|";
+		t << "}\\hline\n";
+	}
+
+	for (int i = topRow; i <= bottomRow; i++){
+		for (int j = leftCol; j < rightCol; j++){
+			t << d_matrix_model->text(i, j);
+			t << sep;
+		}
+		t << d_matrix_model->text(i, rightCol);
+		t << eol;
+	}
+
+	if (exportTex)
+		t << "\\end{tabular}\n";
+
 	f.close();
+	QApplication::restoreOverrideCursor();
 	return true;
 }
 
@@ -1449,10 +1632,12 @@ void Matrix::importASCII(const QString &fname, const QString &sep, int ignoredLi
 							d_matrix_model, 0, numRows() - 1, 0, numCols() - 1, buffer,
 							tr("Import ASCII File") + " \"" + fname + "\""));
 		emit modifiedWindow(this);
+		modifiedData(this);
 	} else if (ignoreUndo()){
 		d_matrix_model->importASCII(fname, sep, ignoredLines, stripSpaces, simplifySpaces,
 									commentString, importAs, locale, endLineChar, maxRows);
 		emit modifiedWindow(this);
+		modifiedData(this);
 	}
 }
 
@@ -1481,6 +1666,26 @@ QString Matrix::sizeToString()
 	return QString::number((sizeof(Matrix) + size*sizeof(double))/1024.0, 'f', 1) + " " + tr("kB");
 }
 
+bool Matrix::eventFilter(QObject *object, QEvent *e)
+{
+	if (object != (QObject*)imageLabel)
+		return MdiSubWindow::eventFilter(object, e);
+
+	if (e->type() == QEvent::MouseButtonPress && object == (QObject*)imageLabel){
+		const QMouseEvent *me = (const QMouseEvent *)e;
+		if (me->button() == Qt::LeftButton){
+			QDrag *drag = new QDrag(this);
+			QMimeData *mimeData = new QMimeData;
+			mimeData->setText(objectName());
+			drag->setMimeData(mimeData);
+			drag->setPixmap(this->windowIcon().pixmap(16));
+			drag->exec();
+		}
+	}
+
+	return QObject::eventFilter(object, e);
+}
+
 Matrix::~Matrix()
 {
     delete d_undo_stack;
