@@ -2,11 +2,8 @@
     File                 : Matrix.h
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2006 by Ion Vasilief,
-                           Tilman Hoener zu Siederdissen
-                           Knut Franke
-    Email (use @ for *)  : ion_vasilief*yahoo.fr, thzs*gmx.net,
-                           knut.franke*gmx.de
+    Copyright            : (C) 2006 - 2009 by Ion Vasilief
+    Email (use @ for *)  : ion_vasilief*yahoo.fr
     Description          : Matrix worksheet class
 
  ***************************************************************************/
@@ -38,9 +35,9 @@
 #include <QMessageBox>
 
 #include "MatrixModel.h"
-#include "../MdiSubWindow.h"
-#include "../ScriptingEnv.h"
-#include "../Script.h"
+#include <MdiSubWindow.h>
+#include <ScriptingEnv.h>
+#include <Script.h>
 
 #include <qwt_double_rect.h>
 #include <qwt_color_map.h>
@@ -83,7 +80,7 @@ public:
 				  RotateCounterClockwise, FFT, Clear, Calculate, MuParserCalculate, SetImage, ImportAscii};
     enum HeaderViewType{ColumnRow, XY};
 	enum ViewType{TableView, ImageView};
-	enum ColorMapType{GrayScale, Rainbow, Custom};
+	enum ColorMapType{Default, GrayScale, Rainbow, Custom};
 	enum ImportMode {
 		NewColumns, //!< add file as new columns to the current matrix
 		NewRows, //!< add file as new rows to the current matrix
@@ -100,10 +97,13 @@ public:
 	void displayImage(const QImage& image);
 	void importImage(const QString& fn);
 	void importImage(const QImage& image);
-	void exportRasterImage(const QString& fileName, int quality = 100);
+	void exportRasterImage(const QString& fileName, int quality = 100, int dpi = 0);
 	void exportSVG(const QString& fileName);
 	void exportToFile(const QString& fileName);
 	void exportVector(const QString& fileName, int res = 0, bool color = true);
+#ifdef EMF_OUTPUT
+	void exportEMF(const QString& fileName);
+#endif
 
 	MatrixModel * matrixModel(){return d_matrix_model;};
     QUndoStack *undoStack(){return d_undo_stack;};
@@ -137,10 +137,12 @@ public:
 	void setColorMapType(ColorMapType mapType);
 
 	QwtLinearColorMap colorMap(){return d_color_map;};
+	QwtLinearColorMap *colorMapPointer(){return &d_color_map;};
 	void setColorMap(const QwtLinearColorMap& map);
 	//! Used when restoring from project files
 	void setColorMap(const QStringList& lst);
-
+	//! Resets the color map to the one defined by the user in the Preferences dialog (3D plots tab)
+	void setDefaultColorMap();
 	void setGrayScale();
 	void setRainbowColorMap();
 	//! Calculate the volume integral
@@ -153,10 +155,11 @@ public:
 	void invert();
 
 	//! Calculate matrix values using the #formula_str.
-	bool calculate(int startRow = 0, int endRow = -1, int startCol = 0, int endCol = -1, bool forceMuParser = true);
+	bool calculate(int startRow = 0, int endRow = -1, int startCol = 0, int endCol = -1, bool forceMuParser = false);
 	//! Calculate matrix values using the #formula_str (optimization for muParser).
 	bool muParserCalculate(int startRow = 0, int endRow = -1, int startCol = 0, int endCol = -1);
 
+	bool exportODF(const QString& fname, bool exportSelection);
 	bool exportASCII(const QString& fname, const QString& separator, bool exportSelection);
 	void importASCII(const QString &fname, const QString &sep, int ignoredLines, bool stripSpaces,
 					bool simplifySpaces, const QString& commentString, ImportMode importAs = Overwrite,
@@ -168,6 +171,7 @@ public slots:
 	void exportPDF(const QString& fileName);
 	//! Print the Matrix
 	void print();
+	void print(QPrinter *);
 	//! Print the Matrix to fileName
 	void print(const QString& fileName);
 
@@ -206,7 +210,7 @@ public slots:
 	 *
 	 * \sa precision(), setNumericFormat(), setTextFormat()
 	 */
-	void setNumericPrecision(int prec){num_precision = prec;};
+	void setNumericPrecision(int prec);
 
 	/*!
 	 * \brief Set the number format for the cells
@@ -294,8 +298,14 @@ public slots:
 	void freeWorkspace(){free(d_workspace); d_workspace = NULL;};
 
 	bool canCalculate(bool useMuParser = true);
+	void notifyModifiedData(){emit modifiedData(this);};
+
+signals:
+	void modifiedData(Matrix *);
 
 private:
+	bool eventFilter(QObject *, QEvent *);
+
 	//! Initialize the matrix
 	void initTable(int rows, int cols);
 	void initImage(const QImage& image);
