@@ -33,9 +33,9 @@
 #include "Matrix.h"
 #include "MatrixModel.h"
 #include "MatrixCommand.h"
-#include "../muParserScript.h"
-#include "../ScriptingEnv.h"
-#include "../analysis/fft2D.h"
+#include <muParserScript.h>
+#include <ScriptingEnv.h>
+#include <fft2D.h>
 
 #include <gsl/gsl_math.h>
 #include <gsl/gsl_linalg.h>
@@ -49,7 +49,7 @@ MatrixModel::MatrixModel(int rows, int cols, QObject *parent)
 	 d_matrix((Matrix*)parent)
 {
 	init();
-	
+
 	if (d_matrix){
 		d_txt_format = d_matrix->textFormat().toAscii();
 		d_num_precision = d_matrix->precision();
@@ -82,7 +82,7 @@ void MatrixModel::init()
 	d_direct_matrix = NULL;
 	d_inv_matrix = NULL;
 	d_inv_perm = NULL;
-	
+
 	d_rows = 1;
 	d_cols = 1;
 	d_data_block_size = QSize(1, 1);
@@ -170,17 +170,16 @@ void MatrixModel::setDimensions(int rows, int cols)
 double MatrixModel::cell(int row, int col)
 {
     int i = d_cols*row + col;
-    double val = d_data[i];
-    if (i < 0 || i>= d_rows*d_cols || gsl_isnan (val))
-        return 0.0;
+    if (i < 0 || i >= d_rows*d_cols)
+        return GSL_NAN;
 
-	return val;
+	return d_data[i];
 }
 
 void MatrixModel::setCell(int row, int col, double val)
 {
 	int i = d_cols*row + col;
-    if (i < 0 || i>= d_rows*d_cols)
+    if (i < 0 || i >= d_rows*d_cols)
 		return;
 
     d_data[i] = val;
@@ -190,7 +189,7 @@ QString MatrixModel::text(int row, int col)
 {
 	int i = d_cols*row + col;
 	double val = d_data[i];
-    if (i < 0 || i>= d_rows*d_cols || gsl_isnan(val))
+    if (i < 0 || i >= d_rows*d_cols || gsl_isnan(val))
         return "";
 
 	if (d_matrix){
@@ -320,8 +319,18 @@ bool MatrixModel::setData(const QModelIndex & index, const QVariant & value, int
  	if(role == Qt::EditRole){
 		if (value.toString().isEmpty())
 			d_data[i] = GSL_NAN;
-		else
-			d_data[i] = value.toDouble();
+		else {
+			double val = GSL_NAN;
+			if (d_matrix)
+				val = d_matrix->locale().toDouble(value.toString());
+			else
+				val = d_locale.toDouble(value.toString());
+
+			if (val != valBefore)
+				d_data[i] = val;
+			else
+				return false;
+		}
 	}
 
 	if(index.row() + 1 >= d_rows){
@@ -333,6 +342,7 @@ bool MatrixModel::setData(const QModelIndex & index, const QVariant & value, int
                                 tr("Edited cell") + " (" + QString::number(index.row() + 1) + "," +
                                 QString::number(index.column() + 1) + ")"));
     d_matrix->notifyChanges();
+    d_matrix->notifyModifiedData();
 	d_matrix->moveCell(index);
 	return false;
 }
@@ -346,7 +356,7 @@ bool MatrixModel::canResize(int rows, int cols)
 		tr("Please enter positive values for which the product rows*columns does not exceed the maximum integer value available on your system!"));
 		return false;
 	}
-	
+
     if (d_data_block_size.width()*d_data_block_size.height() >= rows*cols)
 		return true;
 
@@ -447,41 +457,10 @@ bool MatrixModel::removeRows(int row, int count, const QModelIndex & parent)
 	return true;
 }
 
-QString MatrixModel::saveToString()
-{
-	QString s = "<data>\n";
-	int cols = d_cols - 1;
-	for(int i = 0; i < d_rows; i++){
-		int aux = d_cols*i;
-		bool emptyRow = true;
-		for(int j = 0; j < d_cols; j++){
-			if (gsl_finite(d_data[aux + j])){
-				emptyRow = false;
-				break;
-			}
-		}
-		if (emptyRow)
-			continue;
-
-		s += QString::number(i) + "\t";
-		for(int j = 0; j < cols; j++){
-			double val = d_data[aux + j];
-			if (gsl_finite(val))
-				s += QString::number(val, 'e', 16);
-			s += "\t";
-		}
-		double val = d_data[aux + cols];
-		if (gsl_finite(val))
-			s += QString::number(val, 'e', 16);
-		s += "\n";
-	}
-	return s + "</data>\n";
-}
-
 QImage MatrixModel::renderImage()
 {
     QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
-	
+
 	QImage image(QSize(d_cols, d_rows), QImage::Format_RGB32);
 	QwtLinearColorMap color_map = d_matrix->colorMap();
 
@@ -796,11 +775,15 @@ double * MatrixModel::dataCopy(int startRow, int endRow, int startCol, int endCo
 
 bool MatrixModel::muParserCalculate(int startRow, int endRow, int startCol, int endCol)
 {
-	ScriptingEnv *scriptEnv = d_matrix->scriptingEnv();
-    muParserScript *mup = new muParserScript(scriptEnv, d_matrix->formula(), d_matrix, QString("<%1>").arg(d_matrix->objectName()));
-	connect(mup, SIGNAL(error(const QString&,const QString&,int)), scriptEnv, SIGNAL(error(const QString&,const QString&,int)));
-	connect(mup, SIGNAL(print(const QString&)), scriptEnv, SIGNAL(print(const QString&)));
+	if (d_matrix->formula().count("\n") > 0){
+        QString mess = tr("Multiline expressions take much more time to evaluate! Do you want to continue anyways?");
+        if (QMessageBox::Yes != QMessageBox::warning(matrix(), tr("QtiPlot") + " - " + tr("Warning"), mess,
+                           QMessageBox::Yes, QMessageBox::Cancel))
+			return false;
+	}
 
+    muParserScript *mup = new muParserScript(d_matrix->scriptingEnv(), d_matrix->formula(),
+											d_matrix, QString("<%1>").arg(d_matrix->objectName()));
 	if (endRow < 0)
 		endRow = d_rows - 1;
 	if (endCol < 0)
@@ -812,6 +795,9 @@ bool MatrixModel::muParserCalculate(int startRow, int endRow, int startCol, int
 
 	double dx = d_matrix->dx();
 	double dy = d_matrix->dy();
+	double x_start = d_matrix->xStart();
+	double y_start = d_matrix->yStart();
+
     double *ri = mup->defineVariable("i");
     double *rr = mup->defineVariable("row");
     double *cj = mup->defineVariable("j");
@@ -819,6 +805,12 @@ bool MatrixModel::muParserCalculate(int startRow, int endRow, int startCol, int
     double *x = mup->defineVariable("x");
     double *y = mup->defineVariable("y");
 
+	double r = 1.0, c = 1.0;
+	*ri = r; *rr = r;
+	*y = y_start + dy;
+	*cj = c; *cc = c;
+	*x = x_start + dx;
+
 	if (!mup->compile()){
 		QApplication::restoreOverrideCursor();
 		return false;
@@ -826,8 +818,6 @@ bool MatrixModel::muParserCalculate(int startRow, int endRow, int startCol, int
 
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
-	double x_start = d_matrix->xStart();
-	double y_start = d_matrix->yStart();
     if (mup->codeLines() == 1){
         for(int row = startRow; row <= endRow; row++){
             double r = row + 1.0;
@@ -874,10 +864,10 @@ bool MatrixModel::calculate(int startRow, int endRow, int startCol, int endCol)
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
 	ScriptingEnv *scriptEnv = d_matrix->scriptingEnv();
-	Script *script = scriptEnv->newScript(formula, this, QString("<%1>").arg(objectName()));
-	connect(script, SIGNAL(error(const QString&,const QString&,int)), scriptEnv, SIGNAL(error(const QString&,const QString&,int)));
+	Script *script = scriptEnv->newScript(formula, d_matrix, QString("<%1>").arg(objectName()));
+	connect(script, SIGNAL(error(const QString&, const QString&, int)), scriptEnv, SIGNAL(error(const QString&, const QString&, int)));
 	connect(script, SIGNAL(print(const QString&)), scriptEnv, SIGNAL(print(const QString&)));
-	
+
 	if (!script->compile()){
 		QApplication::restoreOverrideCursor();
 		return false;
@@ -910,6 +900,7 @@ bool MatrixModel::calculate(int startRow, int endRow, int startCol, int endCol)
 			script->setDouble(c, "col");
 			script->setDouble(x_start + col*dx, "x");
 			res = script->eval();
+
 			if (res.canConvert(QVariant::Double))
 				d_data[aux++] = res.toDouble();
 			else {
@@ -993,11 +984,11 @@ void MatrixModel::pasteData(double *clipboardBuffer, int topRow, int leftCol, in
 	int newCols = leftCol + cols;
 	if (newCols > d_cols)
 		insertColumns(d_cols, newCols - d_cols);
-	
+
 	int newRows = topRow + rows;
 	if (newRows > d_rows)
 		insertRows(d_rows, newRows - d_rows);
-		
+
 	int cell = 0;
 	int bottomRow = newRows - 1;
 	int rightCol = newCols - 1;
