@@ -26,8 +26,11 @@
  *   Boston, MA  02110-1301  USA                                           *
  *                                                                         *
  ***************************************************************************/
-#include "ArrowMarker.h"
-#include "dialogs/LineDialog.h"
+#include <ArrowMarker.h>
+#include <Graph.h>
+#include <MultiLayer.h>
+#include <LineDialog.h>
+#include <ApplicationWindow.h>
 
 #include <QPainter>
 #include <QMouseEvent>
@@ -48,7 +51,8 @@ ArrowMarker::ArrowMarker():
 		d_head_length(4),
 		d_rect(0, 0, 1, 1),
 		d_editable(false),
-		d_op(None)
+		d_op(None),
+		d_attach_policy(Scales)
 {
 }
 
@@ -60,58 +64,65 @@ void ArrowMarker::draw(QPainter *p, const QwtScaleMap &xMap, const QwtScaleMap &
 	const int y1 = yMap.transform(d_rect.bottom());
 
 	p->save();
-	QPen pen = linePen();
+	QPen pen = QwtPainter::scaledPen(linePen());
+	pen.setCapStyle(Qt::FlatCap);
+	pen.setJoinStyle(Qt::MiterJoin);
 	p->setPen(pen);
 
 	QBrush brush = QBrush(pen.color(), Qt::SolidPattern);
-	QwtPainter::drawLine(p,x0,y0,x1,y1);
+	QwtPainter::drawLine(p, x0, y0, x1, y1);
 	p->restore();
 
+	int headLength = qRound(d_head_length*(double)p->device()->logicalDpiX()/(double)plot()->logicalDpiX());
+
 	if (d_end_arrow){
 		p->save();
-		p->translate(x1,y1);
+		p->setClipping(false);
+		p->translate(x1, y1);
 		const double t = theta(x0, y0, x1, y1);
 		p->rotate(-t);
 
 		QPolygon endArray(3);
-		endArray[0] = QPoint(0,0);
+		endArray[0] = QPoint(0, 0);
 
-		int d=(int)floor(d_head_length*tan(M_PI*d_head_angle/180.0)+0.5);
-		endArray[1] = QPoint(-d_head_length,d);
-		endArray[2] = QPoint(-d_head_length,-d);
+		int d = qRound(headLength*tan(M_PI*d_head_angle/180.0) + 0.5);
+		endArray[1] = QPoint(-headLength, d);
+		endArray[2] = QPoint(-headLength, -d);
 
-		p->setPen(QPen(pen.color(), pen.width(), Qt::SolidLine));
+		pen.setStyle(Qt::SolidLine);
+		p->setPen(pen);
 		if (d_fill_head)
 			p->setBrush(brush);
 
-		QwtPainter::drawPolygon(p,endArray);
+		QwtPainter::drawPolygon(p, endArray);
 		p->restore();
     }
 
 	if (d_start_arrow){
 		p->save();
-		p->translate(x0,y0);
+		p->translate(x0, y0);
 		const double t = theta(x0, y0, x1, y1);
 		p->rotate(-t);
 
 		QPolygon startArray(3);
-		startArray[0] = QPoint(0,0);
+		startArray[0] = QPoint(0, 0);
 
-		int d=(int)floor(d_head_length*tan(M_PI*d_head_angle/180.0)+0.5);
-		startArray[1] = QPoint(d_head_length,d);
-		startArray[2] = QPoint(d_head_length,-d);
+		int d = qRound(headLength*tan(M_PI*d_head_angle/180.0) + 0.5);
+		startArray[1] = QPoint(headLength, d);
+		startArray[2] = QPoint(headLength, -d);
 
-		p->setPen(QPen(pen.color(), pen.width(), Qt::SolidLine));
+		pen.setStyle(Qt::SolidLine);
+		p->setPen(pen);
 		if (d_fill_head)
 			p->setBrush(brush);
-		QwtPainter::drawPolygon(p,startArray);
+		QwtPainter::drawPolygon(p, startArray);
 		p->restore();
     }
 
-	if (d_editable) {
+	if (d_editable){
 		p->save();
-		p->setPen(QPen(Qt::black,1,Qt::SolidLine));
-		QRect handler(QPoint(0,0), QSize(10,10));
+		p->setPen(QPen(Qt::black, 1, Qt::SolidLine));
+		QRect handler(QPoint(0, 0), QSize(10, 10));
 		handler.moveCenter(startPoint());
 		p->fillRect(handler, QBrush(Qt::black));
 		handler.moveCenter(endPoint());
@@ -366,24 +377,31 @@ QwtDoubleRect ArrowMarker::boundingRect() const
 
 void ArrowMarker::updateBoundingRect()
 {
-const QwtScaleMap &xMap = plot()->canvasMap(xAxis());
-const QwtScaleMap &yMap = plot()->canvasMap(yAxis());
+	if (d_attach_policy == Scales)
+		return;
 
-d_rect.setLeft(xMap.invTransform(d_start.x()));
-d_rect.setTop(yMap.invTransform(d_start.y()));
-d_rect.setRight(xMap.invTransform(d_end.x()));
-d_rect.setBottom(yMap.invTransform(d_end.y()));
+	const QwtScaleMap &xMap = plot()->canvasMap(xAxis());
+	const QwtScaleMap &yMap = plot()->canvasMap(yAxis());
+
+	d_rect.setLeft(xMap.invTransform(d_start.x()));
+	d_rect.setTop(yMap.invTransform(d_start.y()));
+	d_rect.setRight(xMap.invTransform(d_end.x()));
+	d_rect.setBottom(yMap.invTransform(d_end.y()));
 }
 
 void ArrowMarker::setEditable(bool yes)
 {
 	if (yes == d_editable)
 		return;
+
+	displayInfo(!yes);
+
 	if (yes) {
 		d_editable = true;
 		plot()->canvas()->installEventFilter(this);
 	} else {
 		d_editable = false;
+
 		plot()->canvas()->removeEventFilter(this);
 	}
 	plot()->replot();
@@ -446,18 +464,21 @@ bool ArrowMarker::eventFilter(QObject *, QEvent *e)
 		case QEvent::MouseButtonRelease:
 			{
 				const QMouseEvent *me = (const QMouseEvent *)e;
+
 				switch(d_op) {
 					case MoveStart:
 						setStartPoint(me->pos());
 						plot()->replot();
 						d_op = None;
 						QApplication::restoreOverrideCursor();
+						displayInfo();
 						return true;
 					case MoveEnd:
 						setEndPoint(me->pos());
 						plot()->replot();
 						d_op = None;
 						QApplication::restoreOverrideCursor();
+						displayInfo();
 						return true;
 					case MoveBoth:
 						setXValue(plot()->invTransform(xAxis(), me->pos().x()-d_op_startat.x()));
@@ -465,6 +486,7 @@ bool ArrowMarker::eventFilter(QObject *, QEvent *e)
 						plot()->replot();
 						d_op = None;
 						QApplication::restoreOverrideCursor();
+						displayInfo();
 						return true;
 					default:
 						d_op = None;
@@ -481,7 +503,71 @@ bool ArrowMarker::eventFilter(QObject *, QEvent *e)
 				ld->exec();
 				return true;
 			}
+		case QEvent::KeyPress:
+			{
+				const QKeyEvent *ke = (const QKeyEvent *)e;
+				if (ke->key() == Qt::Key_Escape){
+					((Graph *)plot())->deselectMarker();
+					return true;
+				}
+			}
+
 		default:
 			return false;
 	}
 }
+
+void ArrowMarker::displayInfo(bool clear)
+{
+	Graph *g = (Graph *)plot();
+	if (!g)
+		return;
+
+	MultiLayer *ml = g->multiLayer();
+	if (!ml)
+		return;
+
+	ApplicationWindow *app = ml->applicationWindow();
+	if (clear){
+		app->displayInfo(QString::null);
+		g->setToolTip(QString::null);
+		return;
+	}
+
+	QLocale locale = app->locale();
+	int prec = app->d_decimal_digits;
+
+	QLineF line(d_rect.topLeft(), d_rect.bottomRight());
+	QString s = tr("dx") + ": " + locale.toString(line.dx(), 'g', prec) + ";";
+	s += " " + tr("dy") + ": " + locale.toString(line.dy(), 'g', prec) + ";";
+	s += " " + tr("angle") + ": " + locale.toString(line.angle(), 'g', prec) + ";";
+	s += " " + tr("length") + ": " + locale.toString(line.length(), 'g', prec) + ";";
+
+	double den = line.x2() - line.x1();
+	if (den != 0.0){
+		double slope = (line.y2() - line.y1())/den;
+		s += " " + tr("eqn") + ": " + tr("y") + "=";
+		if (slope != 0.0){
+			if (slope != 1.0)
+				s += locale.toString(slope, 'g', prec) + "*";
+			s += tr("x");
+		}
+
+		double intercept = line.y1() - slope*line.x1();
+		if (intercept > 0){
+			if (slope != 0.0)
+				s += "+";
+			s += locale.toString(intercept, 'g', prec);
+		} else if (intercept < 0)
+			s += locale.toString(intercept, 'g', prec);
+	} else
+		s += " " + tr("eqn") + ": " + tr("x") + "=" + locale.toString(line.x1(), 'g', prec);
+	app->displayInfo(s);
+	g->setToolTip(s.split("; ").join("\n"));
+}
+
+void ArrowMarker::setAttachPolicy(AttachPolicy attachTo)
+{
+	if (attachTo != d_attach_policy)
+		d_attach_policy = attachTo;
+}
