@@ -2,10 +2,8 @@
     File                 : CanvasPicker.cpp
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2006,2007 by Ion Vasilief,
-                           Tilman Hoener zu Siederdissen, Knut Franke
-    Email (use @ for *)  : ion_vasilief*yahoo.fr, thzs*gmx.net
-                           knut.franke*gmx.de
+    Copyright            : (C) 2006,2007 by Ion Vasilief, Knut Franke
+    Email (use @ for *)  : ion_vasilief*yahoo.fr, knut.franke*gmx.de
     Description          : Canvas picker
 
  ***************************************************************************/
@@ -31,7 +29,9 @@
 #include "CanvasPicker.h"
 #include "ArrowMarker.h"
 #include "PlotCurve.h"
-#include "../ApplicationWindow.h"
+#include <Spectrogram.h>
+#include <ApplicationWindow.h>
+#include <MultiLayer.h>
 
 #include <QVector>
 #include <QMessageBox>
@@ -88,6 +88,19 @@ bool CanvasPicker::eventFilter(QObject *object, QEvent *e)
 					return true;
 				}
 
+				if (me->button() == Qt::LeftButton && !g->zoomOn() &&
+					!g->hasPanningMagnifierEnabled() && !g->activeTool() && !g->selectedCurveLabels()){
+					QDrag *drag = new QDrag(plot());
+					QMimeData *mimeData = new QMimeData;
+					QPoint p = plot()->canvas()->mapToParent(me->pos());
+					mimeData->setText(QString::number(abs(plot()->x() - p.x())) + ";" +
+									QString::number(abs(plot()->y() - p.y())));
+					drag->setMimeData(mimeData);
+					drag->setPixmap(plot()->multiLayer()->windowIcon().pixmap(16));
+					drag->exec();
+					return true;
+				}
+
 				return !(me->modifiers() & Qt::ShiftModifier);
 			}
 			break;
@@ -107,7 +120,7 @@ bool CanvasPicker::eventFilter(QObject *object, QEvent *e)
 				} else {
 					const QMouseEvent *me = (const QMouseEvent *)e;
                     int dist, point;
-                    QwtPlotCurve *c = g->closestCurve(me->pos().x(), me->pos().y(), dist, point);
+                    QwtPlotItem *c = g->closestCurve(me->pos().x(), me->pos().y(), dist, point);
                     if (c && dist < 10)
                         emit showPlotDialog(g->curveIndex(c));
                     else
@@ -125,9 +138,12 @@ bool CanvasPicker::eventFilter(QObject *object, QEvent *e)
 
 				QPoint pos = me->pos();
 
-				DataCurve *c = g->selectedCurveLabels();
+				QwtPlotItem *c = g->selectedCurveLabels();
 				if (c){
-					c->moveLabels(pos);
+					if (c->rtti() == QwtPlotItem::Rtti_PlotSpectrogram)
+						((Spectrogram *)c)->moveLabel(pos);
+					else
+						((DataCurve *)c)->moveLabels(pos);
 					return true;
 				}
 
