@@ -2,8 +2,8 @@
     File                 : FunctionCurve.cpp
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2006 by Ion Vasilief, Tilman Hoener zu Siederdissen
-    Email (use @ for *)  : ion_vasilief*yahoo.fr, thzs*gmx.net
+    Copyright            : (C) 2006-2009 by Ion Vasilief
+    Email (use @ for *)  : ion_vasilief*yahoo.fr
     Description          : Function curve class
 
  ***************************************************************************/
@@ -27,14 +27,17 @@
  *                                                                         *
  ***************************************************************************/
 #include "FunctionCurve.h"
-#include "../MyParser.h"
+#include <Graph.h>
+#include <MyParser.h>
+#include <ScaleEngine.h>
 
 #include <QMessageBox>
 
 FunctionCurve::FunctionCurve(const QString& name):
-	PlotCurve(name)
+	PlotCurve(name),
+	d_function_type(Normal),
+	d_variable("x")
 {
-	d_variable = "x";
 	setType(Graph::Function);
 	setPlotStyle(Graph::Line);
 	d_formulas = QStringList();
@@ -42,9 +45,9 @@ FunctionCurve::FunctionCurve(const QString& name):
 
 FunctionCurve::FunctionCurve(const FunctionType& t, const QString& name):
 	PlotCurve(name),
-	d_function_type(t)
+	d_function_type(t),
+	d_variable("x")
 {
-	d_variable = "x";
 	setType(Graph::Function);
 	setPlotStyle(Graph::Line);
 }
@@ -66,7 +69,7 @@ void FunctionCurve::copy(FunctionCurve* f)
 }
 
 QString FunctionCurve::saveToString()
-{	
+{
 	QString s = "<Function>\n";
 	s += "<Type>" + QString::number(d_function_type) + "</Type>\n";
 	s += "<Title>" + title().text() + "</Title>\n";
@@ -74,6 +77,12 @@ QString FunctionCurve::saveToString()
 	s += "<Variable>" + d_variable + "</Variable>\n";
 	s += "<Range>" + QString::number(d_from,'g',15) + "\t" + QString::number(d_to,'g',15) + "</Range>\n";
 	s += "<Points>" + QString::number(dataSize()) + "</Points>\n";
+
+	ScaleEngine *sc_engine = (ScaleEngine *)plot()->axisScaleEngine(xAxis());
+	if (d_from > 0 && d_to > 0 && sc_engine &&
+		sc_engine->type() == ScaleTransformation::Log10)
+		s += "<Log10>1</Log10>\n";
+
 	QMapIterator<QString, double> i(d_constants);
  	while (i.hasNext()) {
      	i.next();
@@ -88,9 +97,11 @@ void FunctionCurve::restore(Graph *g, const QStringList& lst)
 {
 	if (!g)
 		return;
-	
+
 	int type = 0;
 	int points = 0, style = 0;
+	int logScale = 0;
+	QwtPlotCurve::CurveStyle lineStyle = QwtPlotCurve::NoCurve;
 	QStringList formulas;
 	QString var, title = QString::null;
 	double start = 0.0, end = 0.0;
@@ -114,13 +125,16 @@ void FunctionCurve::restore(Graph *g, const QStringList& lst)
 			}
 		} else if (s.contains("<Points>"))
 			points = s.remove("<Points>").remove("</Points>").stripWhiteSpace().toInt();
+		else if (s.contains("<Log10>"))
+			logScale = s.remove("<Log10>").remove("</Log10>").stripWhiteSpace().toInt();
 		else if (s.contains("<Constant>")){
 			QStringList l = s.remove("<Constant>").remove("</Constant>").split("\t");
 			if (l.size() == 2)
 				constants.insert(l[0], l[1].toDouble());
-		}
-		else if (s.contains("<Style>")){
+		} else if (s.contains("<Style>")){
 			style = s.remove("<Style>").remove("</Style>").stripWhiteSpace().toInt();
+		} else if (s.contains("<LineStyle>")){
+			lineStyle = (QwtPlotCurve::CurveStyle)(s.remove("<LineStyle>").remove("</LineStyle>").stripWhiteSpace().toInt());
 			break;
 		}
 	}
@@ -130,16 +144,15 @@ void FunctionCurve::restore(Graph *g, const QStringList& lst)
 	c->setFormulas(formulas);
 	c->setVariable(var);
 	c->setConstants(constants);
-	c->loadData(points);
+	c->loadData(points, logScale);
 	c->setPlotStyle(style);
-
 	g->insertCurve(c);
+	g->setCurveStyle(g->curveIndex(c), lineStyle);
 
 	QStringList l;
 	for (line++; line != lst.end(); line++)
         l << *line;
 	c->restoreCurveLayout(l);
-
 	g->updatePlot();
 }
 
@@ -161,11 +174,11 @@ QString FunctionCurve::legend()
 	return label;
 }
 
-void FunctionCurve::loadData(int points)
+void FunctionCurve::loadData(int points, bool xLog10Scale)
 {
     if (!points)
         points = dataSize();
-	
+
 	double *X = (double *)malloc(points*sizeof(double));
 	if (!X){
 		QMessageBox::critical(0, "QtiPlot - Memory Allocation Error",
@@ -179,7 +192,7 @@ void FunctionCurve::loadData(int points)
 		free(X);
 		return;
 	}
-	
+
     double step = (d_to - d_from)/(double)(points - 1);
     bool error = false;
 
@@ -189,19 +202,48 @@ void FunctionCurve::loadData(int points)
 		try {
 			parser.DefineVar(d_variable.ascii(), &x);
 			QMapIterator<QString, double> i(d_constants);
- 			while (i.hasNext()) {
+ 			while (i.hasNext()){
      			i.next();
 				parser.DefineConst(i.key().ascii(), i.value());
  			}
 			parser.SetExpr(d_formulas[0].ascii());
 
-			X[0] = d_from; x = d_from; Y[0]=parser.Eval();
-			for (int i = 1; i<points; i++ ){
-				x += step;
-				X[i] = x;
-				Y[i] = parser.Eval();
+			X[0] = d_from;
+			x = d_from;
+			try {
+				Y[0] = parser.EvalRemoveSingularity(&x, false);
+			} catch (MyParser::Pole) {
+			    error = true;
 			}
-		} catch(mu::ParserError &) {
+
+			ScaleEngine *sc_engine = 0;
+			if (plot())
+				sc_engine = (ScaleEngine *)plot()->axisScaleEngine(xAxis());
+
+			if (xLog10Scale || (d_from > 0 && d_to > 0 && sc_engine &&
+				sc_engine->type() == ScaleTransformation::Log10)){
+				step = log10(d_to/d_from)/(double)(points - 1);
+				for (int i = 1; i < points; i++ ){
+					x = d_from*pow(10, i*step);
+					X[i] = x;
+					try {
+						Y[i] = parser.EvalRemoveSingularity(&x, false);
+					} catch (MyParser::Pole){
+						error = true;
+					}
+				}
+			} else {
+				for (int i = 1; i < points; i++ ){
+					x += step;
+					X[i] = x;
+					try {
+						Y[i] = parser.EvalRemoveSingularity(&x, false);
+					} catch (MyParser::Pole){
+						error = true;
+					}
+				}
+			}
+		} catch(mu::ParserError &e) {
 			error = true;
 		}
 	} else if (d_function_type == Parametric || d_function_type == Polar) {
@@ -222,18 +264,15 @@ void FunctionCurve::loadData(int points)
 			yparser.SetExpr(aux[1].ascii());
 			par = d_from;
 			for (int i = 0; i<points; i++ ){
-				X[i]=xparser.Eval();
-				Y[i]=yparser.Eval();
-				par+=step;
+				X[i] = xparser.Eval();
+				Y[i] = yparser.Eval();
+				par += step;
 			}
 		} catch(mu::ParserError &) {
 			error = true;
 		}
 	}
 
-	if (error)
-		return;
-
 	setData(X, Y, points);
 	free(X); free(Y);
 }
