@@ -2,8 +2,8 @@
     File                 : FunctionCurve.h
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2006 by Ion Vasilief, Tilman Hoener zu Siederdissen
-    Email (use @ for *)  : ion_vasilief*yahoo.fr, thzs*gmx.net
+    Copyright            : (C) 2006 by Ion Vasilief
+    Email (use @ for *)  : ion_vasilief*yahoo.fr
     Description          : Function curve class
 
  ***************************************************************************/
@@ -31,6 +31,8 @@
 
 #include "PlotCurve.h"
 
+class Graph;
+
 // Function curve class
 class FunctionCurve: public PlotCurve
 {
@@ -66,8 +68,8 @@ public:
 	//! Returns a string that can be displayed in a plot legend
 	QString legend();
 
-	void loadData(int points = 0);
-	
+	void loadData(int points = 0, bool xLog10Scale = false);
+
 	QMap<QString, double> constants(){return d_constants;};
 	void setConstants(const QMap<QString, double>& map){d_constants = map;};
 	void setConstant(const QString& parName, double val){d_constants.insert(parName, val);};
