@@ -41,14 +41,14 @@
 #include <qwt_plot_marker.h>
 #include <qwt_plot_curve.h>
 
-#include "../Table.h"
-#include "dialogs/AxesDialog.h"
-#include "PlotToolInterface.h"
-#include "MultiLayer.h"
-#include "ScaleDraw.h"
-
+#include <AxesDialog.h>
+#include <PlotToolInterface.h>
+#include <ScaleDraw.h>
+#include <FrameWidget.h>
 #include <float.h>
 
+class QwtPlotPanner;
+class QwtPlotMagnifier;
 class QwtPlotCurve;
 class QwtPlotZoomer;
 class QwtPieCurve;
@@ -74,18 +74,17 @@ class QwtHistogram;
 class Grid;
 class TexWidget;
 class LegendWidget;
-class FrameWidget;
 
 //! Structure containing curve layout parameters
 typedef struct{
-  int lCol;        //!< line color
+  QColor lCol;     //!< line color
   float lWidth;    //!< line width
   int lStyle;      //!< line style
   int filledArea;  //!< flag: toggles area filling under curve
-  int aCol;        //!< curve area color
+  QColor aCol;     //!< curve area color
   int aStyle;      //!< area filling style
-  int symCol;      //!< symbol outline color
-  int fillCol;     //!< symbol fill color
+  QColor symCol;   //!< symbol outline color
+  QColor fillCol;  //!< symbol fill color
   float penWidth;  //!< symbol outline width
   int sSize;       //!< symbol size
   int sType;       //!< symbol type (shape)
@@ -134,7 +133,9 @@ class Graph: public QwtPlot
 		enum MarkerType{None = -1, Text = 0, Arrow = 1, Image = 2};
 		enum CurveType{Line, Scatter, LineSymbols, VerticalBars, Area, Pie, VerticalDropLines,
 			Spline, HorizontalSteps, Histogram, HorizontalBars, VectXYXY, ErrorBars,
-			Box, VectXYAM, VerticalSteps, ColorMap, GrayScale, Contour, Function, ImagePlot};
+			Box, VectXYAM, VerticalSteps, ColorMap, GrayScale, Contour, Function, ImagePlot,
+			StackBar, StackColumn};
+		enum LegendDisplayMode{ColumnName, ColumnComment, TableName, TableLegend};
 
 		//! Returns a pointer to the parent MultiLayer object.
 		MultiLayer *multiLayer();
@@ -143,6 +144,8 @@ class Graph: public QwtPlot
 		void setActiveTool(PlotToolInterface *tool);
 		//! Return the active tool, or NULL if none is active.
 		PlotToolInterface* activeTool() const { return d_active_tool; }
+		//! Returns true if a plot/data tool is enabled.
+		bool hasActiveTool();
 
 		QList <LegendWidget *> textsList();
 		LegendWidget *activeText();
@@ -156,16 +159,14 @@ class Graph: public QwtPlot
 		void deselect();
 		void deselect(QWidget *);
 
-		DataCurve* selectedCurveLabels();
+		QwtPlotItem* selectedCurveLabels();
         //! Used when restoring DataCurve curveID from a project file
         void restoreCurveLabels(int curveID, const QStringList& lst);
-		//! Called first time we add curves in order to determine the best plot limits.
-		void initScaleLimits();
 
 		Grid *grid(){return (Grid *)d_grid;};
 		QList<QwtPlotItem *> curvesList(){return d_curves;};
 
-		PlotCurve* closestCurve(int xpos, int ypos, int &dist, int &point);
+		QwtPlotItem* closestCurve(int xpos, int ypos, int &dist, int &point);
 
 		void insertMarker(QwtPlotMarker *m);
 
@@ -192,6 +193,7 @@ class Graph: public QwtPlot
 
     	void print(QPainter *, const QRect &rect, const QwtPlotPrintFilter & = QwtPlotPrintFilter());
 		void updateLayout();
+		void adjustGeometryToCanvas(const QRect &canvasRect);
 
     	void updateCurveLabels();
 
@@ -202,6 +204,20 @@ class Graph: public QwtPlot
 
 		QRect boundingRect();
 		void raiseEnrichements();
+		void addLegendItem();
+		bool isPrinting(){return d_is_printing;};
+
+		void enablePanningMagnifier(bool on = true);
+		bool hasPanningMagnifierEnabled(){if (d_magnifier && d_panner) return true; return false;}
+
+		static QString escapeTeXSpecialCharacters(const QString &);
+		static QString texSuperscripts(const QString &);
+
+		void changeCurveIndex(int fromIndex, int toIndex);
+		void enableDouglasPeukerSpeedMode(double tolerance, int maxPoints);
+
+		int speedModeMaxPoints(){return d_speed_mode_points;};
+		double getDouglasPeukerTolerance(){return d_Douglas_Peuker_tolerance;};
 
 	public slots:
 		void copy(Graph* g);
@@ -227,9 +243,9 @@ class Graph: public QwtPlot
 		//@}
 
 		bool addCurves(Table* w, const QStringList& names, int style = 0, double lWidth = 1, int sSize = 3, int startRow = 0, int endRow = -1);
-		PlotCurve* insertCurve(Table* w, const QString& name, int style, int startRow = 0, int endRow = -1);
-		PlotCurve* insertCurve(Table* w, int xcol, const QString& name, int style);
-		PlotCurve* insertCurve(Table* w, const QString& xColName, const QString& yColName, int style, int startRow = 0, int endRow = -1);
+		DataCurve* insertCurve(Table* w, const QString& name, int style, int startRow = 0, int endRow = -1);
+		DataCurve* insertCurve(Table* w, int xcol, const QString& name, int style);
+		DataCurve* insertCurve(Table* w, const QString& xColName, const QString& yColName, int style, int startRow = 0, int endRow = -1);
 		void insertPlotItem(QwtPlotItem *i, int type);
 		void insertCurve(QwtPlotItem *c);
 
@@ -257,13 +273,15 @@ class Graph: public QwtPlot
 		double selectedXEndValue();
 
 		//! Map curve pointer to index.
-		int curveIndex(QwtPlotCurve *c){return d_curves.indexOf(c);};
+		int curveIndex(QwtPlotItem *c){return d_curves.indexOf(c);};
 		//! map curve title to index
   	    int curveIndex(const QString &title){return plotItemsList().findIndex(title);}
   	    //! get curve by index
   	    QwtPlotCurve* curve(int index);
   	    //! get curve by name
-  	    QwtPlotCurve* curve(const QString &title){return curve(curveIndex(title));}
+  	    QwtPlotCurve* curve(const QString &title){return curve(curveIndex(title));};
+  	    //! get curve title string by inde (convenience function for scripts)
+		QString curveTitle(int index);
 
 		//! Returns the names of all the curves suitable for data analysis, as a string list. The list excludes error bars and spectrograms.
 		QStringList analysableCurvesList();
@@ -276,8 +294,16 @@ class Graph: public QwtPlot
 
         void updateCurveNames(const QString& oldName, const QString& newName, bool updateTableName = true);
 
+		//! \name Customizing plot curves
+		//@{
 		void setCurveStyle(int index, int s);
 		void setCurveFullRange(int curveIndex);
+		void setCurveLineColor(int curveIndex, int colorIndex);
+		void setCurveLineStyle(int curveIndex, Qt::PenStyle style);
+		void setCurveLineWidth(int curveIndex, double width);
+		void setGrayScale();
+		void setIndexedColors();
+		//@}
 
 		//! \name Output: Copy/Export/Print
 		//@{
@@ -286,15 +312,27 @@ class Graph: public QwtPlot
 		void printCropmarks(bool on){d_print_cropmarks = on;};
 
 		void copyImage();
-		QPixmap graphPixmap();
+		QPixmap graphPixmap(const QSize& size = QSize(), double scaleFontsFactor = 1.0);
 		//! Provided for convenience in scripts
 		void exportToFile(const QString& fileName);
-		void exportSVG(const QString& fname);
+		void exportSVG(const QString& fname, const QSizeF& customSize = QSizeF(), int unit = FrameWidget::Pixel, double fontsFactor = 1.0);
 #ifdef EMF_OUTPUT
-		void exportEMF(const QString& fname);
+		void exportEMF(const QString& fname, const QSizeF& customSize = QSizeF(), int unit = FrameWidget::Pixel, double fontsFactor = 1.0);
 #endif
-		void exportVector(const QString& fileName, int res = 0, bool color = true);
-		void exportImage(const QString& fileName, int quality = 100, bool transparent = false);
+		void exportTeX(const QString& fname, bool color = true, bool escapeStrings = true, bool fontSizes = true,
+						const QSizeF& customSize = QSizeF(), int unit = FrameWidget::Pixel, double fontsFactor = 1.0);
+		void exportVector(const QString& fileName, int res = 0, bool color = true,
+						const QSizeF& customSize = QSizeF (), int unit = FrameWidget::Pixel, double fontsFactor = 1.0);
+		void exportImage(const QString& fileName, int quality = 100, bool transparent = false,
+						 int dpi = 0, const QSizeF& customSize = QSizeF (),
+						 int unit = FrameWidget::Pixel, double fontsFactor = 1.0);
+
+		void draw(QPaintDevice *, const QSize& size, double fontsFactor = 1.0);
+		static QSize customPrintSize(const QSizeF& customSize, int unit, int dpi);
+
+		bool isExportingTeX(){return d_is_exporting_tex;};
+		void setTeXExportingMode(bool on = true){d_is_exporting_tex = on;};
+		void setEscapeTeXStringsMode(bool on = true){d_tex_escape_strings = on;};
 		//@}
 
 		void updatePlot();
@@ -336,7 +374,7 @@ class Graph: public QwtPlot
 
 		//! \name Curves Layout
 		//@{
-		CurveLayout initCurveLayout(int style, int curves = 0);
+		CurveLayout initCurveLayout(int style, int curves = 0, bool guessLayout = true);
 		static CurveLayout initCurveLayout();
 		void updateCurveLayout(PlotCurve* c, const CurveLayout *cL);
 		//! Tries to guess not already used curve color and symbol style
@@ -390,7 +428,7 @@ class Graph: public QwtPlot
 
 		LegendWidget* newLegend(const QString& text = QString());
 		//! Creates a new legend text using the curves titles
-		QString legendText();
+		QString legendText(bool layerSpec = false);
 		//@}
 
 		//! \name Obsolete functions provided to keep existing Python scripts working
@@ -450,6 +488,7 @@ class Graph: public QwtPlot
 		QwtScaleWidget* currentScale();
 		QwtScaleWidget* selectedScale();
 		QRect axisTitleRect(const QwtScaleWidget *scale);
+		bool axisTitleSelected();
 
 		ScaleDraw::ScaleType axisType(int axis);
 
@@ -475,6 +514,9 @@ class Graph: public QwtPlot
 		int axisTitleAlignment (int axis);
 		void setAxisTitleAlignment(int axis, int align);
 
+		int axisTitleDistance(int axis);
+		void setAxisTitleDistance(int axis, int dist);
+
         QColor axisColor(int axis);
 		void setAxisColor(int axis, const QColor& color);
 
@@ -562,7 +604,8 @@ class Graph: public QwtPlot
 		 * tool interface.
 		 */
 		bool enableRangeSelectors(const QObject *status_target=NULL, const char *status_slot="");
-
+		bool rangeSelectorsEnabled();
+		QPointer<RangeSelectorTool> rangeSelectorTool(){return d_range_selector;};
 		//! \name Border and background
 		//@{
 		void setFrame(int width = 1, const QColor& color = Qt::black);
@@ -577,6 +620,11 @@ class Graph: public QwtPlot
 		 * \return the number of selected or total points
 		 */
 		int range(const QString& curveTitle, double *start, double *end);
+		/*! Set start and end to selected X range of curve "curve" or, if there's no selection, to the curve's total range.
+		 *
+		 * \return the number of selected or total points
+		 */
+		int range(QwtPlotCurve *c, double *start, double *end);
 
 		//!  Used for VerticalBars, HorizontalBars and Histograms
 		void setBarsGap(int curve, int gapPercent, int offset);
@@ -619,7 +667,7 @@ class Graph: public QwtPlot
 
 		void notifyChanges();
 
-		void updateSecondaryAxis(int axis);
+		void updateSecondaryAxis(int axis, bool changeFormat = false);
 
 		bool isAutoscalingEnabled(){return d_auto_scale;};
 		void enableAutoscaling(bool on = true){d_auto_scale = on;};
@@ -685,7 +733,7 @@ signals:
 		void hiddenPlot(QWidget*);
 
 		void showContextMenu();
-		void showCurveContextMenu(QwtPlotCurve *);
+		void showCurveContextMenu(QwtPlotItem *);
 		void showMarkerPopupMenu();
 
 		void showAxisDialog(int);
@@ -697,17 +745,22 @@ signals:
 		void showFitResults(const QString&);
 		void currentFontChanged(const QFont&);
         void enableTextEditor(Graph *);
+        void axisDivChanged(Graph *, int);
 
 	private:
+		QList<FrameWidget*> stackingOrderEnrichmentsList();
         //! Finds bounding interval of the plot data.
         QwtDoubleInterval axisBoundingInterval(int axis);
         void deselectCurves();
-		void addLegendItem();
 
+		void dropEvent(QDropEvent*);
+		void dragEnterEvent(QDragEnterEvent*);
 		void showEvent (QShowEvent * event);
     	void printFrame(QPainter *painter, const QRect &rect) const;
 		void printCanvas(QPainter *painter, const QRect &canvasRect,
    			 const QwtScaleMap map[axisCnt], const QwtPlotPrintFilter &pfilter) const;
+		virtual void printScale (QPainter *, int axisId, int startDist, int endDist,
+			int baseDist, const QRect &) const;
 		virtual void drawItems (QPainter *painter, const QRect &rect,
 			const QwtScaleMap map[axisCnt], const QwtPlotPrintFilter &pfilter) const;
 
@@ -730,6 +783,8 @@ signals:
 		bool autoScaleFonts;
 		bool d_scale_on_print, d_print_cropmarks;
 		bool drawLineOn, drawArrowOn, drawAxesBackbone;
+		//! Flag telling if we are performing a print operation
+		bool d_is_printing;
 		//! Stores the step the user specified for the four scale. If step = 0.0, the step will be calculated automatically by the Qwt scale engine.
 		QVector<double> d_user_step;
 		//! Arrows/lines on plot
@@ -748,6 +803,13 @@ signals:
 		bool d_auto_scale;
 		//! Axes tick lengths
 		int d_min_tick_length, d_maj_tick_length;
+		bool d_is_exporting_tex;
+		bool d_tex_escape_strings;
 		QList<FrameWidget*> d_enrichments;
+		QwtPlotMagnifier *d_magnifier;
+		QwtPlotPanner *d_panner;
+
+		double d_Douglas_Peuker_tolerance;
+		int d_speed_mode_points;
 };
 #endif // GRAPH_H
