@@ -2,11 +2,8 @@
     File                 : MultiLayer.h
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2006 by Ion Vasilief,
-                           Tilman Hoener zu Siederdissen,
-                           Knut Franke
-    Email (use @ for *)  : ion_vasilief*yahoo.fr, thzs*gmx.net,
-                           knut.franke*gmx.de
+    Copyright            : (C) 2006 - 2009 by Ion Vasilief, Knut Franke
+    Email (use @ for *)  : ion_vasilief*yahoo.fr, knut.franke*gmx.de
     Description          : Multi layer widget
 
  ***************************************************************************/
@@ -32,16 +29,19 @@
 #ifndef MULTILAYER_H
 #define MULTILAYER_H
 
-#include "../MdiSubWindow.h"
-#include "Graph.h"
+#include <MdiSubWindow.h>
+#include <FrameWidget.h>
 #include <QPushButton>
 #include <QLayout>
 #include <QPointer>
 
+class QTextDocument;
 class QLabel;
 class LayerButton;
 class SelectionMoveResizer;
 class LegendWidget;
+class Graph;
+class QwtPlotCurve;
 
 /**
  * \brief An MDI window (MdiSubWindow) managing one or more Graph objects.
@@ -94,6 +94,16 @@ public:
 	QRect canvasChildrenRect();
 	virtual QString sizeToString();
 
+	void setWaterfallLayout(bool on = true);
+	void createWaterfallBox();
+	bool isWaterfallPlot(){return d_is_waterfall_plot;};
+	int waterfallXOffset(){return d_waterfall_offset_x;};
+	int waterfallYOffset(){return d_waterfall_offset_y;};
+	void setWaterfallOffset(int x, int y, bool update = false);
+	bool sideLinesEnabled(){return d_side_lines;};
+
+	void setEqualSizedLayers();
+
 public slots:
 	Graph* addLayer(int x = 0, int y = 0, int width = 0, int height = 0);
 
@@ -113,7 +123,7 @@ public slots:
 	void findBestLayout(int &rows, int &cols);
 
 	QSize arrangeLayers(bool userSize);
-	void arrangeLayers(bool fit, bool userSize);
+	bool arrangeLayers(bool fit, bool userSize);
 	bool swapLayers(int src, int dest);
     void adjustSize();
 
@@ -142,15 +152,28 @@ public slots:
 
 	//! \name Print and Export
 	//@{
-	QPixmap canvasPixmap();
+	QPixmap canvasPixmap(const QSize& size = QSize(), double scaleFontsFactor = 1.0);
 	void exportToFile(const QString& fileName);
-	void exportImage(const QString& fileName, int quality = 100, bool transparent = false);
-	void exportSVG(const QString& fname);
+	void exportImage(QTextDocument *document, int quality = 100, bool transparent = false,
+		int dpi = 0, const QSizeF& customSize = QSizeF (), int unit = FrameWidget::Pixel, double fontsFactor = 1.0);
+	void exportImage(const QString& fileName, int quality = 100, bool transparent = false,
+		int dpi = 0, const QSizeF& customSize = QSizeF (), int unit = FrameWidget::Pixel, double fontsFactor = 1.0);
+	void exportSVG(const QString& fname, const QSizeF& customSize = QSizeF(), int unit = FrameWidget::Pixel, double fontsFactor = 1.0);
     void exportPDF(const QString& fname);
-	void exportVector(const QString& fileName, int res = 0, bool color = true);
+	void exportVector(const QString& fileName, int res = 0, bool color = true,
+		const QSizeF& customSize = QSizeF (), int unit = FrameWidget::Pixel, double fontsFactor = 1.0);
+
+	void draw(QPaintDevice *, const QSizeF& customSize, int unit, int res, double fontsFactor = 1.0);
+
+#ifdef EMF_OUTPUT
+	void exportEMF(const QString& fname, const QSizeF& customSize = QSizeF(), int unit = FrameWidget::Pixel, double fontsFactor = 1.0);
+#endif
+	void exportTeX(const QString& fname, bool color = true, bool escapeStrings = true, bool fontSizes = true,
+					const QSizeF& customSize = QSizeF(), int unit = FrameWidget::Pixel, double fontsFactor = 1.0);
 
 	void copyAllLayers();
 	void print();
+	void print(QPrinter *);
 	void printAllLayers(QPainter *painter);
 	void printActiveLayer();
 	//@}
@@ -164,13 +187,27 @@ public slots:
 
     bool hasSelectedLayers();
 
+    //! \name Waterfall Plots
+	//@{
+    void showWaterfallOffsetDialog();
+    void reverseWaterfallOrder();
+    void showWaterfallFillDialog();
+    void setWaterfallFillColor(const QColor&);
+    void updateWaterfallFill(bool on);
+    void setWaterfallSideLines(bool on = true);
+	void changeWaterfallXOffset(int);
+    void changeWaterfallYOffset(int);
+    void updateWaterfallLayout();
+    void updateWaterfallScales(Graph *g, int axis);
+    //@}
+
 signals:
 	void showEnrichementDialog();
 	void showPlotDialog(int);
 	void showAxisDialog(int);
 	void showScaleDialog(int);
 	void showGraphContextMenu();
-	void showCurveContextMenu(QwtPlotCurve *);
+	void showCurveContextMenu(QwtPlotItem *);
 	void showCurvesDialog();
 	void drawLineEnded(bool);
 	void showAxisTitleDialog();
@@ -188,6 +225,8 @@ signals:
 private:
 	//! \name Event Handlers
 	//@{
+	void dropEvent(QDropEvent*);
+	void dragEnterEvent(QDragEnterEvent*);
 	void wheelEvent(QWheelEvent *);
 	void keyPressEvent(QKeyEvent *);
 	bool eventFilter(QObject *object, QEvent *);
@@ -208,10 +247,18 @@ private:
 
     QList<LayerButton *> buttonsList;
     QList<Graph *> graphsList;
-	QHBoxLayout *layerButtonsBox;
+	QHBoxLayout *layerButtonsBox, *waterfallBox, *toolbuttonsBox;
     QWidget *d_canvas;
 
 	QPointer<SelectionMoveResizer> d_layers_selector;
+
+	int d_waterfall_offset_x, d_waterfall_offset_y;
+	bool d_is_waterfall_plot;
+	//! Flag telling if we need to draw side lines for curves in a waterfall plot
+	bool d_side_lines;
+	QColor d_waterfall_fill_color;
+
+	QPushButton *d_add_layer_btn, *d_remove_layer_btn;
 };
 
 //! Button with layer number
