@@ -28,9 +28,9 @@
  ***************************************************************************/
 #include "MultiPeakFitTool.h"
 #include "RangeSelectorTool.h"
-#include "../ApplicationWindow.h"
+#include <ApplicationWindow.h>
 #include "DataPickerTool.h"
-#include "../cursors.h"
+#include <cursors.h>
 
 #include <qwt_plot_curve.h>
 #include <qwt_plot_marker.h>
@@ -39,14 +39,12 @@
 #include <gsl/gsl_statistics.h>
 
 MultiPeakFitTool::MultiPeakFitTool(Graph *graph, ApplicationWindow *app, MultiPeakFit::PeakProfile profile, int num_peaks, const QObject *status_target, const char *status_slot)
-	: PlotToolInterface(graph),
-	d_profile(profile),
-	d_num_peaks(num_peaks)
+	: PlotToolInterface(graph)
 {
 	d_selected_peaks = 0;
 	d_curve = 0;
 
-	d_fit = new MultiPeakFit(app, d_graph, d_profile, d_num_peaks);
+	d_fit = new MultiPeakFit(app, graph, profile, num_peaks);
 	d_fit->enablePeakCurves(app->generatePeakCurves);
 	d_fit->setPeakCurvesColor(app->peakCurvesColor);
 	d_fit->generateFunction(app->generateUniformFitPoints, app->fitPoints);
@@ -65,11 +63,11 @@ MultiPeakFitTool::MultiPeakFitTool(Graph *graph, ApplicationWindow *app, MultiPe
 MultiPeakFitTool::~MultiPeakFitTool()
 {
 	d_graph->canvas()->releaseMouse();
-	
+
 	foreach(QwtPlotMarker *m, d_lines)
 		m->detach();//remove peak line markers
 	d_lines.clear();
-	
+
 	if (d_picker_tool)
 		delete d_picker_tool;
 	if (d_fit)
@@ -95,7 +93,7 @@ void MultiPeakFitTool::selectPeak(QwtPlotCurve *curve, int point_index)
 	d_graph->replot();
 
 	d_selected_peaks++;
-	if (d_selected_peaks == d_num_peaks)
+	if (d_selected_peaks == d_fit->peaks())
 		finalize();
 	else
 		emit statusText(
@@ -140,7 +138,7 @@ void MultiPeakFitTool::finalize()
 		    int aux = 3*i;
 			d_fit->setInitialGuess(aux + 2, w);
 			double yc = d_fit->initialGuess(aux);
-			if (d_profile == MultiPeakFit::Lorentz)
+			if (d_fit->profile() == MultiPeakFit::Lorentz)
                 d_fit->setInitialGuess(aux, (yc - offset)*M_PI_2*w);
             else
                 d_fit->setInitialGuess(aux, (yc - offset)*sqrt(M_PI_2)*w);
@@ -155,12 +153,12 @@ void MultiPeakFitTool::finalize()
 	foreach(QwtPlotMarker *m, d_lines)
 		m->detach();
 	d_lines.clear();
-	
+
 	d_graph->replot();
     if (d_graph->activeTool() && d_graph->activeTool()->rtti() == PlotToolInterface::Rtti_RangeSelector){
         ((RangeSelectorTool *)d_graph->activeTool())->setEnabled();
     } else
         d_graph->canvas()->unsetCursor();
-	
+
 	d_graph->setActiveTool(NULL);
 }
