@@ -31,7 +31,7 @@
 
 #include <qwt_plot_curve.h>
 #include <qwt_plot_marker.h>
-#include "../Table.h"
+#include <Table.h>
 
 class PlotMarker;
 class Table;
@@ -41,12 +41,12 @@ class PlotCurve: public QwtPlotCurve
 {
 
 public:
-	PlotCurve(const QString& name = QString()): QwtPlotCurve(name), d_type(0), d_plot_style(0), d_x_offset(0.0), d_y_offset(0.0){};
+    PlotCurve(const QString& name = QString());
 
 	//! Sort of rtti()
 	int type(){return d_type;};
 	void setType(int t){d_type = t;};
-	
+
 	/*!\brief The plot style of the curve.
 	 * \sa Graph::CurveType
 	 */
@@ -59,15 +59,36 @@ public:
 	double yOffset(){return d_y_offset;};
 	void setYOffset(double dy){d_y_offset = dy;};
 
+	bool sideLinesEnabled(){return d_side_lines;};
+	void enableSideLines(bool on){d_side_lines = on;};
+
 	QString saveCurveLayout();
 	void restoreCurveLayout(const QStringList& lst);
 
+	//! Set the number of symbols not to be drawn: usefull for large data sets
+	void setSkipSymbolsCount(int count);
+	//! Returns the number of symbols not to be drawn
+	int skipSymbolsCount(){return d_skip_symbols;};
+
 protected:
+	virtual void drawCurve(QPainter *p, int style, const QwtScaleMap &xMap, const QwtScaleMap &yMap, int from, int to) const;
+	void drawSideLines(QPainter *p, const QwtScaleMap &xMap, const QwtScaleMap &yMap, int from, int to) const;
+
+    virtual void drawSymbols(QPainter *p, const QwtSymbol &,
+        const QwtScaleMap &xMap, const QwtScaleMap &yMap, int from, int to) const;
+
+	void drawSticks(QPainter *p,
+        const QwtScaleMap &xMap, const QwtScaleMap &yMap, int from, int to) const;
+
+	QwtDoubleRect boundingRect() const;
+
 	// Rtti
 	int d_type;
 	// The plot style of the curve
 	int d_plot_style;
 	double d_x_offset, d_y_offset;
+	bool d_side_lines;
+	int d_skip_symbols;
 };
 
 class DataCurve: public PlotCurve
@@ -76,7 +97,8 @@ public:
 	DataCurve(Table *t, const QString& xColName, const QString& name, int startRow = 0, int endRow = -1);
     void clone(DataCurve* c);
 
-    QString saveToString();
+    virtual QString saveToString();
+    void restoreLabels(const QStringList& lst);
 
 	QString xColumnName(){return d_x_column;};
 	void setXColumnName(const QString& name){d_x_column = name;};
@@ -115,6 +137,7 @@ public:
 
 	virtual bool updateData(Table *t, const QString& colName);
 	virtual void loadData();
+	void enableSpeedMode();
 
 	//! Returns the row index in the data source table corresponding to the data point index.
 	int tableRow(int point);
@@ -201,7 +224,9 @@ public:
 	double angle(){return d_angle;};
 	void setAngle(double a){d_angle = a;};
 
-	//QwtDoubleRect boundingRect() const;
+	double xLabelOffset(){return d_label_x_offset;};
+	double yLabelOffset(){return d_label_y_offset;};
+	void setLabelOffset(double xOffset, double yOffset){d_label_x_offset = xOffset; d_label_y_offset = yOffset;};
 
 protected:
 	//! Does the actual drawing; see QwtPlotItem::draw.
@@ -209,5 +234,9 @@ protected:
 
 	int d_index;
 	double d_angle;
+	//! x coordinate offset
+	double d_label_x_offset;
+	//! y coordinate offset
+	double d_label_y_offset;
 };
 #endif
