@@ -71,11 +71,12 @@ class PlotToolInterface
         Rtti_MultiPeakFitTool,
         Rtti_LineProfileTool,
 		Rtti_AddWidgetTool,
-		
+		Rtti_DrawDataPoints,
+
         Rtti_PlotUserTool = 1000
     };
 
-		PlotToolInterface(Graph *graph, const QObject *status_target = NULL, const char *status_slot = "") { d_graph = graph;};
+		PlotToolInterface(Graph *graph, const QObject *status_target = NULL, const char *status_slot = "") {Q_UNUSED(status_target); Q_UNUSED(status_slot); d_graph = graph;};
 		virtual ~PlotToolInterface() {};
 
         virtual int rtti() const { return Rtti_PlotTool;};
