@@ -2,8 +2,8 @@
     File                 : QwtErrorPlotCurve.cpp
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2006 by Ion Vasilief, Tilman Hoener zu Siederdissen
-    Email (use @ for *)  : ion_vasilief*yahoo.fr, thzs*gmx.net
+    Copyright            : (C) 2006 by Ion Vasilief
+    Email (use @ for *)  : ion_vasilief*yahoo.fr
     Description          : Error bars curve
 
  ***************************************************************************/
@@ -29,6 +29,7 @@
 #include "QwtErrorPlotCurve.h"
 #include "QwtBarCurve.h"
 #include "ScaleEngine.h"
+#include "Graph.h"
 
 #include <qwt_painter.h>
 #include <qwt_symbol.h>
@@ -39,22 +40,20 @@ QwtErrorPlotCurve::QwtErrorPlotCurve(int orientation, Table *t, const QString& n
 	DataCurve(t, QString(), name),
 	d_master_curve(NULL)
 {
-	cap = 10;
 	type = orientation;
-	plus = true;
-	minus = true;
-	through = false;
-	setType(Graph::ErrorBars);
-	setPlotStyle(Graph::ErrorBars);
-    setStyle(QwtPlotCurve::UserCurve);
+	init();
 }
 
 QwtErrorPlotCurve::QwtErrorPlotCurve(Table *t, const QString& name):
 	DataCurve(t, QString(), name),
 	d_master_curve(NULL)
 {
-	cap=10;
 	type = Vertical;
+	init();
+}
+
+void QwtErrorPlotCurve::init()
+{
 	plus = true;
 	minus = true;
 	through = false;
@@ -65,7 +64,7 @@ QwtErrorPlotCurve::QwtErrorPlotCurve(Table *t, const QString& name):
 
 void QwtErrorPlotCurve::copy(const QwtErrorPlotCurve *e)
 {
-	cap = e->cap;
+	d_cap_length = e->d_cap_length;
 	type = e->type;
 	plus = e->plus;
 	minus = e->minus;
@@ -84,7 +83,10 @@ void QwtErrorPlotCurve::draw(QPainter *painter,
 		to = dataSize() - 1;
 
   painter->save();
-  painter->setPen(pen());
+  QPen p = QwtPainter::scaledPen(pen());
+  p.setCapStyle(Qt::FlatCap);
+  p.setJoinStyle(Qt::MiterJoin);
+  painter->setPen(p);
   drawErrorBars(painter, xMap, yMap, from, to);
   painter->restore();
 }
@@ -92,11 +94,13 @@ void QwtErrorPlotCurve::draw(QPainter *painter,
 void QwtErrorPlotCurve::drawErrorBars(QPainter *painter,
 		const QwtScaleMap &xMap, const QwtScaleMap &yMap, int from, int to) const
 {
-    int sh = 0, sw =0;
+    int sh2 = 0, sw2 = 0;
+    double x_factor = (double)painter->device()->logicalDpiX()/(double)plot()->logicalDpiX();
+    double y_factor = (double)painter->device()->logicalDpiY()/(double)plot()->logicalDpiY();
     const QwtSymbol symbol = d_master_curve->symbol();
     if (symbol.style() != QwtSymbol::NoSymbol){
-        sh = symbol.size().height();
-        sw = symbol.size().width();
+        sh2 = int(0.5*y_factor*symbol.size().height());
+        sw2 = int(0.5*x_factor*symbol.size().width());
     }
 
 	double d_xOffset = 0.0;
@@ -107,43 +111,51 @@ void QwtErrorPlotCurve::drawErrorBars(QPainter *painter,
 		d_yOffset = ((QwtBarCurve *)d_master_curve)->dataOffset();
 
 	ScaleEngine *yScaleEngine = (ScaleEngine *)plot()->axisScaleEngine(yAxis());
-	bool logYScale = (yScaleEngine->type() == QwtScaleTransformation::Log10) ? true : false;
-	
-	for (int i = from; i <= to; i++){
+	bool logYScale = (yScaleEngine->type() == ScaleTransformation::Log10) ? true : false;
+
+	int skipPoints = d_master_curve->skipSymbolsCount() + d_skip_symbols;
+	if (d_skip_symbols > 0)
+		skipPoints--;
+	if (skipPoints == 0)
+		skipPoints = 1;
+
+	for (int i = from; i <= to; i += skipPoints){
 		const int xi = xMap.transform(x(i) + d_xOffset);
 		const int yi = yMap.transform(y(i) + d_yOffset);
 
 		if (type == Vertical){
 			const int yh = yMap.transform(y(i) + err[i]);
 			const int yl = yMap.transform(y(i) - err[i]);
-			const int yhl = yi - sh/2;
-			const int ylh = yi + sh/2;
+			const int yhl = yi - sh2;
+			const int ylh = yi + sh2;
+			const int cap2 = qRound(d_cap_length*0.5*x_factor);
 
 			if (plus){
 				QwtPainter::drawLine(painter, xi, yhl, xi, yh);
-				QwtPainter::drawLine(painter, xi - cap/2, yh, xi + cap/2, yh);
+				QwtPainter::drawLine(painter, xi - cap2, yh, xi + cap2, yh);
 			}
 			if (minus && (!logYScale || (logYScale && yl > 0))){
 				QwtPainter::drawLine(painter, xi, ylh, xi, yl);
-				QwtPainter::drawLine(painter, xi - cap/2, yl, xi + cap/2, yl);
+				QwtPainter::drawLine(painter, xi - cap2, yl, xi + cap2, yl);
 			}
-			if (through)
+			if (through && (plus || minus))
 				QwtPainter::drawLine(painter, xi, yhl, xi, ylh);
 		} else if (type == Horizontal) {
 			const int xp = xMap.transform(x(i) + err[i]);
 			const int xm = xMap.transform(x(i) - err[i]);
-  			const int xpm = xi + sw/2;
-  	        const int xmp = xi - sw/2;
+  			const int xpm = xi + sw2;
+  	        const int xmp = xi - sw2;
+			const int cap2 = qRound(d_cap_length*0.5*y_factor);
 
 			if (plus){
 				QwtPainter::drawLine(painter, xp, yi, xpm, yi);
-				QwtPainter::drawLine(painter, xp, yi - cap/2, xp, yi + cap/2);
+				QwtPainter::drawLine(painter, xp, yi - cap2, xp, yi + cap2);
 			}
 			if (minus){
 				QwtPainter::drawLine(painter, xm, yi, xmp, yi);
-				QwtPainter::drawLine(painter, xm, yi - cap/2, xm, yi + cap/2);
+				QwtPainter::drawLine(painter, xm, yi - cap2, xm, yi + cap2);
 			}
-			if (through)
+			if (through && (plus || minus))
 				QwtPainter::drawLine(painter, xmp, yi, xpm, yi);
 		}
 	}
@@ -262,9 +274,6 @@ void QwtErrorPlotCurve::loadData()
 	if (xcol<0 || ycol<0 || errcol<0)
 		return;
 
-	int xColType = mt->columnType(xcol);
-	int yColType = mt->columnType(ycol);
-
 	d_start_row = d_master_curve->startRow();
 	d_end_row = d_master_curve->endRow();
     int r = abs(d_end_row - d_start_row) + 1;
@@ -276,20 +285,10 @@ void QwtErrorPlotCurve::loadData()
 		QString yval = mt->text(i, ycol);
 		QString errval = d_table->text(i, errcol);
 		if (!xval.isEmpty() && !yval.isEmpty() && !errval.isEmpty()){
-		    bool ok = true;
-			if (xColType == Table::Text)
-				X[data_size] = (double)(data_size + 1);
-			else
-				X[data_size] = locale.toDouble(xval, &ok);
-
-			if (yColType == Table::Text)
-				Y[data_size] = (double)(data_size + 1);
-			else
-				Y[data_size] = locale.toDouble(yval, &ok);
-
-            if (!ok)
-                continue;
+			X[data_size] = d_master_curve->x(data_size);
+			Y[data_size] = d_master_curve->y(data_size);
 
+			bool ok = true;
 			err[data_size] = locale.toDouble(errval, &ok);
 			if (ok)
                 data_size++;
