@@ -2,8 +2,8 @@
     File                 : QwtHistogram.cpp
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2006 by Ion Vasilief, Tilman Hoener zu Siederdissen
-    Email (use @ for *)  : ion_vasilief*yahoo.fr, thzs*gmx.net
+    Copyright            : (C) 2006 - 2009 by Ion Vasilief
+    Email (use @ for *)  : ion_vasilief*yahoo.fr
     Description          : Histogram class
 
  ***************************************************************************/
@@ -27,14 +27,16 @@
  *                                                                         *
  ***************************************************************************/
 #include "QwtHistogram.h"
-#include "../matrix/Matrix.h"
-#include "../matrix/MatrixModel.h"
+#include "Graph.h"
+#include <Matrix.h>
+#include <MatrixModel.h>
 #include <QPainter>
 #include <gsl/gsl_vector.h>
 #include <gsl/gsl_histogram.h>
 
-QwtHistogram::QwtHistogram(Table *t, const QString& xColName, const QString& name, int startRow, int endRow):
-	QwtBarCurve(QwtBarCurve::Vertical, t, xColName, name, startRow, endRow)
+QwtHistogram::QwtHistogram(Table *t, const QString& name, int startRow, int endRow):
+	QwtBarCurve(QwtBarCurve::Vertical, t, "", name, startRow, endRow),
+	d_autoBin(true)
 {
     d_matrix = 0;
     setType(Graph::Histogram);
@@ -110,6 +112,22 @@ void QwtHistogram::setBinning(bool autoBin, double size, double begin, double en
 	d_end = end;
 }
 
+void QwtHistogram::setBinning(double binSize, double begin, double end)
+{
+	d_autoBin = false;
+	d_bin_size = binSize;
+	d_begin = begin;
+	d_end = end;
+}
+
+void QwtHistogram::setAutoBinning(bool autoBin)
+{
+	if (d_autoBin == autoBin)
+		return;
+
+	d_autoBin = autoBin;
+}
+
 void QwtHistogram::loadData()
 {
     if (d_matrix){
@@ -180,11 +198,7 @@ void QwtHistogram::loadData()
 	for (int i = 0; i<size; i++ )
 		gsl_histogram_increment (h, Y[i]);
 
-#ifdef Q_CC_MSVC
-    QVarLengthArray<double> X(n); //stores ranges (x) and bins (y)
-#else
     double X[n]; //stores ranges (x) and bins (y)
-#endif
 	Y.resize(n);
 	for (int i = 0; i<n; i++ ){
 		Y[i] = gsl_histogram_get (h, i);
@@ -192,11 +206,7 @@ void QwtHistogram::loadData()
 		gsl_histogram_get_range (h, i, &lower, &upper);
 		X[i] = lower;
 	}
-#ifdef Q_CC_MSVC
-	setData(X.data(), Y.data(), n);
-#else
 	setData(X, Y.data(), n);
-#endif
 
 	d_mean = gsl_histogram_mean(h);
 	d_standard_deviation = gsl_histogram_sigma(h);
@@ -251,22 +261,15 @@ void QwtHistogram::loadDataFromMatrix()
 	for (int i = 0; i<size; i++ )
 		gsl_histogram_increment (h, data[i]);
 
-#ifdef Q_CC_MSVC
-	QVarLengthArray<double> X(n), Y(n); //stores ranges (x) and bins (y)
-#else
 	double X[n], Y[n]; //stores ranges (x) and bins (y)
-#endif
 	for (int i = 0; i<n; i++ ){
 		Y[i] = gsl_histogram_get (h, i);
 		double lower, upper;
 		gsl_histogram_get_range (h, i, &lower, &upper);
 		X[i] = lower;
 	}
-#ifdef Q_CC_MSVC
-	setData(X.data(), Y.data(), n);
-#else
+
 	setData(X, Y, n);
-#endif
 
 	d_mean = gsl_histogram_mean(h);
 	d_standard_deviation = gsl_histogram_sigma(h);
@@ -275,66 +278,3 @@ void QwtHistogram::loadDataFromMatrix()
 
 	gsl_histogram_free (h);
 }
-
-void QwtHistogram::initData(double *Y, int size)
-{
-	if(size<2 || (size == 2 && Y[0] == Y[1]))
-	{//non valid histogram data
-		double x[2], y[2];
-		for (int i = 0; i<2; i++ ){
-			y[i] = 0;
-			x[i] = 0;
-		}
-		setData(x, y, 2);
-		return;
-	}
-
-	int n = 10;//default value
-#ifdef Q_CC_MSVC
-	QVarLengthArray<double> x(n), y(n); //store ranges (x) and bins (y)
-#else
-	double x[n], y[n]; //store ranges (x) and bins (y)
-#endif
-	gsl_histogram * h = gsl_histogram_alloc (n);
-	if (!h)
-		return;
-
-	gsl_vector *v;
-	v = gsl_vector_alloc (size);
-	for (int i = 0; i<size; i++ )
-		gsl_vector_set (v, i, Y[i]);
-
-	double min, max;
-	gsl_vector_minmax (v, &min, &max);
-	gsl_vector_free (v);
-
-	d_begin = floor(min);
-	d_end = ceil(max);
-
-	gsl_histogram_set_ranges_uniform (h, floor(min), ceil(max));
-
-	for (int i = 0; i<size; i++ )
-		gsl_histogram_increment (h, Y[i]);
-
-	for (int i = 0; i<n; i++ ){
-		y[i] = gsl_histogram_get (h, i);
-		double lower, upper;
-		gsl_histogram_get_range (h, i, &lower, &upper);
-		x[i] = lower;
-	}
-
-#ifdef Q_CC_MSVC
-	setData(x.data(), y.data(), n);
-#else
-	setData(x, y, n);
-#endif
-
-	d_bin_size = (d_end - d_begin)/(double)n;
-	d_autoBin = true;
-    d_mean = gsl_histogram_mean(h);
-	d_standard_deviation = gsl_histogram_sigma(h);
-	d_min = gsl_histogram_min_val(h);
-	d_max = gsl_histogram_max_val(h);
-
-	gsl_histogram_free (h);
-}
