@@ -27,15 +27,20 @@
  *                                                                         *
  ***************************************************************************/
 #include "QwtPieCurve.h"
-#include "../ColorBox.h"
-#include "../Table.h"
-#include "../PenStyleBox.h"
+#include "MultiLayer.h"
+#include <Table.h>
+
+#include <ColorBox.h>
+#include <PenStyleBox.h>
 
 #include <QPaintDevice>
 #include <QPainter>
 #include <QPainterPath>
 #include <QVarLengthArray>
 
+#include <qwt_plot_canvas.h>
+#include <qwt_painter.h>
+
 QwtPieCurve::QwtPieCurve(Table *t, const QString& name, int startRow, int endRow):
 	DataCurve(t, QString(), name, startRow, endRow),
 	d_pie_ray(50),
@@ -106,10 +111,19 @@ void QwtPieCurve::draw(QPainter *painter, const QwtScaleMap &xMap, const QwtScal
 
 void QwtPieCurve::drawDisk(QPainter *painter, const QwtScaleMap &xMap, const QwtScaleMap &yMap) const
 {
-    const double x_width = fabs(xMap.p1() - xMap.p2());
-	const double x_center = (xMap.p1() + xMap.p2())*0.5 + d_horizontal_offset*0.01*x_width;
-	const double y_center = (yMap.p1() + yMap.p2())*0.5;
-	const double ray_x = d_pie_ray*0.005*qMin(x_width, fabs(yMap.p1() - yMap.p2()));
+	QwtPlotCanvas* canvas = plot()->canvas();
+	QPoint center = QPoint(canvas->x() + canvas->width()/2, canvas->y() + canvas->height()/2);
+
+	// calculate resolution factors
+	double x_factor = (double)painter->device()->logicalDpiX()/(double)plot()->logicalDpiX();
+	double y_factor = (double)painter->device()->logicalDpiY()/(double)plot()->logicalDpiY();
+
+	const double width = canvas->width()*x_factor;
+	const double height = canvas->height()*y_factor;
+	const double x_center = center.x()*x_factor + d_horizontal_offset*0.01*width;
+	const double y_center = center.y()*y_factor;
+
+	const double ray_x = d_pie_ray*0.005*qMin(width, height);
 	const double view_angle_rad = d_view_angle*M_PI/180.0;
 	const double ray_y = ray_x*sin(view_angle_rad);
 	const double thick = 0.01*d_thickness*ray_x*cos(view_angle_rad);
@@ -125,7 +139,7 @@ void QwtPieCurve::drawDisk(QPainter *painter, const QwtScaleMap &xMap, const Qwt
 
     painter->save();
 
-    painter->setPen(QwtPlotCurve::pen());
+    painter->setPen(QwtPainter::scaledPen(pen()));
     painter->setBrush(QBrush(color(0), QwtPlotCurve::brush().style()));
 
     QPointF start(x_center + ray_x, y_center);
@@ -171,8 +185,8 @@ void QwtPieCurve::drawDisk(QPainter *painter, const QwtScaleMap &xMap, const Qwt
                 if (a_deg > 0 && a_deg < 180)
                     y += thick;
 
-                double dx = xMap.invTransform(x - l->width()/2);
-                double dy = yMap.invTransform(y - l->height()/2);
+                double dx = xMap.invTransform(x - l->width()*x_factor/2);
+                double dy = yMap.invTransform(y - l->height()*y_factor/2);
                 l->setOriginCoord(dx, dy);
             }
         }
@@ -182,10 +196,19 @@ void QwtPieCurve::drawDisk(QPainter *painter, const QwtScaleMap &xMap, const Qwt
 
 void QwtPieCurve::drawSlices(QPainter *painter, const QwtScaleMap &xMap, const QwtScaleMap &yMap, int from, int to) const
 {
-    const double x_width = fabs(xMap.p1() - xMap.p2());
-	const double x_center = (xMap.p1() + xMap.p2())*0.5 + d_horizontal_offset*0.01*x_width;
-	const double y_center = (yMap.p1() + yMap.p2())*0.5;
-	const double ray_x = d_pie_ray*0.005*qMin(x_width, fabs(yMap.p1() - yMap.p2()));
+	QwtPlotCanvas* canvas = plot()->canvas();
+	QPoint center = QPoint(canvas->x() + canvas->width()/2, canvas->y() + canvas->height()/2);
+
+	// calculate resolution factors
+	double x_factor = (double)painter->device()->logicalDpiX()/(double)plot()->logicalDpiX();
+	double y_factor = (double)painter->device()->logicalDpiY()/(double)plot()->logicalDpiY();
+
+	const double width = canvas->width()*x_factor;
+	const double height = canvas->height()*y_factor;
+	const double x_center = center.x()*x_factor + d_horizontal_offset*0.01*width;
+	const double y_center = center.y()*y_factor;
+
+	const double ray_x = d_pie_ray*0.005*qMin(width, height);
 	const double view_angle_rad = d_view_angle*M_PI/180.0;
 	const double ray_y = ray_x*sin(view_angle_rad);
 	const double thick = 0.01*d_thickness*ray_x*cos(view_angle_rad);
@@ -235,7 +258,7 @@ void QwtPieCurve::drawSlices(QPainter *painter, const QwtScaleMap &xMap, const Q
 		const double q = yi/sum;
 		const int value = (int)(q*5760);
 
-		painter->setPen(QwtPlotCurve::pen());
+		painter->setPen(QwtPainter::scaledPen(pen()));
 		painter->setBrush(QBrush(color(i), QwtPlotCurve::brush().style()));
 
 		double deg = q*360;
@@ -342,8 +365,8 @@ void QwtPieCurve::drawSlices(QPainter *painter, const QwtScaleMap &xMap, const Q
                 if (a_deg > 0 && a_deg < 180)
                     y += thick;
 
-                double dx = xMap.invTransform(x - l->width()/2);
-                double dy = yMap.invTransform(y - l->height()/2);
+                double dx = xMap.invTransform(x - l->width()*x_factor/2);
+                double dy = yMap.invTransform(y - l->height()*y_factor/2);
                 l->setOriginCoord(dx, dy);
             }
 		}
@@ -476,15 +499,15 @@ QString PieLabel::saveToString()
 {
 	if (!d_pie_curve)
 		return LegendWidget::saveToString();
-	
+
 	if (text().isEmpty())
 		return QString::null;
-	
+
 	QString s = "<PieText>\n";
 	s += "<index>" + QString::number(d_pie_curve->labelsList().indexOf(this)) + "</index>\n";
 	s += FrameWidget::saveToString();
 	s += "<Text>\n" + text() + "\n</Text>\n";
-	
+
 	QFont f = font();
 	s += "<Font>" + f.family() + "\t";
 	s += QString::number(f.pointSize())+"\t";
@@ -492,8 +515,8 @@ QString PieLabel::saveToString()
 	s += QString::number(f.italic())+"\t";
 	s += QString::number(f.underline())+"\t";
 	s += QString::number(f.strikeOut())+"</Font>\n";
-	
-	s += "<TextColor>" + textColor().name()+"</TextColor>\n";	
+
+	s += "<TextColor>" + textColor().name()+"</TextColor>\n";
 	QColor bc = backgroundColor();
 	s += "<Background>" + bc.name() + "</Background>\n";
 	s += "<Alpha>" + QString::number(bc.alpha()) + "</Alpha>\n";
@@ -507,12 +530,12 @@ void PieLabel::restore(Graph *g, const QStringList& lst)
 	QStringList::const_iterator line;
 	QColor backgroundColor = Qt::white, textColor = Qt::black;
 	QFont f = QFont();
-	double x = 0.0, y = 0.0;	
-	QString text;	
+	double x = 0.0, y = 0.0;
+	QString text;
 	int frameStyle = 0, angle = 0;
 	QPen framePen = QPen(Qt::black, 1, Qt::SolidLine, Qt::SquareCap, Qt::MiterJoin);
 	for (line = lst.begin(); line != lst.end(); line++){
-        QString s = *line;	
+        QString s = *line;
 		if (s.contains("<index>")){
 			int index = s.remove("<index>").remove("</index>").toInt();
 			QwtPieCurve *pie = (QwtPieCurve *)g->curve(0);
@@ -555,7 +578,7 @@ void PieLabel::restore(Graph *g, const QStringList& lst)
 		else if (s.contains("<Angle>"))
 			angle = s.remove("<Angle>").remove("</Angle>").toInt();
 	}
-	
+
 	if (l){
 		l->setFrameStyle(frameStyle);
 		l->setFramePen(framePen);
