@@ -28,7 +28,8 @@
  ***************************************************************************/
 #include "ScaleDraw.h"
 #include "Graph.h"
-#include "../MyParser.h"
+#include "MultiLayer.h"
+#include <MyParser.h>
 #include "ScaleEngine.h"
 
 #include <QDateTime>
@@ -49,7 +50,7 @@ ScaleDraw::ScaleDraw(Graph *plot, const QString& formula):
 	d_type(Numeric),
 	d_numeric_format(Automatic),
 	d_fmt('g'),
-    d_prec(4),
+	d_prec(4),
 	d_formula(formula),
 	d_majTicks(Out),
 	d_minTicks(Out),
@@ -65,7 +66,7 @@ ScaleDraw::ScaleDraw(Graph *plot, const QStringList& labels, const QString& form
 	d_type(type),
 	d_numeric_format(Automatic),
 	d_fmt('g'),
-    d_prec(4),
+	d_prec(4),
 	d_formula(""),
 	d_majTicks(Out),
 	d_minTicks(Out),
@@ -82,7 +83,7 @@ ScaleDraw::ScaleDraw(Graph *plot, ScaleDraw* sd):
 	d_type = sd->d_type;
 	d_numeric_format = sd->d_numeric_format;
 	d_fmt = sd->d_fmt;
-    d_prec = sd->d_prec;
+	d_prec = sd->d_prec;
 	d_formula = sd->d_formula;
 	d_majTicks = sd->d_majTicks;
 	d_minTicks = sd->d_minTicks;
@@ -127,6 +128,87 @@ QwtText ScaleDraw::label(double value) const
 					return QwtText("10<sup>" + s + "</sup>");
 				else
 					return QwtText(list[0] + "x10<sup>" + s + "</sup>");
+
+			}
+			else if (d_numeric_format == Engineering){
+
+				QString eng_suff;
+				bool ok;
+				double new_value = QString("%1").arg(value, 0, 'e', d_prec).toDouble(&ok);
+
+				if(fabs(new_value) >= 1e18)
+				{
+					eng_suff = 'E';
+					new_value /= 1e18;
+				}
+				else if(fabs(new_value) >= 1e15)
+				{
+					eng_suff = 'P';
+					new_value /= 1e15;
+				}
+				else if(fabs(new_value) >= 1e12)
+				{
+					eng_suff = 'T';
+					new_value /= 1e12;
+				}
+				else if(fabs(new_value) >= 1e9)
+				{
+					eng_suff = 'G';
+					new_value /= 1e9;
+				}
+				else if(fabs(new_value) >= 1e6)
+				{
+					eng_suff = 'M';
+					new_value /= 1e6;
+				}
+				else if(fabs(new_value) >= 1e3)
+				{
+					eng_suff = 'k';
+					new_value /= 1e3;
+				}
+				else if(fabs(new_value) >= 1)
+				{
+					eng_suff = "";
+					new_value /= 1;
+				}
+				else if(fabs(new_value) >= 1e-3)
+				{
+					eng_suff = 'm';
+					new_value /= 1e-3;
+				}
+				else if(fabs(new_value) >= 1e-6)
+				{
+					eng_suff = '';
+					new_value /= 1e-6;
+				}
+				else if(fabs(new_value) >= 1e-9)
+				{
+					eng_suff = 'n';
+					new_value /= 1e-9;
+				}
+				else if(fabs(new_value) >= 1e-12)
+				{
+					eng_suff = 'p';
+					new_value /= 1e-12;
+				}
+				else if(fabs(new_value) >= 1e-15)
+				{
+					eng_suff = 'f';
+					new_value /= 1e-15;
+				}
+				else
+				{
+					eng_suff = 'a';
+					new_value /= 1e-18;
+				}
+
+				QString txt = locale.toString((new_value), 'f', d_prec);
+
+				if(txt.contains(QRegExp("^0[\\.,]?0*$")))
+					return QwtText("0");
+
+				return QwtText(txt + eng_suff);
+
 			} else
 				return QwtText(locale.toString(transformValue(value), d_fmt, d_prec));
 		break;
@@ -267,26 +349,36 @@ void ScaleDraw::drawLabel(QPainter *painter, double value) const
         	return;
 	}
 
-    QwtText lbl = tickLabel(painter->font(), value);
-    if (lbl.isEmpty())
+	QwtText lbl = tickLabel(painter->font(), value);
+    if ( lbl.isEmpty() )
         return;
 
-    const QPoint pos = labelPosition(value);
+    QPoint pos = labelPosition(value);
 
     QSize labelSize = lbl.textSize(painter->font());
     if ( labelSize.height() % 2 )
         labelSize.setHeight(labelSize.height() + 1);
 
-    const QMatrix m = labelMatrix(pos, labelSize);
+    const QwtMetricsMap metricsMap = QwtPainter::metricsMap();
+    QwtPainter::resetMetricsMap();
+
+    labelSize = metricsMap.layoutToDevice(labelSize);
+    pos = metricsMap.layoutToDevice(pos);
 
     painter->save();
-    painter->setMatrix(m, true);
-    if (d_selected)
+    painter->setMatrix(labelMatrix( pos, labelSize), true);
+
+	if (d_selected)
         lbl.setBackgroundPen(QPen(Qt::blue));
     else
         lbl.setBackgroundPen(QPen(Qt::NoPen));
 
-    lbl.draw(painter, QRect(QPoint(0, 0), labelSize));
+	lbl.setRenderFlags(labelAlignment());
+
+    lbl.draw (painter, QRect(QPoint(0, 0), labelSize) );
+
+    QwtPainter::setMetricsMap(metricsMap); // restore metrics map
+
     painter->restore();
 }
 
@@ -333,6 +425,9 @@ void ScaleDraw::setNumericFormat(NumericFormat format)
 		case Superscripts:
 			d_fmt = 's';
 		break;
+		case Engineering:
+			d_fmt = 'f';
+		break;
 	}
 }
 
@@ -476,28 +571,25 @@ void ScaleDraw::drawBackbone(QPainter *painter) const
 {
     ScaleEngine *sc_engine = (ScaleEngine *)d_plot->axisScaleEngine(axis());
     if (!sc_engine->hasBreak()){
-        const int len = length();
-        const int bw = painter->pen().width();
-        const int bw2 = bw / 2;
-        QPoint pos = this->pos();
-        switch(alignment()){
-            case LeftScale:
-                QwtPainter::drawLine(painter, pos.x() - bw2,
-                pos.y(), pos.x() - bw2, pos.y() + len );
-            break;
-            case RightScale:
-                QwtPainter::drawLine(painter, pos.x() + bw2,
-                pos.y(), pos.x() + bw2, pos.y() + len);
-            break;
-            case TopScale:
-                QwtPainter::drawLine(painter, pos.x(), pos.y() - bw2,
-                pos.x() + len, pos.y() - bw2);
-            break;
-            case BottomScale:
-                QwtPainter::drawLine(painter, pos.x(), pos.y() + bw2,
-                pos.x() + len, pos.y() + bw2);
-            break;
-        }
+    	if (d_plot->isPrinting()){
+			const int len = length();
+			QPoint pos = this->pos();
+			switch(alignment()){
+				case LeftScale:
+					QwtPainter::drawLine(painter, pos.x(), pos.y(), pos.x(), pos.y() + len );
+				break;
+				case RightScale:
+					QwtPainter::drawLine(painter, pos.x(), pos.y(), pos.x(), pos.y() + len);
+				break;
+				case TopScale:
+					QwtPainter::drawLine(painter, pos.x(), pos.y(), pos.x() + len, pos.y());
+				break;
+				case BottomScale:
+					QwtPainter::drawLine(painter, pos.x(), pos.y(), pos.x() + len, pos.y());
+				break;
+			}
+    	} else
+			QwtScaleDraw::drawBackbone(painter);
         return;
     }
 
@@ -585,10 +677,10 @@ void ScaleDraw::setDateFormat(const QDateTime& d, const QString& format)
 
 QString ScaleDraw::formatString()
 {
-    if (d_type == Time)
-        return d_date_time_origin.time().toString() + ";" + d_format_info;
-    else if (d_type == Date)
-        return d_date_time_origin.toString(Qt::ISODate) + ";" + d_format_info;
+	if (d_type == Time)
+		return d_date_time_origin.time().toString() + ";" + d_format_info;
+	else if (d_type == Date)
+		return d_date_time_origin.toString(Qt::ISODate) + ";" + d_format_info;
 
-    return d_format_info;
+	return d_format_info;
 }
