@@ -30,8 +30,8 @@
 #include "Graph.h"
 #include "PlotCurve.h"
 #include "FunctionCurve.h"
-#include "../ApplicationWindow.h"
-#include "../cursors.h"
+#include <ApplicationWindow.h>
+#include <cursors.h>
 #include "DataPickerTool.h"
 #include "ScreenPickerTool.h"
 #include <QMessageBox>
@@ -49,20 +49,23 @@ TranslateCurveTool::TranslateCurveTool(Graph *graph, ApplicationWindow *app, Dir
 
 	// Phase 1: select curve point
 	d_sub_tool = new DataPickerTool(d_graph, app, DataPickerTool::Display, this, SIGNAL(statusText(const QString&)));
-	connect((DataPickerTool*)d_sub_tool, SIGNAL(selected(QwtPlotCurve*,int)),
+	connect((DataPickerTool*)d_sub_tool, SIGNAL(selected(QwtPlotCurve*, int)),
 			this, SLOT(selectCurvePoint(QwtPlotCurve*,int)));
-	
+
 	emit statusText(tr("Double-click on plot to select a data point!"));
 }
 
 void TranslateCurveTool::selectCurvePoint(QwtPlotCurve *curve, int point_index)
 {
+	if (!curve)
+		return;
+
 	if(((PlotCurve *)curve)->type() != Graph::Function){
 		DataCurve *c = (DataCurve *)curve;
 		Table *t = c->table();
 		if (!t)
 			return;
-		
+
 	    if (d_dir == Horizontal && t->isReadOnlyColumn(t->colIndex(c->xColumnName()))){
             QMessageBox::warning(d_app, tr("QtiPlot - Warning"),
             tr("The column '%1' is read-only! Operation aborted!").arg(c->xColumnName()));
@@ -75,9 +78,9 @@ void TranslateCurveTool::selectCurvePoint(QwtPlotCurve *curve, int point_index)
 			delete d_sub_tool;
 			d_graph->setActiveTool(NULL);
 			return;
-        } 
+        }
 	}
-		
+
 	d_selected_curve = curve;
 	d_curve_point = QwtDoublePoint(curve->x(point_index), curve->y(point_index));
 	delete d_sub_tool;
@@ -87,7 +90,7 @@ void TranslateCurveTool::selectCurvePoint(QwtPlotCurve *curve, int point_index)
 	((ScreenPickerTool*)d_sub_tool)->append(d_curve_point);
 	ScreenPickerTool::MoveRestriction moveRestriction = ScreenPickerTool::Vertical;
 	if (d_dir == Horizontal)
-		moveRestriction = ScreenPickerTool::Horizontal;	
+		moveRestriction = ScreenPickerTool::Horizontal;
 	((ScreenPickerTool*)d_sub_tool)->setMoveRestriction(moveRestriction);
 	connect((ScreenPickerTool*)d_sub_tool, SIGNAL(selected(const QwtDoublePoint&)), this, SLOT(selectDestination(const QwtDoublePoint&)));
 	emit statusText(tr("Curve selected! Move cursor and click to choose a point and double-click/press 'Enter' to finish!"));
