@@ -105,22 +105,17 @@ private slots:
 	void customAxisFont();
 	void showAxis();
 	void updateShowBox(int axis);
-	void pickAxisColor();
-	void pickAxisNumColor();
 	void updateAxisColor(int);
 	int mapToQwtAxis(int axis);
 	int mapToQwtAxisId();
 	void updateTickLabelsList(bool);
 	void updateMinorTicksList(int scaleType);
 	void setTicksType(int);
-	void updateMajTicksType(int);
-	void updateMinTicksType(int);
 	void updateGrid();
 	void setLabelsNumericFormat(int);
 	void updateLabelsFormat(int);
 	void showAxisFormatOptions(int format);
 	void setBaselineDist(int);
-	void changeBaselineDist(int baseline);
 	void changeMinorTicksLength (int minLength);
 	void changeMajorTicksLength (int majLength);
 	void pageChanged ( QWidget *page);
@@ -128,6 +123,8 @@ private slots:
 				  const QColor&, int, int, int, int, const QString&, const QColor&);
     void applyCanvasFormat();
     void setFrameDefaultValues();
+	void applyAxisFormat();
+	void setDisplayDateTimeFormat( const QString &);
 
 protected:
     void applyCanvasFormatTo(Graph *g);
@@ -141,6 +138,8 @@ protected:
 	void initFramePage();
 	//! Modifies the grid
 	void applyChangesToGrid(Grid *grid);
+	//! Customizes graph axes
+	void applyAxisFormatToLayer(Graph *g);
 
     QPushButton* buttonApply;
     QPushButton* buttonOk;
@@ -152,9 +151,10 @@ protected:
     QComboBox* boxScaleType;
     QComboBox* boxMinorValue;
     DoubleSpinBox* boxStep;
-    QCheckBox* btnStep, *btnInvert;
+    QRadioButton* btnStep;
+    QCheckBox *btnInvert;
     QSpinBox* boxMajorValue;
-    QCheckBox* btnMajor;
+    QRadioButton* btnMajor;
     QListWidget* axesList;
     QWidget* gridPage;
     QCheckBox* boxMajorGrid;
@@ -191,7 +191,6 @@ protected:
 
 	QStringList tickLabelsOn, tablesList;
 	QList<int> majTicks, minTicks, axesBaseline;
-	QFont xBottomFont, yLeftFont, xTopFont, yRightFont;
 	bool xAxisOn, yAxisOn, topAxisOn, rightAxisOn;
 	int xBottomLabelsRotation, xTopLabelsRotation;
 
@@ -208,6 +207,11 @@ protected:
     QTimeEdit *boxStartTime, *boxEndTime;
     QComboBox *canvasFrameApplyToBox;
     QPushButton *frameDefaultBtn;
+	QSpinBox *boxLabelsDistance;
+	QGroupBox *axisFormatBox;
+	QComboBox *axisFormatApplyToBox;
+	QDateTimeEdit *originDateTimeBox;
+	QCheckBox *invertTitleBox;
 };
 
 #endif
