@@ -2,8 +2,8 @@
     File                 : CurvesDialog.cpp
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2006 by Ion Vasilief, Tilman Hoener zu Siederdissen
-    Email (use @ for *)  : ion_vasilief*yahoo.fr, thzs*gmx.net
+    Copyright            : (C) 2006 by Ion Vasilief
+    Email (use @ for *)  : ion_vasilief*yahoo.fr
     Description          : Add/remove curves dialog
 
  ***************************************************************************/
@@ -27,14 +27,16 @@
  *                                                                         *
  ***************************************************************************/
 #include "CurvesDialog.h"
-#include "../Graph.h"
-#include "../../Table.h"
-#include "../../matrix/Matrix.h"
-#include "../FunctionCurve.h"
-#include "../PlotCurve.h"
-#include "../../ApplicationWindow.h"
-#include "../../Folder.h"
-#include "../../pixmaps.h"
+#include "QwtHistogram.h"
+#include <Graph.h>
+#include <Table.h>
+#include <Matrix.h>
+#include <FunctionCurve.h>
+#include <PlotCurve.h>
+#include <ApplicationWindow.h>
+#include <Folder.h>
+#include <pixmaps.h>
+#include <ColorBox.h>
 
 #include <QPushButton>
 #include <QLabel>
@@ -47,6 +49,7 @@
 #include <QShortcut>
 #include <QKeySequence>
 #include <QMenu>
+#include <QTreeWidget>
 
 #include <QMessageBox>
 
@@ -72,6 +75,7 @@ CurvesDialog::CurvesDialog( QWidget* parent, Qt::WFlags fl )
 	boxStyle->addItem( QPixmap(area_xpm), tr( " Area" ) );
 	boxStyle->addItem( QPixmap(vertBars_xpm), tr( " Vertical Bars" ) );
 	boxStyle->addItem( QPixmap(hBars_xpm), tr( " Horizontal Bars" ) );
+	boxStyle->addItem( QPixmap(histogram_xpm), tr( " Histogram" ) );
     hl->addWidget(boxStyle);
 
     boxMatrixStyle = new QComboBox();
@@ -79,15 +83,30 @@ CurvesDialog::CurvesDialog( QWidget* parent, Qt::WFlags fl )
 	boxMatrixStyle->addItem( QPixmap(contour_map_xpm), tr("Contour Lines"));
 	boxMatrixStyle->addItem( QPixmap(gray_map_xpm), tr("Gray Scale Map"));
 	boxMatrixStyle->addItem( QPixmap(histogram_xpm), tr("Histogram"));
-	boxMatrixStyle->hide();
     hl->addWidget(boxMatrixStyle);
     hl->addStretch();
 
     QGridLayout *gl = new QGridLayout();
     gl->addWidget(new QLabel( tr( "Available data" )), 0, 0);
-    gl->addWidget(new QLabel( tr( "Graph contents" )), 0, 2);
 
-	available = new QListWidget();
+    QHBoxLayout *hbc = new QHBoxLayout;
+    hbc->addWidget(new QLabel( tr( "Graph contents" )));
+
+    btnUp = new QPushButton();
+    btnUp->setIcon(QPixmap(arrow_up_xpm));
+    btnUp->setMaximumWidth(20);
+    hbc->addWidget(btnUp);
+	btnDown = new QPushButton();
+	btnDown->setIcon(QPixmap(arrow_down_xpm));
+	btnDown->setMaximumWidth(20);
+	hbc->addWidget(btnDown);
+	hbc->addStretch();
+	gl->addLayout(hbc, 0, 2);
+
+	available = new QTreeWidget();
+	available->setColumnCount(1);
+	available->header()->hide();
+    available->setIndentation(15);
 	available->setSelectionMode (QAbstractItemView::ExtendedSelection);
     gl->addWidget(available, 1, 0);
 
@@ -145,6 +164,9 @@ CurvesDialog::CurvesDialog( QWidget* parent, Qt::WFlags fl )
 
     init();
 
+	connect(btnUp, SIGNAL(clicked()),this, SLOT(raiseCurve()));
+	connect(btnDown, SIGNAL(clicked()),this, SLOT(shiftCurveBy()));
+
 	connect(boxShowCurrentFolder, SIGNAL(toggled(bool)), this, SLOT(showCurrentFolder(bool)));
     connect(boxShowRange, SIGNAL(toggled(bool)), this, SLOT(showCurveRange(bool)));
 	connect(btnRange, SIGNAL(clicked()),this, SLOT(showCurveRangeDialog()));
@@ -155,7 +177,7 @@ CurvesDialog::CurvesDialog( QWidget* parent, Qt::WFlags fl )
 	connect(btnOK, SIGNAL(clicked()),this, SLOT(close()));
 	connect(btnCancel, SIGNAL(clicked()),this, SLOT(close()));
 	connect(contents, SIGNAL(currentRowChanged(int)), this, SLOT(showCurveBtn(int)));
-    connect(contents, SIGNAL(itemSelectionChanged()), this, SLOT(enableRemoveBtn()));
+    connect(contents, SIGNAL(itemSelectionChanged()), this, SLOT(enableContentsBtns()));
     connect(available, SIGNAL(itemSelectionChanged()), this, SLOT(enableAddBtn()));
 
     QShortcut *shortcut = new QShortcut(QKeySequence(Qt::Key_Delete), this);
@@ -244,11 +266,22 @@ void CurvesDialog::contextMenuEvent(QContextMenuEvent *e)
 	QRect rect = available->visualItemRect(available->currentItem());
 	if (rect.contains(pos))
 	{
+       QList<QTreeWidgetItem *> lst = available->selectedItems();
+       int count = 0;
+	   foreach (QTreeWidgetItem *item, lst){
+			if (item->type() == FolderItem)
+				continue;
+
+			count++;
+	   }
+
+	   if (!count)
+		return;
+
 	   QMenu contextMenu(this);
-       QList<QListWidgetItem *> lst = available->selectedItems();
-       if (lst.size() > 1)
+       if (count > 1)
 	       contextMenu.insertItem(tr("&Plot Selection"), this, SLOT(addCurves()));
-       else if (lst.size() == 1)
+       else if (count == 1)
 	       contextMenu.insertItem(tr("&Plot"), this, SLOT(addCurves()));
 	   contextMenu.exec(QCursor::pos());
     }
@@ -277,12 +310,6 @@ void CurvesDialog::init()
         boxShowCurrentFolder->setChecked(currentFolderOnly);
 		showCurrentFolder(currentFolderOnly);
 
-        QStringList matrices = app->matrixNames();
-        if (!matrices.isEmpty ()){
-            boxMatrixStyle->show();
-            available->addItems(matrices);
-        }
-
         int style = app->defaultCurveStyle;
         if (style == Graph::Line)
             boxStyle->setCurrentItem(0);
@@ -306,7 +333,7 @@ void CurvesDialog::init()
             boxStyle->setCurrentItem(9);
     }
 
-	if (!available->count())
+	if (!available->topLevelItemCount())
 		btnAdd->setDisabled(true);
 }
 
@@ -314,21 +341,42 @@ void CurvesDialog::setGraph(Graph *graph)
 {
 	d_graph = graph;
 	contents->addItems(d_graph->plotItemsList());
-	enableRemoveBtn();
+	enableContentsBtns();
     enableAddBtn();
 }
 
 void CurvesDialog::addCurves()
 {
 	QStringList emptyColumns;
-    QList<QListWidgetItem *> lst = available->selectedItems();
-    for (int i = 0; i < lst.size(); ++i){
-        QString text = lst.at(i)->text();
+    QList<QTreeWidgetItem *> lst = available->selectedItems ();
+    foreach (QTreeWidgetItem *item, lst){
+		if (item->type() == FolderItem)
+			continue;
+
+		QString text = item->text(0);
+		if (item->type() == TableItem){
+			ApplicationWindow *app = (ApplicationWindow *)this->parent();
+			Table *t = app->table(text);
+			if (!t)
+				continue;
+
+			QStringList lst = t->YColumns();
+			for(int i = 0; i < lst.size(); i++){
+				QString s = lst[i];
+				if (contents->findItems(s, Qt::MatchExactly ).isEmpty ()){
+					if (!addCurve(s))
+						emptyColumns << s;
+				}
+			}
+			continue;
+		}
+
         if (contents->findItems(text, Qt::MatchExactly ).isEmpty ()){
 			if (!addCurve(text))
 				emptyColumns << text;
-			}
+		}
     }
+
 	d_graph->updatePlot();
 	Graph::showPlotErrorMessage(this, emptyColumns);
 
@@ -366,45 +414,63 @@ bool CurvesDialog::addCurve(const QString& name)
 		return true;
     }
 
-	int style = curveStyle();
 	Table* t = app->table(name);
-	if (t){
-		PlotCurve *c = d_graph->insertCurve(t, name, style);
-		CurveLayout cl = Graph::initCurveLayout();
-		int color, symbol;
-		d_graph->guessUniqueCurveLayout(color, symbol);
+	if (!t)
+		return false;
 
-		cl.lCol = color;
-		cl.symCol = color;
-		cl.fillCol = color;
-		cl.lWidth = app->defaultCurveLineWidth;
-		cl.sSize = app->defaultSymbolSize;
-		cl.sType = symbol;
+	int style = curveStyle();
+	DataCurve *c = NULL;
+	if (style == Graph::Histogram){
+		c = new QwtHistogram(t, name);
+		if (c){
+			d_graph->insertCurve(c);
+			((QwtHistogram *)c)->loadData();
+			d_graph->addLegendItem();
+		}
+	} else
+		c = d_graph->insertCurve(t, name, style);
 
-		if (style == Graph::Line)
-			cl.sType = 0;
-		else if (style == Graph::VerticalBars || style == Graph::HorizontalBars ){
-			cl.filledArea = 1;
-			cl.lCol = 0;
-			cl.aCol = color;
-			cl.sType = 0;
-		} else if (style == Graph::Area ){
-			cl.filledArea = 1;
-			cl.aCol = color;
-			cl.sType = 0;
-		} else if (style == Graph::VerticalDropLines)
-			cl.connectType = 2;
-		else if (style == Graph::VerticalSteps || style == Graph::HorizontalSteps){
-			cl.connectType = 3;
-			cl.sType = 0;
-		} else if (style == Graph::Spline)
-			cl.connectType = 5;
+	if (!c)
+		return false;
 
-		d_graph->updateCurveLayout(c, &cl);
-		contents->addItem(name);
-		return true;
-	}
-	return false;
+	CurveLayout cl = Graph::initCurveLayout();
+	int color, symbol;
+	d_graph->guessUniqueCurveLayout(color, symbol);
+
+	cl.lCol = ColorBox::color(color);
+	cl.symCol = cl.lCol;
+	cl.fillCol = cl.lCol;
+	cl.lWidth = app->defaultCurveLineWidth;
+	cl.sSize = app->defaultSymbolSize;
+	cl.sType = symbol;
+
+	if (style == Graph::Line)
+		cl.sType = 0;
+	else if (style == Graph::Histogram){
+		cl.filledArea = 1;
+		cl.aCol = cl.lCol;
+		cl.aStyle = 4;
+		cl.sType = 0;
+	} else if (style == Graph::VerticalBars || style == Graph::HorizontalBars){
+		cl.filledArea = 1;
+		cl.aCol = cl.lCol;
+		cl.lCol = Qt::black;
+		cl.sType = 0;
+	} else if (style == Graph::Area){
+		cl.filledArea = 1;
+		cl.aCol = cl.lCol;
+		cl.sType = 0;
+	} else if (style == Graph::VerticalDropLines)
+		cl.connectType = 2;
+	else if (style == Graph::VerticalSteps || style == Graph::HorizontalSteps){
+		cl.connectType = 3;
+		cl.sType = 0;
+	} else if (style == Graph::Spline)
+		cl.connectType = 5;
+
+	d_graph->updateCurveLayout(c, &cl);
+	contents->addItem(name);
+	return true;
 }
 
 void CurvesDialog::removeCurves()
@@ -426,12 +492,18 @@ void CurvesDialog::removeCurves()
 
 void CurvesDialog::enableAddBtn()
 {
-    btnAdd->setEnabled (available->count()>0 && !available->selectedItems().isEmpty());
+    btnAdd->setEnabled (available->topLevelItemCount() > 0 &&
+						!available->selectedItems().isEmpty());
 }
 
-void CurvesDialog::enableRemoveBtn()
+void CurvesDialog::enableContentsBtns()
 {
-    btnRemove->setEnabled (contents->count()>0 && !contents->selectedItems().isEmpty());
+	QList<QListWidgetItem *> lst = contents->selectedItems();
+    btnRemove->setEnabled (contents->count()>0 && !lst.isEmpty());
+
+    int row = contents->currentRow();
+	btnUp->setEnabled (lst.size() == 1 && row > 0);
+	btnDown->setEnabled (lst.size() == 1 && row < contents->count() - 1);
 }
 
 int CurvesDialog::curveStyle()
@@ -469,6 +541,9 @@ int CurvesDialog::curveStyle()
 		case 9:
 			style = Graph::HorizontalBars;
 			break;
+		case 10:
+			style = Graph::Histogram;
+			break;
 	}
 	return style;
 }
@@ -496,7 +571,7 @@ void CurvesDialog::showCurveRange(bool on )
         contents->addItems(d_graph->plotItemsList());
 
     contents->setCurrentRow(row);
-    enableRemoveBtn();
+    enableContentsBtns();
 }
 
 void CurvesDialog::updateCurveRange()
@@ -515,23 +590,69 @@ void CurvesDialog::showCurrentFolder(bool currentFolder)
 
     if (currentFolder){
     	Folder *f = app->currentFolder();
-		if (f){
-			QStringList columns;
-			foreach (QWidget *w, f->windowsList()){
-				if (!w->inherits("Table"))
-					continue;
+		if (f)
+			addFolderItems(f);
+    } else {
+    	Folder *f = app->projectFolder();
+		addFolderItems(f);
 
-				Table *t = (Table *)w;
-				for (int i=0; i < t->numCols(); i++){
-					if(t->colPlotDesignation(i) == Table::Y)
-						columns << QString(t->objectName()) + "_" + t->colLabel(i);
-				}
-			}
-			available->addItems(columns);
+		f = f->folderBelow();
+		QTreeWidgetItem *folderItem = NULL;
+		int depth = 0;
+		while (f){
+			if (folderItem && f->depth() > depth)
+				folderItem = new QTreeWidgetItem(folderItem, QStringList(f->objectName()), FolderItem);
+			else
+				folderItem = new QTreeWidgetItem(available, QStringList(f->objectName()), FolderItem);
+
+			folderItem->setIcon(0, QIcon(QPixmap(folder_open)));
+			folderItem->setExpanded(true);
+			available->addTopLevelItem(folderItem);
+
+			addFolderItems(f, folderItem);
+			depth = f->depth();
+			f = f->folderBelow();
 		}
     }
-	else
-        available->addItems(app->columnsList(Table::Y));
+}
+
+void CurvesDialog::addFolderItems(Folder *f, QTreeWidgetItem* parent)
+{
+	if (!f)
+		return;
+
+	foreach (MdiSubWindow *w, f->windowsList()){
+		if (w->inherits("Table")){
+			Table *t = (Table *)w;
+
+			QTreeWidgetItem *tableItem;
+			if (!parent)
+				tableItem = new QTreeWidgetItem(available, QStringList(t->objectName()), TableItem);
+			else
+				tableItem = new QTreeWidgetItem(parent, QStringList(t->objectName()), TableItem);
+			tableItem->setIcon(0, QIcon(QPixmap(worksheet_xpm)));
+			available->addTopLevelItem(tableItem);
+
+			for (int i=0; i < t->numCols(); i++){
+				if(t->colPlotDesignation(i) == Table::Y){
+					QTreeWidgetItem *colItem = new QTreeWidgetItem(tableItem, QStringList(t->objectName() + "_" + t->colLabel(i)), ColumnItem);
+					available->addTopLevelItem(colItem);
+				}
+			}
+
+			continue;
+		}
+		Matrix *m = qobject_cast<Matrix *>(w);
+		if (m){
+			QTreeWidgetItem *item;
+			if (!parent)
+				item = new QTreeWidgetItem(available, QStringList(m->objectName()), MatrixItem);
+			else
+				item = new QTreeWidgetItem(parent, QStringList(m->objectName()), MatrixItem);
+			item->setIcon(0, QIcon(QPixmap(matrix_xpm)));
+			available->addTopLevelItem(item);
+		}
+	}
 }
 
 void CurvesDialog::closeEvent(QCloseEvent* e)
@@ -542,3 +663,21 @@ void CurvesDialog::closeEvent(QCloseEvent* e)
 
 	e->accept();
 }
+
+void CurvesDialog::raiseCurve()
+{
+	shiftCurveBy(-1);
+}
+
+void CurvesDialog::shiftCurveBy(int offset)
+{
+	if (!d_graph)
+		return;
+
+	int row = contents->currentRow();
+	d_graph->changeCurveIndex(row, row + offset);
+
+	contents->clear();
+	contents->addItems(d_graph->plotItemsList());
+	contents->setCurrentRow(row + offset);
+}
