@@ -31,20 +31,24 @@
 #include <QtGui>
 #include <QHttp>
 #include <QIODevice>
+#include <QNetworkProxy>
 
 #include "EnrichmentDialog.h"
-#include "../Graph.h"
-#include "../TexWidget.h"
-#include "../FrameWidget.h"
-#include "../ImageWidget.h"
-#include "../RectangleWidget.h"
-#include "../LegendWidget.h"
-#include "../../ColorButton.h"
-#include "../../ApplicationWindow.h"
-#include "../../DoubleSpinBox.h"
-#include "../../PatternBox.h"
-#include "../../PenStyleBox.h"
-#include "../../TextFormatButtons.h"
+#include <Graph.h>
+#include <MultiLayer.h>
+#include <TexWidget.h>
+#include <EllipseWidget.h>
+#include <FrameWidget.h>
+#include <ImageWidget.h>
+#include <RectangleWidget.h>
+#include <LegendWidget.h>
+#include <ApplicationWindow.h>
+#include <TextFormatButtons.h>
+#include <ColorButton.h>
+#include <DoubleSpinBox.h>
+#include <PatternBox.h>
+#include <PenStyleBox.h>
+#include <pixmaps.h>
 
 static const char* choose_folder_xpm[]={
     "16 16 11 1",
@@ -112,7 +116,7 @@ EnrichmentDialog::EnrichmentDialog(WidgetType wt, Graph *g, QWidget *parent)
 		initEditorPage();
 	else if (wt == Image)
 		initImagePage();
-	else if (wt == Frame)
+	else if (wt == Frame || wt == Ellipse)
 		initPatternPage();
 
     if (wt != MDIWindow)
@@ -131,19 +135,41 @@ void EnrichmentDialog::initEditorPage()
 {
 	http = new QHttp(this);
     connect(http, SIGNAL(done(bool)), this, SLOT(updateForm(bool)));
+    http->setHost("mathtran.org");
+	QNetworkProxy proxy = QNetworkProxy::applicationProxy();
+	if (!proxy.hostName().isEmpty())
+		http->setProxy(proxy.hostName(), proxy.port(), proxy.user(), proxy.password());
+
+	compileProcess = NULL;
+	dvipngProcess = NULL;
 
 	editPage = new QWidget();
 
     equationEditor = new QTextEdit;
+
+	texFormatButtons = new TextFormatButtons(equationEditor, TextFormatButtons::Equation);
+
+	texCompilerBox = new QComboBox;
+	texCompilerBox->addItem(tr("MathTran (http://www.mathtran.org/)"));
+	texCompilerBox->addItem(tr("locally installed"));
+	texCompilerBox->setCurrentIndex(((ApplicationWindow *)parentWidget())->d_latex_compiler);
+	connect(texCompilerBox, SIGNAL(activated(int)), this, SLOT(updateCompilerInterface(int)));
+
+	QHBoxLayout *hl = new QHBoxLayout;
+	hl->addWidget(new QLabel(tr("LaTeX Compiler")));
+	hl->addWidget(texCompilerBox);
+
 	outputLabel = new QLabel;
     outputLabel->setFrameShape(QFrame::StyledPanel);
 
 	QVBoxLayout *layout = new QVBoxLayout(editPage);
     layout->addWidget(equationEditor, 1);
+	layout->addWidget(texFormatButtons);
+	layout->addLayout(hl);
 	layout->addWidget(new QLabel(tr("Preview:")));
 	layout->addWidget(outputLabel);
 
-	tabWidget->addTab(editPage, tr( "&Text" ) );
+	tabWidget->addTab(editPage, tr("&Text" ));
 }
 
 void EnrichmentDialog::initTextPage()
@@ -188,6 +214,10 @@ void EnrichmentDialog::initTextPage()
     autoUpdateTextBox = new QCheckBox(tr("Auto-&update"));
 	gl1->addWidget(autoUpdateTextBox, 1, 2);
 
+	texOutputBox = new QCheckBox(tr("TeX &Output"));
+	gl1->addWidget(texOutputBox, 2, 2);
+	connect(texOutputBox, SIGNAL(clicked()), this, SLOT(updateButtons()));
+
 	gl1->setColumnStretch(4, 1);
 
     QVBoxLayout *vl = new QVBoxLayout();
@@ -195,7 +225,7 @@ void EnrichmentDialog::initTextPage()
     connect(textDefaultBtn, SIGNAL(clicked()), this, SLOT(setTextDefaultValues()));
 	vl->addWidget(textDefaultBtn);
 
-    textApplyToBtn = new QPushButton(tr("Apply &to..."));
+    textApplyToBtn = new QPushButton(tr("Apply format &to..."));
 	connect(textApplyToBtn, SIGNAL(clicked()), this, SLOT(textFormatApplyTo()));
 	vl->addWidget(textApplyToBtn);
 
@@ -213,10 +243,8 @@ void EnrichmentDialog::initTextPage()
 
 	textEditBox = new QTextEdit();
 	textEditBox->setTextFormat(Qt::PlainText);
-	textEditBox->setFont(QFont());
 
-	formatButtons =  new TextFormatButtons(textEditBox);
-	formatButtons->toggleCurveButton(true);
+	formatButtons =  new TextFormatButtons(textEditBox, TextFormatButtons::Legend);
 
 	setFocusPolicy(Qt::StrongFocus);
 	setFocusProxy(textEditBox);
@@ -268,9 +296,13 @@ void EnrichmentDialog::initFramePage()
     gl->addWidget(new QLabel( tr("Shape")), 0, 0);
 
 	frameBox = new QComboBox();
-	frameBox->addItem( tr( "None" ) );
-	frameBox->addItem( tr( "Rectangle" ) );
-	frameBox->addItem( tr( "Shadow" ) );
+	frameBox->addItem(tr("None"));
+	if (d_widget_type == Ellipse)
+		frameBox->addItem(tr("Line"));
+	else {
+		frameBox->addItem(tr("Rectangle"));
+		frameBox->addItem(tr("Shadow"));
+	}
 	connect(frameBox, SIGNAL(activated(int)), this, SLOT(frameApplyTo()));
     gl->addWidget(frameBox, 0, 1);
 
@@ -286,9 +318,19 @@ void EnrichmentDialog::initFramePage()
 
 	gl->setColumnStretch(1, 1);
 	gl->addWidget(new QLabel(tr("Width")), 3, 0);
-	boxFrameWidth = new QSpinBox();
-	boxFrameWidth->setRange(1, 100);
-	connect(boxFrameWidth, SIGNAL(valueChanged(int)), this, SLOT(frameApplyTo()));
+	boxFrameWidth = new DoubleSpinBox();
+	if(d_widget_type == Ellipse){
+		boxFrameWidth->setDecimals(2);
+		boxFrameWidth->setLocale(((ApplicationWindow *)parent())->locale());
+		boxFrameWidth->setSingleStep(0.1);
+		boxFrameWidth->setRange(0.1, 100);
+	} else {
+		boxFrameWidth->setRange(1, 100);
+		boxFrameWidth->setDecimals(0);
+		boxFrameWidth->setSingleStep(1.0);
+	}
+
+	connect(boxFrameWidth, SIGNAL(valueChanged(double)), this, SLOT(frameApplyTo()));
 	gl->addWidget(boxFrameWidth, 3, 1);
 	gl->setRowStretch(4, 1);
 
@@ -384,6 +426,16 @@ void EnrichmentDialog::initGeometryPage()
 {
     geometryPage = new QWidget();
 
+	QGridLayout *bl1 = new QGridLayout();
+
+	attachToBox = new QComboBox();
+	attachToBox->insertItem(tr("Page"));
+	attachToBox->insertItem(tr("Layer Scales"));
+	//attachToBox->setCurrentIndex((int)lm->attachPolicy());
+
+	bl1->addWidget(new QLabel(tr( "Attach to" )), 0, 0);
+	bl1->addWidget(attachToBox, 0, 1);
+
 	unitBox = new QComboBox();
 	unitBox->insertItem(tr("inch"));
 	unitBox->insertItem(tr("mm"));
@@ -393,9 +445,8 @@ void EnrichmentDialog::initGeometryPage()
 	if (d_widget_type != MDIWindow)
         unitBox->insertItem(tr("scale"));
 
-	QBoxLayout *bl1 = new QBoxLayout (QBoxLayout::LeftToRight);
-	bl1->addWidget(new QLabel(tr( "Unit" )));
-	bl1->addWidget(unitBox);
+	bl1->addWidget(new QLabel(tr( "Unit" )), 1, 0);
+	bl1->addWidget(unitBox, 1, 1);
 
 	ApplicationWindow *app = (ApplicationWindow *)parent();
 	QLocale locale = QLocale();
@@ -482,6 +533,10 @@ void EnrichmentDialog::setWidget(QWidget *w)
 	if (!w)
 		return;
 
+	ApplicationWindow *app = (ApplicationWindow *)parent();
+	if (!app)
+		return;
+
 	d_widget = w;
 
     FrameWidget *fw = qobject_cast<FrameWidget *>(d_widget);
@@ -489,28 +544,38 @@ void EnrichmentDialog::setWidget(QWidget *w)
 		frameBox->blockSignals(true);
         frameBox->setCurrentIndex(fw->frameStyle());
 		frameBox->blockSignals(false);
-		
+
 		frameColorBtn->blockSignals(true);
         frameColorBtn->setColor(fw->frameColor());
 		frameColorBtn->blockSignals(false);
-		
+
 		boxFrameLineStyle->blockSignals(true);
 		boxFrameLineStyle->setStyle(fw->framePen().style());
 		boxFrameLineStyle->blockSignals(false);
-		
-		boxFrameWidth->blockSignals(true);
-		boxFrameWidth->setValue(fw->framePen().width());
-		boxFrameWidth->blockSignals(false);
-    }
 
-    unitBox->setCurrentIndex(FrameWidget::Pixel);
-	displayCoordinates(FrameWidget::Pixel);
+		boxFrameWidth->blockSignals(true);
+		if (d_widget_type == Ellipse)
+			boxFrameWidth->setValue(fw->framePen().widthF());
+		else
+			boxFrameWidth->setValue(fw->framePen().width());
+		boxFrameWidth->blockSignals(false);
+
+		unitBox->setCurrentIndex(app->d_frame_geometry_unit);
+		attachToBox->setCurrentIndex((int)fw->attachPolicy());
+		displayCoordinates(app->d_frame_geometry_unit);
+    } else {
+		unitBox->setCurrentIndex(FrameWidget::Pixel);
+		displayCoordinates(FrameWidget::Pixel);
+	}
 
 	if (d_widget_type == Text){
 		LegendWidget *l = qobject_cast<LegendWidget *>(d_widget);
 		if (l){
-			setText(l->text());
+			setText(textEditBox, l->text());
 			textFont = l->font();
+			QFont fnt = textFont;
+			fnt.setPointSize(QFont().pointSize() + 2);
+			textEditBox->setFont(fnt);
 			textColorBtn->blockSignals(true);
 			textColorBtn->setColor(l->textColor());
 			textColorBtn->blockSignals(false);
@@ -519,7 +584,7 @@ void EnrichmentDialog::setWidget(QWidget *w)
 			boxBackgroundTransparency->blockSignals(true);
 			boxBackgroundTransparency->setValue(bc.alpha());
 			boxBackgroundTransparency->blockSignals(false);
-			
+
 			textBackgroundBtn->blockSignals(true);
 			textBackgroundBtn->setEnabled(bc.alpha());
 			textBackgroundBtn->setColor(bc);
@@ -529,11 +594,13 @@ void EnrichmentDialog::setWidget(QWidget *w)
             boxTextAngle->setValue(l->angle());
 			boxTextAngle->blockSignals(false);
 			autoUpdateTextBox->setChecked(l->isAutoUpdateEnabled());
+			texOutputBox->setChecked(l->hasTeXOutput());
+			updateButtons();
 		}
 	} else if (d_widget_type == Tex){
 		TexWidget *tw = qobject_cast<TexWidget *>(d_widget);
 		if (tw){
-			equationEditor->setText(tw->formula());
+			setText(equationEditor, tw->formula());
 			outputLabel->setPixmap(tw->pixmap());
 			bestSizeButton->show();
 		}
@@ -552,15 +619,34 @@ void EnrichmentDialog::setWidget(QWidget *w)
 			backgroundColorBtn->blockSignals(true);
 			backgroundColorBtn->setColor(r->backgroundColor());
 			backgroundColorBtn->blockSignals(false);
-			
+
 			boxTransparency->blockSignals(true);
 			boxTransparency->setValue(r->backgroundColor().alpha());
 			boxTransparency->blockSignals(false);
-			
+
 			patternBox->blockSignals(true);
 			patternBox->setPattern(r->brush().style());
 			patternBox->blockSignals(false);
-			
+
+			patternColorBtn->blockSignals(true);
+			patternColorBtn->setColor(r->brush().color());
+			patternColorBtn->blockSignals(false);
+		}
+	} else if (d_widget_type == Ellipse){
+		EllipseWidget *r = qobject_cast<EllipseWidget *>(d_widget);
+		if (r){
+			backgroundColorBtn->blockSignals(true);
+			backgroundColorBtn->setColor(r->backgroundColor());
+			backgroundColorBtn->blockSignals(false);
+
+			boxTransparency->blockSignals(true);
+			boxTransparency->setValue(r->backgroundColor().alpha());
+			boxTransparency->blockSignals(false);
+
+			patternBox->blockSignals(true);
+			patternBox->setPattern(r->brush().style());
+			patternBox->blockSignals(false);
+
 			patternColorBtn->blockSignals(true);
 			patternColorBtn->setColor(r->brush().color());
 			patternColorBtn->blockSignals(false);
@@ -578,32 +664,53 @@ void EnrichmentDialog::apply()
 {
 	if (tabWidget->currentPage() == editPage)
 		fetchImage();
-	else if (tabWidget->currentPage() == framePage){
-	    FrameWidget *fw = qobject_cast<FrameWidget *>(d_widget);
-        if (fw){
-			setFrameTo(fw);
-			d_plot->multiLayer()->notifyChanges();
-		}
-	} else if (imagePage && tabWidget->currentPage() == imagePage)
+	else if (tabWidget->currentPage() == framePage)
+		frameApplyTo();
+	else if (imagePage && tabWidget->currentPage() == imagePage)
 		chooseImageFile(imagePathBox->text());
-	else if (tabWidget->currentPage() == geometryPage)
+	else if (tabWidget->currentPage() == geometryPage){
 		setCoordinates(unitBox->currentIndex());
-	else if (patternPage && tabWidget->currentPage() == patternPage){
-		RectangleWidget *r = qobject_cast<RectangleWidget *>(d_widget);
-        if (r){
-			setPatternTo(r);
-			d_plot->multiLayer()->notifyChanges();
-		}
-	} else if (textPage && tabWidget->currentPage() == textPage){
+		FrameWidget *fw = qobject_cast<FrameWidget *>(d_widget);
+        if (fw)
+            fw->setAttachPolicy((FrameWidget::AttachPolicy)attachToBox->currentIndex());
+	} else if (patternPage && tabWidget->currentPage() == patternPage)
+		patternApplyTo();
+	else if (textPage && tabWidget->currentPage() == textPage){
 		LegendWidget *l = qobject_cast<LegendWidget *>(d_widget);
-        if (l){
-            l->setText(textEditBox->text());
-			setTextFormatTo(l);
-			d_plot->multiLayer()->notifyChanges();
-		}
+		if (l)
+			l->setText(textEditBox->text());
+
+		textFormatApplyTo();
+		ApplicationWindow *app = (ApplicationWindow *)this->parent();
+		if (app)
+			app->setFormatBarFont(textFont);
 	}
 }
 
+QString EnrichmentDialog::createTempTexFile()
+{
+	QString path = QDir::tempPath();
+	QString name = path + "/" + "QtiPlot_temp.tex";
+
+	QFile file(name);
+
+	if (file.open(QIODevice::WriteOnly)){
+		QTextStream t( &file );
+		t.setEncoding(QTextStream::UnicodeUTF8);
+		t << "\\documentclass{article}\n";
+		t << "\\pagestyle{empty}\n";
+		t << "\\begin{document}\n";
+		t << "\\huge{\\mbox{$";
+		t << equationEditor->toPlainText();
+		t << "$}}\n";
+		t << "\\end{document}";
+		file.close();
+		return QDir::cleanPath(name);
+	}
+
+	return QString();
+}
+
 void EnrichmentDialog::fetchImage()
 {
 	TexWidget *tw = qobject_cast<TexWidget *>(d_widget);
@@ -614,6 +721,28 @@ void EnrichmentDialog::fetchImage()
     updateButton->setEnabled(false);
     equationEditor->setReadOnly(true);
 
+	if (texCompilerBox->currentIndex() == 1){
+		if (compileProcess)
+			delete compileProcess;
+
+		compileProcess = new QProcess(this);
+		connect(compileProcess, SIGNAL(finished(int, QProcess::ExitStatus)),
+			this, SLOT(finishedCompiling(int, QProcess::ExitStatus)));
+		connect(compileProcess, SIGNAL(error(QProcess::ProcessError)),
+			this, SLOT(displayCompileError(QProcess::ProcessError)));
+
+		compileProcess->setWorkingDirectory(QDir::tempPath());
+
+		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
+		QString program = ((ApplicationWindow *)parentWidget())->d_latex_compiler_path;
+		QStringList arguments;
+		arguments << createTempTexFile();
+
+		compileProcess->start(program, arguments);
+		return;
+	}
+
     QUrl url;
     url.setPath("/cgi-bin/mathtran");
     url.setQueryDelimiters('=', ';');
@@ -621,7 +750,6 @@ void EnrichmentDialog::fetchImage()
     url.addQueryItem("tex", QUrl::toPercentEncoding(
                      equationEditor->toPlainText()));
 
-    http->setHost("mathtran.org");
     http->get(url.toString());
 
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
@@ -662,7 +790,8 @@ void EnrichmentDialog::chooseImageFile(const QString& fn)
 
 	QString path = fn;
 	if (path.isEmpty())
-		path = QFileDialog::getOpenFileName(this, tr("QtiPlot - Import image from file"), app->imagesDirPath);
+		path = ApplicationWindow::getFileName(this, tr("QtiPlot - Import image from file"), app->imagesDirPath,
+					ApplicationWindow::imageFilter(), 0, false);
 
 	if (!path.isEmpty()){
 		ImageWidget *i = qobject_cast<ImageWidget *>(d_widget);
@@ -711,6 +840,10 @@ void EnrichmentDialog::setCoordinates(int unit)
 
     if (d_plot)
         d_plot->multiLayer()->notifyChanges();
+
+	ApplicationWindow *app = (ApplicationWindow *)parentWidget();
+	if (app)
+		app->d_frame_geometry_unit = unit;
 }
 
 void EnrichmentDialog::displayCoordinates(int unit)
@@ -789,7 +922,7 @@ void EnrichmentDialog::frameApplyTo()
 				setFrameTo(fw);
 		}
 		break;
-		
+
 		case 1://this layer
 		{
 			QList <FrameWidget *> lst = d_plot->enrichmentsList();
@@ -834,7 +967,7 @@ void EnrichmentDialog::frameApplyTo()
 
 void EnrichmentDialog::setFrameTo(FrameWidget *fw)
 {
-	fw->setFrameStyle(frameBox->currentIndex());	
+	fw->setFrameStyle(frameBox->currentIndex());
 	QPen pen = QPen(frameColorBtn->color(), boxFrameWidth->value(),
 				boxFrameLineStyle->style(), Qt::SquareCap, Qt::MiterJoin);
 	fw->setFramePen(pen);
@@ -847,20 +980,17 @@ void EnrichmentDialog::patternApplyTo()
 	switch(patternApplyToBox->currentIndex()){
 		case 0://this object
 		{
-			RectangleWidget *r = qobject_cast<RectangleWidget *>(d_widget);
+			FrameWidget *r = qobject_cast<FrameWidget *>(d_widget);
 			if (r)
 				setPatternTo(r);
 		}
 		break;
-		
+
 		case 1://this layer
 		{
 			QList <FrameWidget *> lst = d_plot->enrichmentsList();
-			foreach(FrameWidget *fw, lst){
-				RectangleWidget *r = qobject_cast<RectangleWidget *>(fw);
-				if (r)
-					setPatternTo(r);
-			}
+			foreach(FrameWidget *fw, lst)
+				setPatternTo(fw);
 		}
 		break;
 
@@ -869,11 +999,8 @@ void EnrichmentDialog::patternApplyTo()
 			QList<Graph *> layersLst = d_plot->multiLayer()->layersList();
 			foreach(Graph *g, layersLst){
 				QList <FrameWidget *> lst = g->enrichmentsList();
-				foreach(FrameWidget *fw, lst){
-					RectangleWidget *r = qobject_cast<RectangleWidget *>(fw);
-					if (r)
-						setPatternTo(r);
-				}
+				foreach(FrameWidget *fw, lst)
+						setPatternTo(fw);
 			}
 		}
 		break;
@@ -888,11 +1015,8 @@ void EnrichmentDialog::patternApplyTo()
 				QList<Graph *> layersLst = ml->layersList();
 				foreach(Graph *g, layersLst){
 					QList <FrameWidget *> lst = g->enrichmentsList();
-					foreach(FrameWidget *fw, lst){
-						RectangleWidget *r = qobject_cast<RectangleWidget *>(fw);
-						if (r)
-							setPatternTo(r);
-					}
+					foreach(FrameWidget *fw, lst)
+						setPatternTo(fw);
 				}
 			}
 		}
@@ -904,7 +1028,7 @@ void EnrichmentDialog::patternApplyTo()
 	app->modifiedProject();
 }
 
-void EnrichmentDialog::setPatternTo(RectangleWidget *r)
+void EnrichmentDialog::setPatternTo(FrameWidget *r)
 {
 	QColor c = backgroundColorBtn->color();
 	c.setAlpha(boxTransparency->value());
@@ -918,9 +1042,12 @@ void EnrichmentDialog::setPatternTo(RectangleWidget *r)
 	r->repaint();
 }
 
-void EnrichmentDialog::setText(const QString & t)
+void EnrichmentDialog::setText(QTextEdit *editor, const QString & t)
 {
-	QTextCursor cursor = textEditBox->textCursor();
+	if (!editor)
+		return;
+
+	QTextCursor cursor = editor->textCursor();
 	// select the whole (old) text
 	cursor.movePosition(QTextCursor::Start);
 	cursor.movePosition(QTextCursor::End,QTextCursor::KeepAnchor);
@@ -932,17 +1059,22 @@ void EnrichmentDialog::setText(const QString & t)
 	// this line makes the selection visible to the user
 	// (the 2 lines above only change the selection in the
 	// underlying QTextDocument)
-	textEditBox->setTextCursor(cursor);
+	editor->setTextCursor(cursor);
 	// give focus back to text edit
-	textEditBox->setFocus();
+	editor->setFocus();
 }
 
 void EnrichmentDialog::customFont()
 {
 	bool okF;
 	QFont fnt = QFontDialog::getFont( &okF, textFont, this);
-	if (okF && fnt != textFont)
+	if (okF && fnt != textFont){
 		textFont = fnt;
+
+		fnt.setPointSize(QFont().pointSize() + 2);
+		textEditBox->setFont(fnt);
+	}
+
 }
 
 void EnrichmentDialog::updateTransparency(int alpha)
@@ -962,7 +1094,7 @@ void EnrichmentDialog::textFormatApplyTo()
 				setTextFormatTo(l);
 		}
 		break;
-		
+
 		case 1://this layer
 		{
 			QList <FrameWidget *> lst = d_plot->enrichmentsList();
@@ -1023,6 +1155,7 @@ void EnrichmentDialog::setTextFormatTo(LegendWidget *l)
     l->setFont(textFont);
     l->setAngle(boxTextAngle->value());
     l->setAutoUpdate(autoUpdateTextBox->isChecked());
+    l->setTeXOutput(texOutputBox->isChecked());
     l->repaint();
 }
 
@@ -1071,6 +1204,145 @@ void EnrichmentDialog::setRectangleDefaultValues()
 	app->saveSettings();
 }
 
+void EnrichmentDialog::createImage()
+{
+	QApplication::restoreOverrideCursor();
+
+	QString path = QDir::tempPath();
+	QString fileName = QDir::cleanPath(path + "/" + "QtiPlot_temp.png");
+
+	QImage image;
+	if (image.load(fileName)){
+		QPixmap pixmap = QPixmap::fromImage(image);
+		outputLabel->setPixmap(pixmap);
+		TexWidget *tw = qobject_cast<TexWidget *>(d_widget);
+		if (tw){
+			tw->setPixmap(pixmap);
+			tw->setFormula(equationEditor->toPlainText());
+			d_plot->multiLayer()->notifyChanges();
+		}
+	}
+
+    clearButton->setEnabled(true);
+    updateButton->setEnabled(true);
+    equationEditor->setReadOnly(false);
+
+	if (dvipngProcess)
+		delete dvipngProcess;
+    dvipngProcess = NULL;
+    QFile::remove(fileName);
+
+    fileName = QDir::cleanPath(path + "/" + "QtiPlot_temp.dvi");
+    QFile::remove(fileName);
+}
+
+void EnrichmentDialog::finishedCompiling(int exitCode, QProcess::ExitStatus exitStatus)
+{
+	QApplication::restoreOverrideCursor();
+	if (exitStatus != QProcess::NormalExit){
+		QMessageBox::critical(this, tr("Compile process ended"),
+		tr("Compiling process ended with exit code: %1").arg(exitCode));
+		return;
+	}
+
+	QString compiler = ((ApplicationWindow *)parentWidget())->d_latex_compiler_path;
+	QFileInfo fi(compiler);
+
+	QString dir = fi.dir().absolutePath();
+        QString program = dir + "/dvipng";
+        #ifdef Q_OS_WIN
+            program += ".exe";
+        #endif
+
+	QStringList arguments;
+	arguments << "-T" << "tight" << "QtiPlot_temp.dvi" << "-o" << "QtiPlot_temp.png";
+
+	if (compileProcess)
+		delete compileProcess;
+	compileProcess = NULL;
+
+	dvipngProcess = new QProcess(this);
+	connect(dvipngProcess, SIGNAL(error(QProcess::ProcessError)),
+		this, SLOT(displayCompileError(QProcess::ProcessError)));
+	connect(dvipngProcess, SIGNAL(finished(int, QProcess::ExitStatus)),
+		this, SLOT(createImage()));
+
+	dvipngProcess->setWorkingDirectory (QDir::tempPath());
+
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+	dvipngProcess->start(program, arguments);
+	return;
+}
+
+void EnrichmentDialog::displayCompileError(QProcess::ProcessError error)
+{
+	QString process = QString::null;
+	if (compileProcess)
+		process = tr("LaTeX compile process");
+	else if (dvipngProcess)
+		process = tr("dvipng process");
+
+	QString msg;
+	switch(error){
+		case QProcess::FailedToStart:
+			if (compileProcess)
+				msg = process + " " + tr("failed to start!") + " " +
+				tr("Please set the correct path to the compiler in the preferences dialog!");
+			else if (dvipngProcess)
+				msg = process + " " + tr("failed to start!") + " " +
+				tr("Please verify that you have dvipng installed in the same folder as your LaTeX compiler!");
+		break;
+
+		case QProcess::Crashed:
+			msg = process + " " + tr("crashed");
+		break;
+
+		case QProcess::Timedout:
+			msg = process + " " + tr("timedout");
+		break;
+
+		case QProcess::WriteError:
+			msg = process + " " + tr("write error");
+		break;
+
+		case QProcess::ReadError:
+			msg = process + " " + tr("read error");
+		break;
+
+		case QProcess::UnknownError:
+			msg =  process + " " + tr("unknown error");
+		break;
+	}
+
+	QApplication::restoreOverrideCursor();
+	if (compileProcess){
+		compileProcess->kill();
+		compileProcess = NULL;
+	} else if (dvipngProcess){
+		dvipngProcess->kill();
+		dvipngProcess = NULL;
+	}
+
+	QMessageBox::critical(this, tr("Compile error"), msg);
+
+	clearButton->setEnabled(true);
+    updateButton->setEnabled(true);
+    equationEditor->setReadOnly(false);
+}
+
+void EnrichmentDialog::updateCompilerInterface(int compiler)
+{
+	((ApplicationWindow *)parentWidget())->d_latex_compiler = compiler;
+}
+
+void EnrichmentDialog::updateButtons()
+{
+	if (texOutputBox->isChecked())
+		formatButtons->setButtons(TextFormatButtons::TexLegend);
+	else
+		formatButtons->setButtons(TextFormatButtons::Legend);
+}
+
 EnrichmentDialog::~EnrichmentDialog()
 {
 	QApplication::restoreOverrideCursor();
