@@ -2,8 +2,8 @@
     File                 : FunctionDialog.cpp
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2006 by Ion Vasilief, Tilman Hoener zu Siederdissen
-    Email (use @ for *)  : ion_vasilief*yahoo.fr, thzs*gmx.net
+    Copyright            : (C) 2006 - 2009 by Ion Vasilief
+    Email (use @ for *)  : ion_vasilief*yahoo.fr
     Description          : Function dialog
 
  ***************************************************************************/
@@ -27,10 +27,11 @@
  *                                                                         *
  ***************************************************************************/
 #include "FunctionDialog.h"
-#include "../../MyParser.h"
-#include "../../ApplicationWindow.h"
-#include "../FunctionCurve.h"
-#include "../../DoubleSpinBox.h"
+#include <MyParser.h>
+#include <ApplicationWindow.h>
+#include <MultiLayer.h>
+#include <FunctionCurve.h>
+#include <DoubleSpinBox.h>
 
 #include <QTextEdit>
 #include <QLineEdit>
@@ -56,10 +57,11 @@ FunctionDialog::FunctionDialog( QWidget* parent, Qt::WFlags fl )
 		locale = app->locale();
 		prec = app->d_decimal_digits;
 	}
-	
+
     setObjectName( "FunctionDialog" );
 	setWindowTitle( tr( "QtiPlot - Add function curve" ) );
 	setSizeGripEnabled(true);
+	setAttribute(Qt::WA_DeleteOnClose);
 
 	QHBoxLayout *hbox1 = new QHBoxLayout();
 	hbox1->addWidget(new QLabel(tr( "Curve type " )));
@@ -108,13 +110,13 @@ FunctionDialog::FunctionDialog( QWidget* parent, Qt::WFlags fl )
     boxConstants->verticalHeader()->hide();
 	boxConstants->setMinimumWidth(200);
 	boxConstants->hide();
-	
+
 	functionPage = new QWidget();
 
 	QHBoxLayout *hb = new QHBoxLayout(functionPage);
 	hb->addLayout(gl1);
 	hb->addWidget(boxConstants);
-	
+
 	optionStack->addWidget( functionPage );
 
 	QGridLayout *gl2 = new QGridLayout();
@@ -183,6 +185,30 @@ FunctionDialog::FunctionDialog( QWidget* parent, Qt::WFlags fl )
 	polarPage->setLayout(gl3);
 	optionStack->addWidget( polarPage );
 
+	boxFunctionExplain = new QTextEdit;
+	boxFunctionExplain->setReadOnly(true);
+	boxFunctionExplain->setMaximumHeight(50);
+	QPalette palette = boxFunctionExplain->palette();
+	palette.setColor(QPalette::Active, QPalette::Base, Qt::lightGray);
+	boxFunctionExplain->setPalette(palette);
+
+	addFunctionBtn = new QPushButton(tr( "&Add Function" ));
+	addFunctionBtn->setAutoDefault(false);
+	connect(addFunctionBtn, SIGNAL(clicked()), this, SLOT(insertFunction()));
+
+	boxMathFunctions = new QComboBox();
+	boxMathFunctions->addItems(MyParser::functionsList());
+	connect(boxMathFunctions, SIGNAL(activated(int)), this, SLOT(updateFunctionExplain(int)));
+	updateFunctionExplain(0);
+
+	QVBoxLayout *vbox = new QVBoxLayout();
+	vbox->addWidget(boxMathFunctions);
+	vbox->addWidget(addFunctionBtn);
+
+	QHBoxLayout *hbox3 = new QHBoxLayout();
+	hbox3->addWidget(boxFunctionExplain);
+	hbox3->addLayout(vbox);
+
 	buttonClear = new QPushButton(tr( "Clea&r Function" ));
 	buttonClear->setAutoDefault(false);
 	buttonOk = new QPushButton(tr( "&Ok" ));
@@ -191,14 +217,15 @@ FunctionDialog::FunctionDialog( QWidget* parent, Qt::WFlags fl )
 	buttonCancel->setAutoDefault(false);
 
 	QHBoxLayout *hbox2 = new QHBoxLayout();
-	hbox2->addStretch();
 	hbox2->addWidget(buttonClear);
 	hbox2->addWidget(buttonOk);
 	hbox2->addWidget(buttonCancel);
+	hbox2->addStretch();
 
 	QVBoxLayout *vbox1 = new QVBoxLayout();
     vbox1->addLayout(hbox1);
 	vbox1->addWidget(optionStack);
+	vbox1->addLayout(hbox3);
 	vbox1->addLayout(hbox2);
 
 	setLayout(vbox1);
@@ -248,7 +275,7 @@ void FunctionDialog::setCurveToModify(Graph *g, int curve)
  		while (i.hasNext()) {
      		i.next();
 			boxConstants->setItem(row, 0, new QTableWidgetItem(i.key()));
-		
+
 			DoubleSpinBox *sb = new DoubleSpinBox();
 			sb->setLocale(app->locale());
 			sb->setDecimals(app->fit_output_precision);
@@ -258,7 +285,7 @@ void FunctionDialog::setCurveToModify(Graph *g, int curve)
 			row++;
  		}
 	}
-	
+
 	if (c->functionType() == FunctionCurve::Normal){
 		boxFunction->setText(formulas[0]);
 		boxFrom->setValue(c->startRange());
@@ -321,7 +348,7 @@ void FunctionDialog::acceptFunction()
 	}
 
 	QMap<QString, double> constants;
-			
+
 	QString formula = boxFunction->text().simplified();
 	bool error = false;
 	try {
@@ -334,7 +361,7 @@ void FunctionDialog::acceptFunction()
 			if (!constName.isEmpty()){
 				constants.insert(constName, val);
 				parser.DefineConst(constName.ascii(), val);
-			}			
+			}
 		}
 		parser.SetExpr(formula.ascii());
 
@@ -373,12 +400,12 @@ void FunctionDialog::acceptParametric()
 	QString to=boxParTo->text().lower();
 	QString points=boxParPoints->text().lower();
 
-	double start, end;
+	double start = 0.0, end = 0.0;
 	try
 	{
 		MyParser parser;
 		parser.SetExpr(from.ascii());
-		start=parser.Eval();
+		start = parser.Eval();
 	}
 	catch(mu::ParserError &e)
 	{
@@ -474,7 +501,7 @@ void FunctionDialog::acceptPolar()
 	QString to=boxPolarTo->text().lower();
 	QString points=boxPolarPoints->text().lower();
 
-	double start, end;
+	double start = 0, end = 0;
 	try
 	{
 		MyParser parser;
@@ -599,3 +626,34 @@ void FunctionDialog::insertPolarFunctionsList(const QStringList& rList, const QS
 	boxPolarRadius->insertItems (0, rList);
 	boxPolarTheta->insertItems (0, thetaList);
 }
+
+void FunctionDialog::insertFunction()
+{
+	QString fname = boxMathFunctions->currentText();
+	if (optionStack->currentWidget () == functionPage){
+		QTextCursor cursor = boxFunction->textCursor();
+		QString markedText = cursor.selectedText();
+		if(markedText.isEmpty()){
+			cursor.insertText(fname);
+			cursor.movePosition(QTextCursor::PreviousCharacter, QTextCursor::MoveAnchor,1);
+		} else
+			cursor.insertText(fname.remove(")").remove(",") + markedText + ")");
+
+		boxFunction->setTextCursor(cursor);
+	} else if (optionStack->currentWidget () == parametricPage){
+		if (boxYFunction->lineEdit()->hasFocus())
+			boxYFunction->lineEdit()->insert(fname);
+		else
+			boxXFunction->lineEdit()->insert(fname);
+	} else if (optionStack->currentWidget () == polarPage){
+		if (boxPolarRadius->lineEdit()->hasFocus())
+			boxPolarRadius->lineEdit()->insert(fname);
+		else
+			boxPolarTheta->lineEdit()->insert(fname);
+	}
+}
+
+void FunctionDialog::updateFunctionExplain(int index)
+{
+	boxFunctionExplain->setText(MyParser::explainFunction(index));
+}
