@@ -2,8 +2,8 @@
     File                 : PlotDialog.h
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2006 by Ion Vasilief, Tilman Hoener zu Siederdissen
-    Email (use @ for *)  : ion_vasilief*yahoo.fr, thzs*gmx.net
+    Copyright            : (C) 2006 by Ion Vasilief
+    Email (use @ for *)  : ion_vasilief*yahoo.fr
     Description          : Custom curves dialog
 
  ***************************************************************************/
@@ -31,7 +31,7 @@
 
 #include <QDialog>
 #include <QTreeWidgetItem>
-#include "../MultiLayer.h"
+#include <MultiLayer.h>
 
 class QCheckBox;
 class QComboBox;
@@ -59,6 +59,9 @@ class ColorMapEditor;
 class QwtPlotItem;
 class DoubleSpinBox;
 class PenStyleBox;
+class Spectrogram;
+class QwtErrorPlotCurve;
+class ContourLinesEditor;
 
 static const char* folder_closed[]={
     "16 16 9 1",
@@ -88,36 +91,6 @@ static const char* folder_closed[]={
     ".........##f#b##",
     "...........####."};
 
-static const char* folder_open[]={
-    "16 16 11 1",
-    "# c #000000",
-    "g c #c0c0c0",
-    "e c #303030",
-    "a c #ffa858",
-    "b c #808080",
-    "d c #a0a0a4",
-    "f c #585858",
-    "c c #ffdca8",
-    "h c #dcdcdc",
-    "i c #ffffff",
-    ". c None",
-    "....###.........",
-    "....#ab##.......",
-    "....#acab####...",
-    "###.#acccccca#..",
-    "#ddefaaaccccca#.",
-    "#bdddbaaaacccab#",
-    ".eddddbbaaaacab#",
-    ".#bddggdbbaaaab#",
-    "..edgdggggbbaab#",
-    "..#bgggghghdaab#",
-    "...ebhggghicfab#",
-    "....#edhhiiidab#",
-    "......#egiiicfb#",
-    "........#egiibb#",
-    "..........#egib#",
-    "............#ee#"};
-
 static const char * layer_disabled_xpm[] = {
 "14 14 3 1",
 " 	c None",
@@ -241,12 +214,7 @@ private slots:
 
 	void removeSelectedCurve();
 
-	/******* error bars options **************/
 	void pickErrorBarsColor();
-	void changeErrorBarsType();
-	void changeErrorBarsPlus();
-	void changeErrorBarsMinus();
-	void changeErrorBarsThrough();
 
 	void setAutomaticBinning();
 	bool validInput();
@@ -279,9 +247,22 @@ protected slots:
 	void chooseLabelsFont();
 	void applyLayerFormat();
 	void setLayerDefaultValues();
+    void setEquidistantLevels();
+    void showCustomPenColumn(bool on);
 
 private:
 	void applyFormatToLayer(Graph *g);
+	void applySymbolsFormatToCurve(QwtPlotCurve *c, bool fillColor = true, bool penColor = true);
+	void applySymbolsFormatToLayer(Graph *g);
+	void applySymbolsFormat(QwtPlotCurve *c);
+
+	void applyLineFormatToLayer(Graph *g);
+	void applyLineFormat(QwtPlotCurve *c);
+
+	void applyErrorBarFormatToCurve(QwtErrorPlotCurve *err, bool color = true);
+	void applyErrorBarFormatToLayer(Graph *g);
+	void applyErrorBarFormat(QwtErrorPlotCurve *c);
+
     int labelsAlignment();
 	void closeEvent(QCloseEvent* e);
 
@@ -296,8 +277,11 @@ private:
 	void initBoxPage();
 	void initPercentilePage();
 	void initSpectrogramPage();
+	void initSpectrogramValuesPage();
+	void initContourLinesPage();
 	void initLayerPage();
 	void initLayerGeometryPage();
+	void initLayerSpeedPage();
 	void initFontsPage();
 	void initPiePage();
 	void initPieGeometryPage();
@@ -305,6 +289,8 @@ private:
 	void initPrintPage();
 	void initLabelsPage();
     void contextMenuEvent(QContextMenuEvent *e);
+    void showAllLabelControls(bool show = true);
+    void updateContourLevelsDisplay(Spectrogram *sp);
 
 	double aspect_ratio;
 
@@ -318,14 +304,15 @@ private:
 
     QPushButton *btnTitle, *btnAxesLabels, *btnAxesNumbers, *btnLegend;
 	ColorMapEditor *colorMapEditor;
-    QWidget *curvePlotTypeBox, *layerPage, *layerGeometryPage, *piePage, *fontsPage, *printPage;
+    QWidget *curvePlotTypeBox, *layerPage, *layerGeometryPage, *piePage, *fontsPage, *printPage, *speedPage;
     QTreeWidget* listBox;
     QCheckBox *boxAntialiasing, *boxScaleLayers, *boxPrintCrops;
     ColorButton *boxBorderColor, *boxBackgroundColor, *boxCanvasColor;
 	QSpinBox *boxBackgroundTransparency, *boxCanvasTransparency, *boxBorderWidth, *boxMargin;
 	QSpinBox *boxRadius;
 	DoubleSpinBox *boxPieLineWidth;
-    ColorBox *boxFirstColor, *boxPieLineColor;
+    ColorBox *boxFirstColor;
+    ColorButton *boxPieLineColor;
     PatternBox *boxPiePattern;
     PenStyleBox* boxPieLineStyle;
 
@@ -337,20 +324,19 @@ private:
     QComboBox* boxConnect;
     PenStyleBox* boxLineStyle;
     DoubleSpinBox *boxLineWidth, *boxPenWidth;
-    ColorBox* boxLineColor, *boxAreaColor;
+    ColorButton* boxLineColor, *boxAreaColor;
     QWidget* symbolPage;
     QSpinBox* boxSymbolSize;
-    ColorBox* boxSymbolColor,*boxFillColor;
+    ColorButton *boxSymbolColor, *boxFillColor;
     SymbolBox* boxSymbolStyle;
     PatternBox *boxPattern;
 	QTabWidget* privateTabWidget;
-	QWidget *errorsPage, *spectrogramPage;
+	QWidget *errorsPage, *spectrogramPage, *contourLinesPage;
 	QGroupBox* fillGroupBox;
     QCheckBox* plusBox;
     QCheckBox* minusBox;
     QCheckBox* xBox;
-    ColorButton *colorBox, *levelsColorBox;
-	ColorBox* vectColorBox;
+    ColorButton *colorBox, *levelsColorBox, *vectColorBox;
     DoubleSpinBox* widthBox;
     QComboBox* capBox;
     QCheckBox* throughBox;
@@ -368,7 +354,7 @@ private:
 	QCheckBox *filledHeadBox;
 	QSpinBox *boxCoef, *boxWhiskersCoef;
 	QCheckBox *boxFillSymbols, *boxFillSymbol;
-	ColorBox *boxPercFillColor, *boxEdgeColor;
+	ColorButton *boxPercFillColor, *boxEdgeColor;
 	QLabel 	*whiskerCoeffLabel, *whiskerRangeLabel, *boxCoeffLabel;
 	QLabel *boxRangeLabel, *whiskerCntLabel, *boxCntLabel;
 	QGroupBox *GroupBoxVectEnd;
@@ -389,7 +375,7 @@ private:
     QCheckBox *boxLabelsWhiteOut;
     QPushButton *btnLabelsFont;
     QComboBox *boxLabelsAlign, *boxLabelsColumn;
-    ColorBox* boxLabelsColor;
+    ColorButton* boxLabelsColor;
     QWidget *labelsPage;
 
     QGroupBox *pieAutoLabelsBox, *boxPieWedge;
@@ -402,6 +388,22 @@ private:
 	QComboBox *unitBox;
 	QComboBox *backgroundApplyToBox;
 	QPushButton *layerDefaultBtn;
+
+	DoubleSpinBox *firstContourLineBox, *contourLinesDistanceBox;
+	QLabel *justifyLabelsLbl, *labelsColumnLbl;
+
+	QWidget *spectroValuesPage;
+	QComboBox *boxSpectroMatrix;
+	QCheckBox *boxUseMatrixFormula;
+	ContourLinesEditor *contourLinesEditor;
+	QPushButton *btnSetEquidistantLevels;
+	QRadioButton *customPenBtn;
+
+	QSpinBox *boxSkipSymbols, *boxSkipErrorBars;
+	QComboBox *symbolsFormatApplyToBox, *lineFormatApplyToBox, *errorBarsFormatApplyToBox;
+	QSpinBox *boxMaxPoints;
+	DoubleSpinBox *boxDouglasPeukerTolerance;
+	QGroupBox *speedModeBox;
 };
 
 /*****************************************************************************
