@@ -27,10 +27,12 @@
  *                                                                         *
  ***************************************************************************/
 #include "TextEditor.h"
-#include "../LegendWidget.h"
-#include "../QwtPieCurve.h"
+#include <LegendWidget.h>
+#include <QwtPieCurve.h>
+#include <MultiLayer.h>
 
 #include <QTextCursor>
+#include <QMessageBox>
 
 #include <qwt_text.h>
 #include <qwt_text_label.h>
@@ -49,6 +51,7 @@ TextEditor::TextEditor(Graph *g): QTextEdit(g), d_graph(g)
 	palette.setColor(QPalette::Active, QPalette::Base, Qt::white);
 	setPalette(palette);
 
+	bool moveCrs = true;
 	QString text;
 	if (g->activeText()){
 		setParent(g->multiLayer()->canvas());
@@ -56,11 +59,13 @@ TextEditor::TextEditor(Graph *g): QTextEdit(g), d_graph(g)
 		setGeometry(d_target->geometry());
 		text = ((LegendWidget*)d_target)->text();
 		d_target->hide();
+		setFont(((LegendWidget*)d_target)->font());
 	} else if (g->titleSelected()){
 		d_target = g->titleLabel();
 		QwtText t = g->title();
 		text = t.text();
 		setAlignment((Qt::Alignment)t.renderFlags());
+		setFont(t.font());
 		setGeometry(d_target->geometry());
 	} else if (g->selectedScale()){
 		d_target = g->selectedScale();
@@ -68,6 +73,7 @@ TextEditor::TextEditor(Graph *g): QTextEdit(g), d_graph(g)
 		QwtText t = scale->title();
 		text = t.text();
 		setAlignment((Qt::Alignment)t.renderFlags());
+		setFont(t.font());
 
 		QRect rect = g->axisTitleRect(scale);
 		if (scale->alignment() == QwtScaleDraw::BottomScale ||
@@ -84,6 +90,7 @@ TextEditor::TextEditor(Graph *g): QTextEdit(g), d_graph(g)
 			t.setText(" ");
 			t.setBackgroundPen(QPen(Qt::NoPen));
 			scale->setTitle(t);
+			moveCrs = false;
 		}
 	}
 
@@ -91,26 +98,40 @@ TextEditor::TextEditor(Graph *g): QTextEdit(g), d_graph(g)
 	cursor.insertText(text);
 	d_initial_text = text;
 
+	setWordWrapMode (QTextOption::NoWrap);
+	setAlignment(Qt::AlignCenter);
+
+	QTextFrame *frame = document()->rootFrame();
+	QTextFrameFormat format = frame->frameFormat();
+	format.setTopMargin(format.topMargin () + 3);
+	frame->setFrameFormat(format);
 	show();
+
+	if (moveCrs)
+		setTextCursor(cursorForPosition(mapFromGlobal(QCursor::pos())));
 	setFocus();
 }
 
 void TextEditor::closeEvent(QCloseEvent *e)
 {
-	QString s = QString();
+	QString s = text();
+	int lines = s.count(QRegExp("\n"));
+	if ( lines >= 100 && QMessageBox::question(d_graph->multiLayer(),
+		tr("QtiPlot") + " - " + tr("Confirmation"),
+		tr("Are you sure you want to add %1 text lines into this text box?").arg(lines),
+		QMessageBox::Yes | QMessageBox::Cancel, QMessageBox::Cancel) == QMessageBox::Cancel)
+		s = d_initial_text;
+
 	if (d_target->isA("LegendWidget")){
-		s = text();
 		((LegendWidget*)d_target)->setText(s);
         d_target->show();
 		d_graph->setActiveText(NULL);
 	} else if (d_target->isA("PieLabel")){
-		s = text();
 		((PieLabel*)d_target)->setCustomText(s);
         d_target->show();
 		d_graph->setActiveText(NULL);
 	} else if (d_target->isA("QwtTextLabel")){
 		QwtText title = d_graph->title();
-		s = text();
 		if(s.isEmpty())
 			s = " ";
 		title.setText(s);
@@ -118,7 +139,6 @@ void TextEditor::closeEvent(QCloseEvent *e)
 	} else if (d_target->isA("QwtScaleWidget")){
 		QwtScaleWidget *scale = (QwtScaleWidget*)d_target;
 		QwtText title = scale->title();
-		s = text();
 		if(s.isEmpty())
 			s = " ";
 		title.setText(s);
