@@ -37,11 +37,13 @@
 #include <QVector>
 #include <QEvent>
 
-#include "../Table.h"
-#include "../matrix/Matrix.h"
+#include <Table.h>
+#include <Matrix.h>
+#include <FrameWidget.h>
 
 using namespace Qwt3D;
 
+class QTextDocument;
 class UserFunction;
 class UserParametricSurface;
 
@@ -63,15 +65,18 @@ public:
 	Graph3D (const QString& label, ApplicationWindow* parent, const char* name=0, Qt::WFlags f=0);
 	~Graph3D();
 
-	enum PlotType{Scatter = 0, Trajectory = 1, Bars = 2, Ribbon =  3};
+	enum PlotType{NoTable = -1, Scatter = 0, Trajectory = 1, Bars = 2, Ribbon =  3};
 	enum PointStyle{None = 0, Dots = 1, VerticalBars = 2, HairCross = 3, Cones = 4};
+	static Graph3D* restore(ApplicationWindow* app, const QStringList &lst, int fileVersion);
+
+	Qwt3D::SurfacePlot* surface(){return sp;};
 
 public slots:
 	void copy(Graph3D* g);
 	void initPlot();
 	void initCoord();
 	void addFunction(const QString& s, double xl, double xr, double yl,
-						  double yr, double zl, double zr, int columns, int rows);
+					double yr, double zl, double zr, int columns = 40, int rows = 30);
 	void addParametricSurface(const QString& xFormula, const QString& yFormula,
 						const QString& zFormula, double ul, double ur, double vl, double vr,
 						int columns, int rows, bool uPeriodic, bool vPeriodic);
@@ -82,13 +87,15 @@ public slots:
 	void addMatrixData(Matrix* m,double xl,double xr,double yl,double yr,double zl,double zr);
 	void updateMatrixData(Matrix* m);
 
-	void addData(Table* table,const QString& xColName,const QString& yColName);
-	void addData(Table* table,const QString& xColName,const QString& yColName,
+	void addRibbon(Table* table,const QString& xColName,const QString& yColName);
+	void addRibbon(Table* table,const QString& xColName,const QString& yColName,
                 double xl, double xr, double yl, double yr, double zl, double zr);
 	void addData(Table* table, int xCol, int yCol, int zCol, int type = 0);
 	void loadData(Table* table, int xCol, int yCol, int zCol,
                 double xl=0.0, double xr=0.0, double yl=0.0, double yr=0.0, double zl=0.0, double zr=0.0);
 
+    PlotType tablePlotType(){return d_table_plot_type;};
+
 	void clearData();
 	bool hasData(){return sp->hasData();};
 
@@ -110,6 +117,8 @@ public slots:
 
 	//! \name Event Handlers
 	//@{
+	void dropEvent(QDropEvent*);
+	void dragEnterEvent(QDragEnterEvent*);
 	bool eventFilter(QObject *object, QEvent *e);
 	void resizeEvent (QResizeEvent *);
 	void scaleFonts(double factor);
@@ -146,7 +155,6 @@ public slots:
 
 	QFont numbersFont();
 	void setNumbersFont(const QFont& font);
-	void setNumbersFont(const QStringList& lst);
 
 	double xStart();
 	double xStop();
@@ -154,8 +162,10 @@ public slots:
 	double yStop();
 	double zStart();
 	double zStop();
-	QStringList scaleLimits();
-	void updateScale(int axis, const QStringList& options);
+
+	int axisType(int axis){return scaleType[axis];};
+	void setAxisType(int axis, int type);
+
 	void setScales(double xl, double xr, double yl, double yr, double zl, double zr);
 	void updateScales(double xl, double xr, double yl, double yr,
 				  		double zl, double zr, int xcol, int ycol);
@@ -234,10 +244,15 @@ public slots:
 	Qwt3D::COORDSTYLE coordStyle();
 
 	void print();
+	void print(QPrinter *printer);
 	void copyImage();
-	void exportImage(const QString& fileName, int quality = 100, bool transparent = false);
+	void exportImage(QTextDocument *document, int quality, bool transparent,
+						int dpi, const QSizeF& customSize, int unit, double fontsFactor);
+	void exportImage(const QString& fileName, int quality = 100, bool transparent = false, int dpi = 0,
+		const QSizeF& customSize = QSizeF(), int unit = FrameWidget::Pixel, double fontsFactor = 1.0);
     void exportPDF(const QString& fileName);
-    void exportVector(const QString& fileName);
+    void exportVector(const QString& fileName, int textExportMode = 0, int sortMode = 1,
+		const QSizeF& customSize = QSizeF(), int unit = FrameWidget::Pixel, double fontsFactor = 1.0);
     void exportToFile(const QString& fileName);
 
 	void save(const QString& fn, const QString& geometry, bool = false);
@@ -249,14 +264,16 @@ public slots:
 
 	//! \name Colors
 	//@{
-	void setDataColors(const QColor& cMax, const QColor& cMin);
+	void setDataColors(const QColor& cMin, const QColor& cMax){setDataColorMap(QwtLinearColorMap(cMin, cMax));};
+	void setDataColorMap(const QwtLinearColorMap& colorMap);
+	void setDataColorMap(const QString& fileName);
+	void setDataColorMap(const ColorVector& colors);
+	void setDataColorMap(const ColorVector& colors, const QwtLinearColorMap& colorMap);
 
 	void changeTransparency(double t);
 	void setTransparency(double t);
-	double transparency(){return alpha;};
+	double transparency(){return d_alpha;};
 
-	QColor minDataColor();
-	QColor maxDataColor();
 	QColor meshColor(){return meshCol;};
 	QColor axesColor(){return axesCol;};
 	QColor labelColor(){return labelsCol;};
@@ -264,9 +281,11 @@ public slots:
 	QColor bgColor(){return bgCol;};
 	QColor gridColor(){return gridCol;};
 
-	QString colorMap(){return color_map;};
-	void setDataColorMap(const QString& fileName);
-	bool openColorMap(ColorVector& cv, QString fname);
+	QString colorMapFile(){return d_color_map_file;};
+	QwtLinearColorMap colorMap(){return d_color_map;};
+	QwtLinearColorMap *colorMapPointer(){return &d_color_map;};
+
+	static bool openColorMapFile(ColorVector& cv, QString fname);
 
 	void setMeshColor(const QColor&);
 	void setAxesColor(const QColor&);
@@ -274,8 +293,6 @@ public slots:
 	void setLabelsColor(const QColor&);
 	void setBackgroundColor(const QColor&);
 	void setGridColor(const QColor&);
-
-	void setColors(const QStringList& colors);
 	//@}
 
 	//! \name Title
@@ -308,6 +325,10 @@ public slots:
 	//@{
 	double barsRadius();
 	void setBarRadius(double rad);
+	bool barLines(){return d_bar_lines;};
+	void setBarLines(bool lines = true);
+	bool filledBars(){return d_filled_bars;};
+	void setFilledBars(bool filled = true);
 	//@}
 
 	//! \name Scatter Plots
@@ -348,15 +369,20 @@ public slots:
 	//! Enables/Disables autoscaling using findBestLayout().
 	void setAutoscale(bool on = true){d_autoscale = on;};
 
+	Qwt3D::CoordinateSystem* coordinateSystem() {return sp->coordinates();};
+	void setScale(int axis, double start, double end, int majorTicks, int minorTicks, Qwt3D::SCALETYPE type);
+
 signals:
 	void showOptionsDialog();
 	void modified();
 
 private:
+	void resetAxesType();
 	//! Wait this many msecs before redraw 3D plot (used for animations)
   	int animation_redraw_wait;
 	//! File name of the color map used for the data (if any)
-  	QString color_map;
+  	QString d_color_map_file;
+	QwtLinearColorMap d_color_map;
 
 	QTimer *d_timer;
 	QString title, plotAssociation;
@@ -364,13 +390,20 @@ private:
 	QFont titleFnt;
 	bool legendOn, d_autoscale;
 	QVector<int> scaleType;
-	QColor axesCol,labelsCol,titleCol,meshCol,bgCol,numCol,gridCol;
-	//! Custom data colors.
-	QColor fromColor, toColor;
+	QColor axesCol, labelsCol, titleCol, meshCol, bgCol, numCol, gridCol;
 	int labelsDist, legendMajorTicks;
 	bool ignoreFonts;
 	Qwt3D::StandardColor* col_;
-	double barsRad, alpha, d_point_size, crossHairRad, crossHairLineWidth, conesRad;
+	//! Transparency
+	double d_alpha;
+
+	//! \name Bar options
+	//@{
+	double d_bars_rad;
+	bool d_filled_bars, d_bar_lines;
+	//@}
+
+	double d_point_size, crossHairRad, crossHairLineWidth, conesRad;
 	//! Draw 3D points with smoothed angles.
 	bool d_smooth_points;
 	bool crossHairSmooth, crossHairBoxed;
@@ -382,6 +415,7 @@ private:
 	UserFunction *d_func;
 	UserParametricSurface *d_surface;
 	Qwt3D::PLOTSTYLE style_;
+	PlotType d_table_plot_type;
 };
 
 //! Class for user defined surfaces
