@@ -27,14 +27,16 @@
  *                                                                         *
  ***************************************************************************/
 #include "Plot3DDialog.h"
-#include "../MyParser.h"
-#include "../SymbolDialog.h"
-#include "../ApplicationWindow.h"
-#include "../ColorButton.h"
-#include "../TextFormatButtons.h"
+#include "ColorMapPreviewDialog.h"
+#include <MyParser.h>
+#include <SymbolDialog.h>
+#include <ApplicationWindow.h>
+#include <TextFormatButtons.h>
+#include <DoubleSpinBox.h>
+#include <ColorMapEditor.h>
+#include <ColorButton.h>
 
 #include <QListWidget>
-#include <QLineEdit>
 #include <QTextEdit>
 #include <QLayout>
 #include <QSpinBox>
@@ -53,11 +55,42 @@
 
 #include <qwt3d_color.h>
 
+static const char* choose_folder_xpm[]={
+    "16 16 11 1",
+    "# c #000000",
+    "g c #c0c0c0",
+    "e c #303030",
+    "a c #ffa858",
+    "b c #808080",
+    "d c #a0a0a4",
+    "f c #585858",
+    "c c #ffdca8",
+    "h c #dcdcdc",
+    "i c #ffffff",
+    ". c None",
+    "....###.........",
+    "....#ab##.......",
+    "....#acab####...",
+    "###.#acccccca#..",
+    "#ddefaaaccccca#.",
+    "#bdddbaaaacccab#",
+    ".eddddbbaaaacab#",
+    ".#bddggdbbaaaab#",
+    "..edgdggggbbaab#",
+    "..#bgggghghdaab#",
+    "...ebhggghicfab#",
+    "....#edhhiiidab#",
+    "......#egiiicfb#",
+    "........#egiibb#",
+    "..........#egib#",
+    "............#ee#"};
+
 Plot3DDialog::Plot3DDialog( QWidget* parent,  Qt::WFlags fl )
     : QDialog( parent, fl )
 {
     setName( "Plot3DDialog" );
 	setWindowTitle( tr( "QtiPlot - Surface Plot Options" ) );
+	setSizeGripEnabled(true);
 
 	bars=0; points=0;
 
@@ -77,6 +110,7 @@ Plot3DDialog::Plot3DDialog( QWidget* parent,  Qt::WFlags fl )
 
     initScalesPage();
 	initAxesPage();
+	initGridPage();
 	initTitlePage();
 	initColorsPage();
 	initGeneralPage();
@@ -100,15 +134,21 @@ void Plot3DDialog::initScalesPage()
 	axesList->setFixedWidth(50);
 	axesList->setCurrentRow(0);
 
+	ApplicationWindow *app = (ApplicationWindow *)parent();
+
     QGridLayout *gl1 = new QGridLayout();
     gl1->addWidget(new QLabel(tr("From")), 0, 0);
-	boxFrom = new QLineEdit();
-	boxFrom->setMaximumWidth(150);
+	boxFrom = new DoubleSpinBox();
+	boxFrom->setLocale(app->locale());
+    boxFrom->setDecimals(app->d_decimal_digits);
     gl1->addWidget(boxFrom, 0, 1);
+
     gl1->addWidget(new QLabel(tr("To")), 1, 0);
-	boxTo = new QLineEdit();
-	boxTo->setMaximumWidth(150);
+	boxTo = new DoubleSpinBox();
+	boxTo->setLocale(app->locale());
+    boxTo->setDecimals(app->d_decimal_digits);
     gl1->addWidget(boxTo, 1, 1);
+
     gl1->addWidget(new QLabel(tr("Type")), 2, 0);
 	boxType=new QComboBox();
 	boxType->addItem(tr("linear"));
@@ -134,8 +174,8 @@ void Plot3DDialog::initScalesPage()
 
 	QHBoxLayout* hb = new QHBoxLayout();
 	hb->addWidget(axesList);
-	hb->addWidget(gb1);
-    hb->addWidget(TicksGroupBox);
+	hb->addWidget(gb1, 1);
+    hb->addWidget(TicksGroupBox, 1);
 
     scale = new QWidget();
     scale->setLayout(hb);
@@ -161,20 +201,28 @@ void Plot3DDialog::initAxesPage()
     QHBoxLayout* hb1 = new QHBoxLayout();
 	btnLabelFont = new QPushButton(tr( "&Choose font" ));
     hb1->addWidget(btnLabelFont);
-	
+
 	axisTitleFormatButtons = new TextFormatButtons(boxLabel);
-	axisTitleFormatButtons->toggleCurveButton(false);
-	axisTitleFormatButtons->toggleFontButtons(false);
 	hb1->addWidget(axisTitleFormatButtons);
 
     hb1->addStretch();
     gl1->addLayout(hb1, 1, 1);
 
+	ApplicationWindow *app = (ApplicationWindow *)parent();
+
     gl1->addWidget(new QLabel(tr("Major Ticks Length")), 2, 0);
-	boxMajorLength = new QLineEdit();
+	boxMajorLength = new DoubleSpinBox();
+	boxMajorLength->setLocale(app->locale());
+    boxMajorLength->setDecimals(app->d_decimal_digits);
+	boxMajorLength->setMinimum(0.0);
     gl1->addWidget(boxMajorLength, 2, 1);
+
     gl1->addWidget(new QLabel(tr("Minor Ticks Length")), 3, 0);
-	boxMinorLength = new QLineEdit();
+	boxMinorLength = new DoubleSpinBox();
+	boxMinorLength->setLocale(app->locale());
+    boxMinorLength->setDecimals(app->d_decimal_digits);
+	boxMinorLength->setMinimum(0.0);
+
     gl1->addWidget(boxMinorLength, 3, 1);
     gl1->setRowStretch(4, 1);
 
@@ -189,8 +237,6 @@ void Plot3DDialog::initAxesPage()
     axes->setLayout(hb2);
 	generalDialog->insertTab(axes, tr( "&Axis" ) );
 
-	connect( axesList2, SIGNAL(currentRowChanged(int)), this, SLOT(viewAxisOptions(int)));
-	connect( axesList, SIGNAL(currentRowChanged(int)), this, SLOT(viewScaleLimits(int)));
 	connect( btnLabelFont, SIGNAL(clicked()), this, SLOT(pickAxisLabelFont()));
 }
 
@@ -206,15 +252,13 @@ void Plot3DDialog::initTitlePage()
 
     btnTitleFont = new QPushButton(tr( "&Font" ));
     hb1->addWidget(btnTitleFont);
-	
+
 	QVBoxLayout* vl = new QVBoxLayout();
 	boxTitle = new QTextEdit();
 	boxTitle->setMaximumHeight(80);
 	vl->addWidget(boxTitle);
-	
+
 	titleFormatButtons = new TextFormatButtons(boxTitle);
-	titleFormatButtons->toggleCurveButton(false);
-	titleFormatButtons->toggleFontButtons(false);
 	hb1->addWidget(titleFormatButtons);
     hb1->addStretch();
 
@@ -230,39 +274,55 @@ void Plot3DDialog::initTitlePage()
 
 void Plot3DDialog::initColorsPage()
 {
-    QGridLayout* vl1 = new QGridLayout();
-    btnFromColor = new ColorButton();
-    QLabel *maxLabel = new QLabel(tr( "&Max" ));
-    maxLabel->setBuddy(btnFromColor);
-    vl1->addWidget(maxLabel, 0, 0);
-    vl1->addWidget(btnFromColor, 0, 1);
+	linearColorMapGroupBox = new QGroupBox(tr( "Linea&r color map" ));
+	linearColorMapGroupBox->setCheckable(true);
+	connect(linearColorMapGroupBox, SIGNAL(clicked(bool)),
+			this, SLOT(updateColorMapFileGroupBox(bool)));
 
-	btnToColor = new ColorButton();
-	QLabel *minLabel = new QLabel(tr( "M&in" ));
-	minLabel->setBuddy(btnToColor);
-    vl1->addWidget(minLabel, 1, 0);
-    vl1->addWidget(btnToColor, 1, 1);
+	QHBoxLayout* hb = new QHBoxLayout(linearColorMapGroupBox);
 
-	btnColorMap = new QPushButton(tr( "Color Ma&p" ));
-    vl1->addWidget(btnColorMap, 2, 1);
-    vl1->setRowStretch(3, 1);
+	ApplicationWindow *app = (ApplicationWindow *)parent();
+	d_color_map_editor = new ColorMapEditor(app->locale());
+	hb->addWidget(d_color_map_editor);
 
-    QGroupBox *gb1 = new QGroupBox(tr( "Data" ));
-    gb1->setLayout(vl1);
+    colorMapFileGroupBox = new QGroupBox(tr( "Color map &file" ));
+	colorMapFileGroupBox->setCheckable(true);
+	connect(colorMapFileGroupBox, SIGNAL(clicked(bool)), this,
+			SLOT(updateLinearColorMapGroupBox(bool)));
+
+	QGridLayout* layout = new QGridLayout(colorMapFileGroupBox);
+
+	btnColorMap = new QPushButton();
+	btnColorMap->setIcon(QIcon(QPixmap(choose_folder_xpm)));
+	layout->addWidget(btnColorMap, 0, 0);
+
+	colorMapPreviewLabel = new QLabel(tr("None"));
+	colorMapPreviewLabel->setScaledContents(true);
+	colorMapPreviewLabel->setFrameShape( QFrame::StyledPanel );
+	colorMapPreviewLabel->setFrameShadow( QFrame::Sunken );
+	layout->addWidget(colorMapPreviewLabel, 0, 1);
+	layout->setRowStretch(1, 1);
+	layout->setColumnStretch(1, 1);
 
     QGridLayout* vl2 = new QGridLayout();
+    vl2->addWidget(new QLabel( tr( "Opacity" )), 0, 0);
+	boxTransparency = new QSpinBox();
+    boxTransparency->setRange(0, 100);
+    boxTransparency->setSingleStep(5);
+    vl2->addWidget(boxTransparency, 0, 1);
+
     btnMesh = new ColorButton();
     QLabel *meshLabel = new QLabel(tr( "&Line" ));
     meshLabel->setBuddy(btnMesh);
-    vl2->addWidget(meshLabel, 0, 0);
-    vl2->addWidget(btnMesh, 0, 1);
+    vl2->addWidget(meshLabel, 1, 0);
+    vl2->addWidget(btnMesh, 1, 1);
 
     btnBackground = new ColorButton();
     QLabel *backgroundLabel = new QLabel(tr( "&Background" ));
     backgroundLabel->setBuddy(btnBackground);
-    vl2->addWidget(backgroundLabel, 1, 0);
-    vl2->addWidget(btnBackground, 1, 1);
-    vl2->setRowStretch(2, 1);
+    vl2->addWidget(backgroundLabel, 2, 0);
+    vl2->addWidget(btnBackground, 2, 1);
+    vl2->setRowStretch(3, 1);
 
     QGroupBox *gb2 = new QGroupBox(tr( "General" ));
     gb2->setLayout(vl2);
@@ -285,36 +345,22 @@ void Plot3DDialog::initColorsPage()
     numbersLabel->setBuddy(btnNumbers);
 	gl1->addWidget(numbersLabel, 2, 0);
     gl1->addWidget(btnNumbers, 2, 1);
-
-	btnGrid = new ColorButton();
-	QLabel *gridLabel = new QLabel(tr( "&Grid" ));
-	gridLabel->setBuddy(btnGrid);
-	gl1->addWidget(gridLabel, 3, 0);
-    gl1->addWidget(btnGrid, 3, 1);
-    gl1->setRowStretch(4, 1);
+    gl1->setRowStretch(3, 1);
 
     AxesColorGroupBox = new QGroupBox(tr( "Coordinate System" ));
     AxesColorGroupBox->setLayout(gl1);
 
     QHBoxLayout* hb1 = new QHBoxLayout();
-	hb1->addWidget(gb1);
+	hb1->addWidget(linearColorMapGroupBox);
     hb1->addWidget(gb2);
     hb1->addWidget(AxesColorGroupBox);
 
-    QHBoxLayout* hb2 = new QHBoxLayout();
-    hb2->addStretch();
-	hb2->addWidget(new QLabel( tr( "Opacity" )));
-	boxTransparency = new QSpinBox();
-    boxTransparency->setRange(0, 100);
-    boxTransparency->setSingleStep(5);
-    hb2->addWidget(boxTransparency);
-
-	QVBoxLayout* vl = new QVBoxLayout();
-	vl->addLayout(hb2);
-	vl->addLayout(hb1);
+    QVBoxLayout *vl0 = new QVBoxLayout();
+	vl0->addLayout(hb1);
+	vl0->addWidget(colorMapFileGroupBox);
 
     colors = new QWidget();
-    colors->setLayout(vl);
+    colors->setLayout(vl0);
 	generalDialog->insertTab(colors, tr( "&Colors" ) );
 
 	connect( btnColorMap, SIGNAL( clicked() ), this, SLOT(pickDataColorMap() ) );
@@ -343,12 +389,12 @@ void Plot3DDialog::initGeneralPage()
     gl1->addWidget(boxResolution, 2, 1);
 
     gl1->addWidget(new QLabel( tr( "Numbers Font" )), 3, 0);
-	btnNumbersFont=new QPushButton(tr( "&Choose Font" ));
+	btnNumbersFont = new QPushButton(tr( "&Choose Font" ));
     gl1->addWidget(btnNumbersFont, 3, 1);
 
     gl1->addWidget(new QLabel( tr( "Distance labels - axis" )), 4, 0);
 	boxDistance = new QSpinBox();
-    boxDistance->setRange(0, 1000);
+    boxDistance->setRange(-1000, 1000);
     boxDistance->setSingleStep(5);
     gl1->addWidget(boxDistance, 4, 1);
     gl1->setRowStretch(5, 1);
@@ -400,6 +446,90 @@ void Plot3DDialog::initGeneralPage()
 	connect( btnNumbersFont, SIGNAL(clicked()), this, SLOT(pickNumbersFont()));
 }
 
+void Plot3DDialog::enableMajorGrids(bool on)
+{
+	btnGrid->setEnabled(on);
+	boxMajorGridStyle->setEnabled(on);
+	boxMajorGridWidth->setEnabled(on);
+}
+
+void Plot3DDialog::enableMinorGrids(bool on)
+{
+	btnGridMinor->setEnabled(on);
+	boxMinorGridStyle->setEnabled(on);
+	boxMinorGridWidth->setEnabled(on);
+}
+
+void Plot3DDialog::initGridPage()
+{
+	QGroupBox *gb1 = new QGroupBox();
+	QGridLayout *gl1 = new QGridLayout(gb1);
+
+	boxMajorGrids = new QCheckBox(tr("Ma&jor Grids"));
+	gl1->addWidget(boxMajorGrids, 0, 1);
+
+	boxMinorGrids = new QCheckBox(tr("Mi&nor Grids"));
+	gl1->addWidget(boxMinorGrids, 0, 2);
+
+    gl1->addWidget(new QLabel(tr("Color")), 1, 0);
+
+	btnGrid = new ColorButton();
+    gl1->addWidget(btnGrid, 1, 1);
+
+    btnGridMinor = new ColorButton();
+    gl1->addWidget(btnGridMinor, 1, 2);
+
+    gl1->addWidget(new QLabel(tr("Style")), 2, 0);
+
+	boxMajorGridStyle = new QComboBox();
+	boxMajorGridStyle->addItem(tr("Solid"));
+	boxMajorGridStyle->addItem(tr("Dash"));
+	boxMajorGridStyle->addItem(tr("Dot"));
+	boxMajorGridStyle->addItem(tr("Dash Dot"));
+	boxMajorGridStyle->addItem(tr("Dash Dot Dot"));
+	boxMajorGridStyle->addItem(tr("Short Dash"));
+	boxMajorGridStyle->addItem(tr("Short Dot"));
+	boxMajorGridStyle->addItem(tr("Short Dash Dot"));
+
+	gl1->addWidget(boxMajorGridStyle, 2, 1);
+
+	boxMinorGridStyle = new QComboBox();
+	boxMinorGridStyle->addItem(tr("Solid"));
+	boxMinorGridStyle->addItem(tr("Dash"));
+	boxMinorGridStyle->addItem(tr("Dot"));
+	boxMinorGridStyle->addItem(tr("Dash Dot"));
+	boxMinorGridStyle->addItem(tr("Dash Dot Dot"));
+	boxMinorGridStyle->addItem(tr("Short Dash"));
+	boxMinorGridStyle->addItem(tr("Short Dot"));
+	boxMinorGridStyle->addItem(tr("Short Dash Dot"));
+
+	gl1->addWidget(boxMinorGridStyle, 2, 2);
+
+	gl1->addWidget(new QLabel(tr("Width")), 3, 0);
+
+	ApplicationWindow *app = (ApplicationWindow *)parent();
+
+	boxMajorGridWidth = new DoubleSpinBox();
+	boxMajorGridWidth->setLocale(app->locale());
+	boxMajorGridWidth->setMinimum(0.0);
+	gl1->addWidget(boxMajorGridWidth, 3, 1);
+
+	boxMinorGridWidth = new DoubleSpinBox();
+	boxMinorGridWidth->setLocale(app->locale());
+	boxMinorGridWidth->setMinimum(0.0);
+	gl1->addWidget(boxMinorGridWidth, 3, 2);
+
+	gl1->setRowStretch(4, 1);
+	gl1->setColumnStretch(3, 1);
+
+    QHBoxLayout* hl = new QHBoxLayout();
+	hl->addWidget(gb1);
+
+    gridPage = new QWidget();
+    gridPage->setLayout(hl);
+	generalDialog->insertTab(gridPage, tr("G&rid"));
+}
+
 void Plot3DDialog::initPointsOptionsStack()
 {
     QHBoxLayout* hl1 = new QHBoxLayout();
@@ -415,15 +545,22 @@ void Plot3DDialog::initPointsOptionsStack()
 	optionStack->setFrameShape( QFrame::StyledPanel );
 	optionStack->setFrameShadow( QStackedWidget::Plain );
 
+	ApplicationWindow *app = (ApplicationWindow *)parent();
+
     QGridLayout *gl1 = new QGridLayout();
     gl1->addWidget(new QLabel( tr( "Width" )), 0, 0);
-	boxSize = new QLineEdit("5");
+	boxSize = new DoubleSpinBox();
+	boxSize->setLocale(app->locale());
+    boxSize->setDecimals(app->d_decimal_digits);
+    boxSize->setMinimum(0.0);
+	boxSize->setValue(5);
     gl1->addWidget(boxSize, 0, 1);
 
 	gl1->addWidget(new QLabel( tr( "Smooth angles" )), 1, 0);
 	boxSmooth = new QCheckBox();
 	boxSmooth->setChecked(false);
     gl1->addWidget(boxSmooth, 1, 1);
+    gl1->setRowStretch(2, 1);
 
     dotsPage = new QWidget();
     dotsPage->setLayout(gl1);
@@ -431,11 +568,21 @@ void Plot3DDialog::initPointsOptionsStack()
 
     QGridLayout *gl2 = new QGridLayout();
     gl2->addWidget(new QLabel( tr( "Radius" )), 0, 0);
-	boxCrossRad = new QLineEdit("0.01");
+	boxCrossRad = new DoubleSpinBox();
+	boxCrossRad->setLocale(app->locale());
+    boxCrossRad->setDecimals(app->d_decimal_digits);
+	boxCrossRad->setValue(0.01);
+	boxCrossRad->setMinimum(0.0);
     gl2->addWidget(boxCrossRad, 0, 1);
+
 	gl2->addWidget(new QLabel(tr( "Line Width")), 1, 0);
-	boxCrossLinewidth = new QLineEdit("1");
+	boxCrossLinewidth = new DoubleSpinBox();
+	boxCrossLinewidth->setLocale(app->locale());
+    boxCrossLinewidth->setDecimals(app->d_decimal_digits);
+	boxCrossLinewidth->setValue(1);
+	boxCrossLinewidth->setMinimum(0.0);
     gl2->addWidget(boxCrossLinewidth, 1, 1);
+
 	gl2->addWidget(new QLabel(tr( "Smooth line" )), 2, 0);
 	boxCrossSmooth = new QCheckBox();
     boxCrossSmooth->setChecked(true);
@@ -444,6 +591,7 @@ void Plot3DDialog::initPointsOptionsStack()
 	boxBoxed = new QCheckBox();
 	boxBoxed->setChecked(false);
     gl2->addWidget(boxBoxed, 3, 1);
+    gl2->setRowStretch(4, 1);
 
 	crossPage = new QWidget();
     crossPage->setLayout(gl2);
@@ -451,13 +599,19 @@ void Plot3DDialog::initPointsOptionsStack()
 
     QGridLayout *gl3 = new QGridLayout();
     gl3->addWidget(new QLabel( tr( "Width" )), 0, 0);
-	boxConesRad = new QLineEdit("0.5");
+	boxConesRad = new DoubleSpinBox();
+	boxConesRad->setLocale(app->locale());
+    boxConesRad->setDecimals(app->d_decimal_digits);
+	boxConesRad->setValue(0.5);
+	boxConesRad->setMinimum(0.0);
     gl3->addWidget(boxConesRad, 0, 1);
+
     gl3->addWidget(new QLabel( tr( "Quality" )), 1, 0);
 	boxQuality = new QSpinBox();
     boxQuality->setRange(0, 40);
 	boxQuality->setValue(32);
     gl3->addWidget(boxQuality, 1, 1);
+    gl3->setRowStretch(2, 1);
 
     conesPage = new QWidget();
     conesPage->setLayout(gl3);
@@ -481,15 +635,35 @@ void Plot3DDialog::setPlot(Graph3D *g)
 
 	d_plot = g;
 
-	btnFromColor->setColor(g->minDataColor());
-	btnToColor->setColor(g->maxDataColor());
+	d_color_map_editor->setRange(g->zStart(), g->zStop());
+	d_color_map_editor->setColorMap(g->colorMap());
 	btnTitleColor->setColor(g->titleColor());
 	btnMesh->setColor(g->meshColor());
 	btnAxes->setColor(g->axesColor());
 	btnNumbers->setColor(g->numColor());
 	btnLabels->setColor(g->labelColor());
 	btnBackground->setColor(g->bgColor());
-	btnGrid->setColor(g->gridColor());
+
+	//Set major grid properties
+	Qwt3D::GridLine majorGridLine = g->surface()->coordinates()->majorGridLine(Qwt3D::X1);
+	boxMajorGrids->setChecked(majorGridLine.visible_);
+	RGBA color = majorGridLine.color_;
+	btnGrid->setColor(GL2Qt(color.r, color.g, color.b));
+	boxMajorGridWidth->setValue(majorGridLine.width_);
+	boxMajorGridStyle->setCurrentIndex(majorGridLine.style_);
+
+	//Set minor grid properties
+	Qwt3D::GridLine minorGridLine = g->surface()->coordinates()->minorGridLine(Qwt3D::X1);
+	boxMinorGrids->setChecked(minorGridLine.visible_);
+	color = minorGridLine.color_;
+	btnGridMinor->setColor(GL2Qt(color.r, color.g, color.b));
+	boxMinorGridWidth->setValue(minorGridLine.width_);
+	boxMinorGridStyle->setCurrentIndex(minorGridLine.style_);
+
+	d_color_map_file = g->colorMapFile();
+	setColorMapPreview(d_color_map_file);
+	linearColorMapGroupBox->setChecked(d_color_map_file.isEmpty());
+	colorMapFileGroupBox->setChecked(!d_color_map_file.isEmpty());
 
 	boxMeshLineWidth->setValue(g->meshLineWidth());
 	boxTransparency->setValue(int(100*g->transparency()));
@@ -515,19 +689,14 @@ void Plot3DDialog::setPlot(Graph3D *g)
 	boxLabel->setText(labels[0]);
 
 	tickLengths = g->axisTickLengths();
-	boxMajorLength->setText(tickLengths[0]);
-	boxMinorLength->setText(tickLengths[1]);
+	boxMajorLength->setValue(tickLengths[0].toDouble());
+	boxMinorLength->setValue(tickLengths[1].toDouble());
 
 	xAxisFont = g->xAxisLabelFont();
 	yAxisFont = g->yAxisLabelFont();
 	zAxisFont = g->zAxisLabelFont();
 
-	scales = g->scaleLimits();
-	boxFrom->setText(scales[0]);
-	boxTo->setText(scales[1]);
-	boxMajors->setValue(scales[2].toInt());
-	boxMinors->setValue(scales[3].toInt());
-	boxType->setCurrentIndex(scales[4].toInt());
+	viewScaleLimits(0);
 
 	boxDistance->setValue(g->labelsDistance());
 	numbersFont = g->numbersFont();
@@ -556,7 +725,7 @@ void Plot3DDialog::setPlot(Graph3D *g)
 					break;
 
 				case Graph3D::VerticalBars :
-					showBarsTab(g->barsRadius());
+					showBarsTab();
 					break;
 
 				case Graph3D::HairCross :
@@ -579,7 +748,7 @@ void Plot3DDialog::setPlot(Graph3D *g)
 			boxLegend->setDisabled(true);
 
 		if (g->grids() == 0)
-			btnGrid->setDisabled(true);
+			gridPage->setDisabled(true);
 
 		if (g->userFunction() || g->parametricSurface())
 			btnTable->hide();
@@ -588,12 +757,52 @@ void Plot3DDialog::setPlot(Graph3D *g)
 		else if (g->matrix())
 			btnTable->setText(tr("&Matrix"));
 
-	connect( boxMeshLineWidth, SIGNAL(valueChanged(double)), d_plot, SLOT(setMeshLineWidth(double)));
-	connect( boxOrthogonal, SIGNAL(toggled(bool)), d_plot, SLOT(setOrthogonal(bool)));
-	connect( boxLegend, SIGNAL(toggled(bool)), d_plot, SLOT(showColorLegend(bool)));
-    connect( boxResolution, SIGNAL(valueChanged(int)), d_plot, SLOT(setResolution(int)));
-	connect( boxDistance, SIGNAL(valueChanged(int)), d_plot, SLOT(setLabelsDistance(int)));
-};
+	initConnections();
+}
+
+void Plot3DDialog::initConnections()
+{
+	// scales page connections
+	connect(boxFrom, SIGNAL(valueChanged(double)), this, SLOT(updatePlot()));
+    connect(boxTo, SIGNAL(valueChanged(double)), this, SLOT(updatePlot()));
+	connect(boxMajors, SIGNAL(valueChanged(int)), this, SLOT(updatePlot()));
+	connect(boxMinors, SIGNAL(valueChanged(int)), this, SLOT(updatePlot()));
+	connect(axesList, SIGNAL(currentRowChanged(int)), this, SLOT(viewScaleLimits(int)));
+
+	// axes page connections
+	connect(boxMajorLength, SIGNAL(valueChanged(double)), this, SLOT(updatePlot()));
+	connect(boxMinorLength, SIGNAL(valueChanged(double)), this, SLOT(updatePlot()));
+	connect(axesList2, SIGNAL(currentRowChanged(int)), this, SLOT(viewAxisOptions(int)));
+
+	//grid page connections
+	connect(boxMajorGrids, SIGNAL(toggled(bool)), this, SLOT(enableMajorGrids(bool)));
+	connect(boxMajorGrids, SIGNAL(toggled(bool)), this, SLOT(updatePlot()));
+	connect(boxMinorGrids, SIGNAL(toggled(bool)), this, SLOT(enableMinorGrids(bool)));
+	connect(boxMinorGrids, SIGNAL(toggled(bool)), this, SLOT(updatePlot()));
+	connect(boxMajorGridStyle, SIGNAL(activated(int)), this, SLOT(updatePlot()));
+	connect(boxMinorGridStyle, SIGNAL(activated(int)), this, SLOT(updatePlot()));
+	connect(boxMajorGridWidth, SIGNAL(valueChanged(double)), this, SLOT(updatePlot()));
+	connect(boxMinorGridWidth, SIGNAL(valueChanged(double)), this, SLOT(updatePlot()));
+	connect(btnGrid, SIGNAL(colorChanged()), this, SLOT(updatePlot()));
+	connect(btnGridMinor, SIGNAL(colorChanged()), this, SLOT(updatePlot()));
+
+	//color page connections
+	connect(btnAxes, SIGNAL(colorChanged()), this, SLOT(updatePlot()));
+	connect(btnLabels, SIGNAL(colorChanged()), this, SLOT(updatePlot()));
+	connect(btnNumbers, SIGNAL(colorChanged()), this, SLOT(updatePlot()));
+	connect(btnMesh, SIGNAL(colorChanged()), this, SLOT(updatePlot()));
+	connect(btnBackground, SIGNAL(colorChanged()), this, SLOT(updatePlot()));
+	connect(d_color_map_editor, SIGNAL(scalingChanged()), this, SLOT(updatePlot()));
+
+	connect(boxMeshLineWidth, SIGNAL(valueChanged(double)), d_plot, SLOT(setMeshLineWidth(double)));
+	connect(boxOrthogonal, SIGNAL(toggled(bool)), d_plot, SLOT(setOrthogonal(bool)));
+	connect(boxLegend, SIGNAL(toggled(bool)), d_plot, SLOT(showColorLegend(bool)));
+    connect(boxResolution, SIGNAL(valueChanged(int)), d_plot, SLOT(setResolution(int)));
+	connect(boxDistance, SIGNAL(valueChanged(int)), d_plot, SLOT(setLabelsDistance(int)));
+
+	//title page connections
+	connect(btnTitleColor, SIGNAL(colorChanged()), this, SLOT(updatePlot()));
+}
 
 void Plot3DDialog::worksheet()
 {
@@ -601,16 +810,28 @@ void Plot3DDialog::worksheet()
 	close();
 }
 
-void Plot3DDialog::showBarsTab(double rad)
+void Plot3DDialog::showBarsTab()
 {
 	bars = new QWidget( generalDialog );
 
-	QHBoxLayout* hb = new QHBoxLayout(bars);
-	hb->addWidget(new QLabel( tr( "Width" )));
+	QGridLayout* hb = new QGridLayout(bars);
+	hb->addWidget(new QLabel( tr( "Width" )), 0, 0);
 
-	boxBarsRad = new QLineEdit();
-	boxBarsRad->setText(QString::number(rad));
-	hb->addWidget(boxBarsRad);
+	ApplicationWindow *app = (ApplicationWindow *)parent();
+	boxBarsRad = new DoubleSpinBox();
+	boxBarsRad->setLocale(app->locale());
+    boxBarsRad->setDecimals(app->d_decimal_digits);
+	boxBarsRad->setValue(d_plot->barsRadius());
+	hb->addWidget(boxBarsRad, 0, 1);
+
+	boxBarLines = new QCheckBox(tr("Draw lines"));
+	boxBarLines->setChecked(d_plot->barLines());
+	hb->addWidget(boxBarLines, 1, 0);
+
+	boxFilledBars = new QCheckBox(tr("Filled bars"));
+	boxFilledBars->setChecked(d_plot->filledBars());
+	hb->addWidget(boxFilledBars, 2, 0);
+    hb->setRowStretch(3, 1);
 
 	generalDialog->insertTab(bars, tr( "Bars" ), 4);
 }
@@ -618,7 +839,7 @@ void Plot3DDialog::showBarsTab(double rad)
 void Plot3DDialog::showPointsTab(double rad, bool smooth)
 {
 	boxPointStyle->setCurrentIndex(0);
-	boxSize->setText(QString::number(rad));
+	boxSize->setValue(rad);
 	boxSmooth->setChecked(smooth);
 	optionStack->setCurrentIndex(0);
 }
@@ -626,7 +847,7 @@ void Plot3DDialog::showPointsTab(double rad, bool smooth)
 void Plot3DDialog::showConesTab(double rad, int quality)
 {
 	boxPointStyle->setCurrentIndex(2);
-	boxConesRad->setText(QString::number(rad));
+	boxConesRad->setValue(rad);
 	boxQuality->setValue(quality);
 	optionStack->setCurrentIndex (2);
 }
@@ -634,8 +855,8 @@ void Plot3DDialog::showConesTab(double rad, int quality)
 void Plot3DDialog::showCrossHairTab(double rad, double linewidth, bool smooth, bool boxed)
 {
 	boxPointStyle->setCurrentIndex(1);
-	boxCrossRad->setText(QString::number(rad));
-	boxCrossLinewidth->setText(QString::number(linewidth));
+	boxCrossRad->setValue(rad);
+	boxCrossLinewidth->setValue(linewidth);
 	boxCrossSmooth->setChecked(smooth);
 	boxBoxed->setChecked(boxed);
 	optionStack->setCurrentIndex(1);
@@ -649,20 +870,56 @@ void Plot3DDialog::disableMeshOptions()
 
 void Plot3DDialog::pickDataColorMap()
 {
-QString fn = QFileDialog::getOpenFileName(d_plot->colorMap(), tr("Colormap files") + " (*.map *.MAP)", this);
-if (!fn.isEmpty())
-   d_plot->setDataColorMap(fn);
+	ColorMapPreviewDialog *pd = new ColorMapPreviewDialog(this);
+	pd->selectFile(d_color_map_file);
+	pd->updatePreview(d_color_map_file);
+	if (pd->exec() != QDialog::Accepted)
+		return;
+
+	QString fn = pd->selectedFiles()[0];
+	if (!fn.isEmpty()){
+   		d_plot->setDataColorMap(fn);
+		d_color_map_file = fn;
+		setColorMapPreview(d_color_map_file);
+	}
+}
+
+void Plot3DDialog::setColorMapPreview(const QString& fileName)
+{
+	if (fileName.isEmpty()){
+		colorMapPreviewLabel->setText(tr("None"));
+   		return;
+	}
+
+	ColorVector cv;
+	if (!Graph3D::openColorMapFile(cv, fileName)){
+		colorMapPreviewLabel->setText(tr("None"));
+   		return;
+	}
+
+	int height = 20;
+	QPixmap pix;
+	pix.resize(cv.size(), height);
+	QPainter p(&pix);
+	for (unsigned i = 0; i != cv.size(); ++i){
+		RGBA rgb = cv[i];
+		p.setPen(GL2Qt(rgb.r, rgb.g, rgb.b));
+		p.drawLine(QPoint(0, 0), QPoint(0, height));
+   		p.translate(1, 0);
+	}
+  	p.end();
+	colorMapPreviewLabel->setPixmap(pix);
 }
 
 void Plot3DDialog::pickTitleFont()
 {
 	bool ok;
 	QFont font = QFontDialog::getFont(&ok,titleFont,this);
-	if ( ok ) {
+	if (ok){
 		titleFont = font;
-	} else {
+		updatePlot();
+	} else
 		return;
-	}
 }
 
 void Plot3DDialog::pickNumbersFont()
@@ -671,26 +928,52 @@ void Plot3DDialog::pickNumbersFont()
 	QFont font = QFontDialog::getFont(&ok,numbersFont,this);
 	if ( ok ) {
 		numbersFont = font;
-	} else {
+		updatePlot();
+	} else
 		return;
-	}
 }
 
 void Plot3DDialog::viewAxisOptions(int axis)
 {
 	boxLabel->setText(labels[axis]);
 
-	boxMajorLength->setText(tickLengths[2*axis+0]);
-	boxMinorLength->setText(tickLengths[2*axis+1]);
+	boxMajorLength->blockSignals(true);
+	boxMajorLength->setValue(tickLengths[2*axis+0].toDouble());
+	boxMajorLength->blockSignals(false);
+
+	boxMinorLength->blockSignals(true);
+	boxMinorLength->setValue(tickLengths[2*axis+1].toDouble());
+	boxMinorLength->blockSignals(false);
+
+	axesList->setCurrentRow(axis);
 }
 
 void Plot3DDialog::viewScaleLimits(int axis)
 {
-	boxFrom->setText(scales[5*axis+0]);
-	boxTo->setText(scales[5*axis+1]);
-	boxMajors->setValue(scales[5*axis+2].toInt());
-	boxMinors->setValue(scales[5*axis+3].toInt());
-	boxType->setCurrentIndex(scales[5*axis+4].toInt());
+	Qwt3D::Axis a = d_plot->surface()->coordinates()->axes[axis];
+
+	double start, end;
+	a.limits(start, end);
+
+	boxFrom->blockSignals(true);
+	boxFrom->setValue(start);
+	boxFrom->blockSignals(false);
+
+	boxTo->blockSignals(true);
+	boxTo->setValue(end);
+	boxTo->blockSignals(false);
+
+	boxMajors->blockSignals(true);
+	boxMajors->setValue(a.majors());
+	boxMajors->blockSignals(false);
+
+	boxMinors->blockSignals(true);
+	boxMinors->setValue(a.minors());
+	boxMinors->blockSignals(false);
+
+	boxType->setCurrentIndex(d_plot->axisType(axis));
+
+	axesList2->setCurrentRow(axis);
 }
 
 void Plot3DDialog::accept()
@@ -729,10 +1012,12 @@ bool Plot3DDialog::updatePlot()
     if (!app)
         return false;
 
-	if (generalDialog->currentPage()==(QWidget*)bars){
+	if (generalDialog->currentPage() == bars){
 		d_plot->setBarRadius(boxBarsRad->text().toDouble());
+		d_plot->setBarLines(boxBarLines->isChecked());
+		d_plot->setFilledBars(boxFilledBars->isChecked());
 		d_plot->setBarStyle();
-	} else if (generalDialog->currentPage() == (QWidget*)points){
+	} else if (generalDialog->currentPage() == points){
 		if (boxPointStyle->currentIndex() == 0) {
 			d_plot->setDotOptions(boxSize->text().toDouble(), boxSmooth->isChecked());
 			d_plot->setDotStyle();
@@ -744,20 +1029,31 @@ bool Plot3DDialog::updatePlot()
 			d_plot->setConeOptions(boxConesRad->text().toDouble(), boxQuality->value());
 			d_plot->setConeStyle();
         }
-
         app->custom3DActions(d_plot);
-	} else if (generalDialog->currentPage()==(QWidget*)title){
+	} else if (generalDialog->currentPage() == title){
 		d_plot->setTitle(boxTitle->text().remove("\n"), btnTitleColor->color(), titleFont);
-	} else if (generalDialog->currentPage()==(QWidget*)colors){
+	} else if (generalDialog->currentPage() == colors){
 		d_plot->changeTransparency(boxTransparency->value()*0.01);
-		d_plot->setDataColors(btnFromColor->color(), btnToColor->color());
+		if (linearColorMapGroupBox->isChecked())
+			d_plot->setDataColorMap(d_color_map_editor->colorMap());
+		else if (colorMapFileGroupBox->isChecked() && !d_color_map_file.isEmpty()){
+			d_plot->setDataColorMap(d_color_map_file);
+			setColorMapPreview(d_color_map_file);
+		}
+
 		d_plot->setMeshColor(btnMesh->color());
 		d_plot->setAxesColor(btnAxes->color());
 		d_plot->setNumbersColor(btnNumbers->color());
 		d_plot->setLabelsColor(btnLabels->color());
 		d_plot->setBackgroundColor(btnBackground->color());
-		d_plot->setGridColor(btnGrid->color());
-	} else if (generalDialog->currentPage()==(QWidget*)general){
+	} else if (generalDialog->currentPage() == gridPage){
+		Qwt3D::GridLine majorGrid(boxMajorGrids->isChecked(), Qt2GL(btnGrid->color()), (Qwt3D::LINESTYLE)boxMajorGridStyle->currentIndex(), boxMajorGridWidth->value());
+		Qwt3D::GridLine minorGrid(boxMinorGrids->isChecked(), Qt2GL(btnGridMinor->color()), (Qwt3D::LINESTYLE)boxMinorGridStyle->currentIndex(), boxMinorGridWidth->value());
+		for (int i = 0; i < 12; i++){
+			d_plot->coordinateSystem()->setMajorGridLines((Qwt3D::AXIS)i, majorGrid);
+			d_plot->coordinateSystem()->setMinorGridLines((Qwt3D::AXIS)i, minorGrid);
+		}
+	} else if (generalDialog->currentPage() == general){
 		d_plot->showColorLegend(boxLegend->isChecked());
 		d_plot->setResolution(boxResolution->value());
 		d_plot->setMeshLineWidth(boxMeshLineWidth->value());
@@ -766,32 +1062,9 @@ bool Plot3DDialog::updatePlot()
 		d_plot->setZoom(zoom*boxZoom->value()*0.01);
 		d_plot->setScale(xScale*boxXScale->value()*0.01, yScale*boxYScale->value()*0.01, zScale*boxZScale->value()*0.01);
 	} else if (generalDialog->currentPage()==(QWidget*)scale){
-		int axis = axesList->currentRow();
-		QString from=boxFrom->text().lower();
-		QString to=boxTo->text().lower();
-		double start, end;
-		try {
-			MyParser parser;
-			parser.SetExpr(from.ascii());
-			start = parser.Eval();
-		} catch(mu::ParserError &e){
-			QMessageBox::critical(0,tr("QtiPlot - Start limit error"),  QString::fromStdString(e.GetMsg()));
-			boxFrom->setFocus();
-			return false;
-		}
-
-		try {
-			MyParser parser;
-			parser.SetExpr(to.ascii());
-			end = parser.Eval();
-		} catch(mu::ParserError &e){
-			QMessageBox::critical(0,tr("QtiPlot - End limit error"), QString::fromStdString(e.GetMsg()));
-			boxTo->setFocus();
-			return false;
-		}
-
-        d_plot->updateScale(axis, scaleOptions(axis, start, end, boxMajors->text(), boxMinors->text()));
-	} else if (generalDialog->currentPage()==(QWidget*)axes){
+		d_plot->setScale(axesList->currentRow(), boxFrom->value(), boxTo->value(), boxMajors->value(),
+						 boxMinors->value(), (Qwt3D::SCALETYPE)boxType->currentIndex());
+	} else if (generalDialog->currentPage() == axes){
 		int axis = axesList2->currentRow();
 		labels[axis] = boxLabel->text();
 
@@ -820,21 +1093,6 @@ bool Plot3DDialog::updatePlot()
 	return true;
 }
 
-QStringList Plot3DDialog::scaleOptions(int axis, double start, double end,
-		const QString& majors, const QString& minors)
-{
-	QStringList l;
-	l<<QString::number(start);
-	l<<QString::number(end);
-	l<<majors;
-	l<<minors;
-	l<<QString::number(boxType->currentIndex());
-
-	for (int i=0;i<5;i++)
-		scales[5*axis+i]=l[i];
-	return l;
-}
-
 void Plot3DDialog::pickAxisLabelFont()
 {
 	bool ok;
@@ -842,26 +1100,29 @@ void Plot3DDialog::pickAxisLabelFont()
 	switch(axesList2->currentRow())
 	{
 		case 0:
-			font= QFontDialog::getFont(&ok,xAxisFont,this);
-			if ( ok )
+			font = QFontDialog::getFont(&ok, xAxisFont, this);
+			if ( ok ){
 				xAxisFont=font;
-			else
+				updatePlot();
+			} else
 				return;
 			break;
 
 		case 1:
-			font= QFontDialog::getFont(&ok,yAxisFont,this);
-			if ( ok )
-				yAxisFont=font;
-			else
+			font = QFontDialog::getFont(&ok, yAxisFont, this);
+			if ( ok ){
+				yAxisFont = font;
+				updatePlot();
+			} else
 				return;
 			break;
 
 		case 2:
-			font= QFontDialog::getFont(&ok,zAxisFont,this);
-			if ( ok )
-				zAxisFont=font;
-			else
+			font = QFontDialog::getFont(&ok, zAxisFont, this);
+			if ( ok ){
+				zAxisFont = font;
+				updatePlot();
+			} else
 				return;
 			break;
 	}
@@ -901,3 +1162,13 @@ void Plot3DDialog::showAxisTab()
 {
 	generalDialog->setCurrentWidget (axes);
 }
+
+void Plot3DDialog::updateColorMapFileGroupBox(bool checked)
+{
+	colorMapFileGroupBox->setChecked(!checked);
+}
+
+void Plot3DDialog::updateLinearColorMapGroupBox(bool checked)
+{
+	linearColorMapGroupBox->setChecked(!checked);
+}
